import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { NbToastrService } from '@nebular/theme';
import { TranslateService } from '@ngx-translate/core';
import { map } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { DashboardPage } from '../../../model/dashboard-page.model';
import { DashboardPageCrudService } from '../services/dashboard-page-crud.service';
import { MenuType } from '../../../model/enumeration/menu-type.model';
import { DashboardCloneWizardComponent } from '../dashboard-clone-wizard/dashboard-clone-wizard.component';
import { TargetCrudService } from '../services/target-crud.service';
import { OidcUserInformationService } from '../../../auth/services/oidc-user-information.service';
import { UserCrudService } from '../services/user-crud.service';
import { IGroupRepresentation } from '../../../model/group-representation.model';
import { PublicDashboardService } from '../services/public-dashboard.service';
import { MatDialog } from '@angular/material/dialog';
import { ManageDashboardPageContentComponent } from '../manage-dashboard-page-content/manage-dashboard-page-content.component';

@Component({
  selector: 'ngx-dashboard-card-view',
  templateUrl: './dashboard-card-view.component.html',
  styleUrls: ['./dashboard-card-view.component.scss']
})
export class DashboardCardViewComponent implements OnInit {
  pages: Array<DashboardPage> = new Array<DashboardPage>();
  dashboardType: string;
  public userRole: string;

  constructor(private route: ActivatedRoute, private service: DashboardPageCrudService, private translateService: TranslateService,
    private toastrService: NbToastrService, private router: Router, private dashboardCloneService: DashboardCloneWizardComponent,
    private targetService: TargetCrudService,     private dialogService: MatDialog,
    private userService: OidcUserInformationService, private userCrudService: UserCrudService, private publicDashboardService: PublicDashboardService
    ) {
    route.params.subscribe(params => {
      if (params['dashboardType'] != undefined && params['dashboardType'] != null) {
        this.dashboardType = params['dashboardType'].split("_")[0];
      } else {
      };
    });
    /*This do the trick!*/
    router.routeReuseStrategy.shouldReuseRoute = () => false;
    this.fetchDashboardData()
    this.userService.getRole().subscribe(res=> {
      this.userRole = res[0];
    });
  }

  ngOnInit(): void {
    console.log("DASHBOARD CARD VIEW COMPONENT LOGS-------->");
  }
  fetchDashboardData() {
    if(this.dashboardType !== "PUBLIC"){
    this.service.getEditablePages().subscribe(
      (data) => {
        for(let i = 0; i< data.length; i++){
          if(data[i].type.toString() === this.dashboardType.toString()){
            this.pages.push(data[i]);
          } 
        }
        
      },
      error => (this.showError(error))
    );
      this.getSharedWithUser();
      this.getSharedWithRole();
      this.getSharedWithGroup();
    }
    this.getPublic();
  }
  openDashboard(dashboard: DashboardPage) {
    if(this.userRole !== "ANONYMOUS"){
      this.dashboardCloneService.previewDashboard(dashboard.id);
    }
    else {
      let dialogRef = this.dialogService.open(ManageDashboardPageContentComponent, {
        height: '720px',
        width: '1280px',
        data: {
          dashboardPage: dashboard
        }
      });
    }
  }

  showError(error) {
    this.toastrService.danger(error.message, this.translateService.instant('general.error'))
  }

  getSharedWithUser(){
    this.targetService.getAll().subscribe(
      (data) => {
        // console.log("Fetching targets");
        // console.log(data)

        for (let i = 0; i < data.length; i++) {
          // console.log("LOG-----> userService.user.name");
          // userService.user.name is GUIDO ROSSI
          // userService.user.preferred_username is MOBILITY MANAGER
          // console.log("LOG-----> userService.user.realm_access.roles[0]");
          // console.log(this.userService.user.realm_access.roles[0]);
          // userServ(ice.user.role IS MANAGER
          console.log(this.userService.user)
          // console.log("LOG-----> userService.user.preferred_username");
          // console.log(this.userService.user.preferred_username);
          // console.log("LOG-----> data[i].name");
          // console.log(data[i].name);
          if (data[i].createdBy !== this.userService.user.preferred_username && data[i].name === this.userService.user.preferred_username) {
            // console.log("IS EQUAL");
            this.service.getPageById(data[i].dashboardPage.id.toString()).subscribe(res => {
              if(res.type.toString()=== this.dashboardType.toString()){
                // console.log("LOG-----> getPageById----> res");
                // console.log(res);
                this.pages.push(res);
              }
            });

          }
        }
      },
      error => (this.showError(error))
    );
  }
  getSharedWithRole(){
    // console.log(this.userService.user);
    // console.log("GROUP IDM")
    this.userCrudService.getIdmGroups().subscribe(res => {console.log(res)});
    // console.log()
    this.targetService.getAll().subscribe(
      (data) => {
        for (let i = 0; i < data.length; i++) {
          if (data[i].createdBy !== this.userService.user.preferred_username && data[i].name === this.userService.user.realm_access.roles[0]) {
            // console.log("IS EQUAL");
            this.service.getPageById(data[i].dashboardPage.id.toString()).subscribe(res => {
              if(res.type.toString()=== this.dashboardType.toString()){

                this.pages.push(res);
              }
            });

          }
        }
      },
      error => (this.showError(error))
    );
  }
  getSharedWithGroup(){
    let groups: string[] = [];
    this.targetService.getAll().subscribe(
      (data) => {
        this.userCrudService.getUserGroups(this.userService.user.preferred_username)
        .subscribe(res => {
          groups = Object.keys(res);
          // console.log(groups)
          for (let i = 0; i < data.length; i++) {
            for(let j = 0; j<groups.length; j++){
              if (data[i].createdBy !== this.userService.user.preferred_username && data[i].name == groups[j]) {
                this.service.getPageById(data[i].dashboardPage.id.toString()).subscribe(res => {
                  if(res.type.toString()=== this.dashboardType.toString()){
                    this.pages.push(res);
                  }
                });
              }
            }
  
          }
        });
      },
      error => (this.showError(error))
    );
  }


  getPublic(){
    this.publicDashboardService.getAllPublicDashboard().subscribe(
      (data) => {
        console.log("Fetching PUBLIC DASHBOARDS");
        console.log(data)
        for(let i = 0; i < data.length; i++)
        if(data[i].type.toString() === this.dashboardType.toString()){
          this.pages = data;
        }
        // for (let i = 0; i < data.length; i++) {
        
        // //   console.log("LOG-----> userService.user.name");
        // //   console.log(this.userService.user.preferred_username);
        // //   console.log("LOG-----> data[i].name");
        // //   console.log(data[i].name);
        // //     console.log("IS EQUAL");
        //     this.service.getPageById(data[i].id.toString()).subscribe(res => {
        //       if(res.type.toString()=== this.dashboardType.toString()){
        //         console.log("LOG-----> getPageById----> res");
        //         console.log(res);
        //         this.pages.push(res);
        //       }
        //     });
        // }
      },
      error => (this.showError(error))
    );
  }
}