import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';
import { NbToastrService } from '@nebular/theme';
import { TranslateService } from '@ngx-translate/core';
import { ICloneDashboard } from '../../../model/clone-dashboard.model';
import { IDashboardPage } from '../../../model/dashboard-page.model';
import { CloningOptionType } from '../../../model/enumeration/cloning-option-type.model';
import { MenuType } from '../../../model/enumeration/menu-type.model';
import { ITarget } from '../../../model/target.model';
import { ManageDashboardPageContentComponent } from '../manage-dashboard-page-content/manage-dashboard-page-content.component';
import { DashboardPageCrudService } from '../services/dashboard-page-crud.service';
import { of as observableOf,  Observable } from 'rxjs';
import { geoConicConformalRaw } from 'd3';
@Component({
  selector: 'ngx-dashboard-clone-wizard',
  templateUrl: './dashboard-clone-wizard.component.html',
  styleUrls: ['./dashboard-clone-wizard.component.scss']
})
export class DashboardCloneWizardComponent implements OnInit {
  selectedDashboardFormGroup: FormGroup;
  createdDashboardFormGroup: FormGroup;
  selectedDashboard: IDashboardPage = {};
  createdDashboard: ICloneDashboard = {}; 
  createdDashboardCloneTargets: ITarget;
  selectedDashboardFromRoute: IDashboardPage = {};
  private foundDashboard: IDashboardPage;

  fullListEditableDashboards: Array<IDashboardPage> = [];
  fullListDashboardPageShared: Array<IDashboardPage> = [];
  fullListDashboardPagePersonal: Array<IDashboardPage> = [];
  fullListDashboardPagePublic: Array<IDashboardPage> = [];

  public parsedId =  0; 

  private CLONED_SUFFIX = " (cloned)";

  constructor(
    private readonly formBuilder: FormBuilder,
    private dashboardPageCrudService: DashboardPageCrudService,
    private toastrService: NbToastrService,
    private translateService: TranslateService,
    private router: Router,
    private route: ActivatedRoute,
    private dialogService: MatDialog
  ) { }

  ngOnInit(): void {
    this.selectedDashboardFormGroup = this.formBuilder.group({
      selectedDashboard: [this.selectedDashboard, Validators.nullValidator]
    });

    this.createdDashboardFormGroup = this.formBuilder.group({
      createdDashboardName: [this.createdDashboard.dashboardName, Validators.required],
      createdDashboardCloneTargets: [this.createdDashboardCloneTargets, ''],
    });

    this.dashboardPageCrudService.getClonablePages().subscribe(
      (res) => {
      this.fullListEditableDashboards = res;
      this.fullListDashboardPageShared = this.fullListEditableDashboards.filter(x => x.type == MenuType.SHARED);
      this.fullListDashboardPagePersonal = this.fullListEditableDashboards.filter(x => x.type == MenuType.PRIVATE);
      this.fullListDashboardPagePublic = this.fullListEditableDashboards.filter(x => x.type == MenuType.PUBLIC);

      // this.route.queryParams.subscribe(params => {
      //   if (params['parsedId'] != undefined && params['parsedId'] != null) {
      //     this.parsedId = params['parsedId'];
      //     this.foundDashboard = this.fullListEditableDashboards.find(x => x.id == this.parsedId);
      //     this.selectedDashboardFormGroup.patchValue( { selectedDashboard: this.foundDashboard });  
      //   }
      // });
    },
      error => (this.showError(error))
    );
  }

  getSharedStatus(){
    return this.selectedDashboardFormGroup.value.selectedDashboard?.type == 'SHARED' ? false : false;
  }

  isEmptyObject(object: any){
    return !!Object.keys(object).length;
  }

  onSubmitFirst(selected : IDashboardPage){
    this.selectedDashboard = selected;
    // this.createdDashboardFormGroup.patchValue( { createdDashboardName: (this.selectedDashboard.name + this.CLONED_SUFFIX) });  
  }

  onSubmitSecond(name: any , targets: any){
    this.createdDashboard.dashboardName = name;
    if (targets !== undefined && targets !== null){
      this.createdDashboardCloneTargets = targets;
    }
  }

  /** Invoke API to clone the Dashboard */
  executeClone() {
    //console.log("## selectedDashboard.id: "+ this.selectedDashboard.id + "  selectedDashboard:" + this.selectedDashboard);
    if (this.selectedDashboard.id == null || this.selectedDashboard.id == undefined){
      this.showError(this.translateService.instant('dashboardClone.clone_missing_parameter'));
    }
    this.createdDashboard.id = this.selectedDashboard.id;
    if (this.createdDashboard){
      this.createdDashboard.option = CloningOptionType.TARGETS;
    }
    return this.dashboardPageCrudService.duplicatePageWithParams(this.createdDashboard);
  }

  showSuccess() {
    this.toastrService.success(this.translateService.instant('dashboardPage.succesfully_cloned'), this.translateService.instant('Success'));
  }

  showError(error) {
    this.toastrService.danger(error.message, this.translateService.instant('general.error'))
  }

  previewDashboard(id: number) {
    if(this.selectedDashboard.id === undefined){
      if (id !== null && id !== undefined) {
        let dialogRef = this.dialogService.open(ManageDashboardPageContentComponent, {
          height: '720px',
          width: '1280px',
          data: {
            parsedId: id,
            parsedEditMode: false,
          }
        });
      }
    } else{
      /** Original code */
      if (id !== null && id !== undefined) {
        let dialogRef = this.dialogService.open(ManageDashboardPageContentComponent, {
          height: '720px',
          width: '1280px',
          data: {
            parsedId: this.selectedDashboard.id,
            parsedEditMode: false,
          }
        });
      }
    }

  }

  cloneFromExternal(selected : IDashboardPage){
    this.onSubmitFirst(selected);
    this.onSubmitSecond(selected.name + "_CLONED", CloningOptionType.ALL);
    return this.executeClone();
  }

}
