import { Component, OnInit, Input } from "@angular/core";
import { AngularEditorConfig } from "@kolkov/angular-editor";
import { NbDialogRef, NbToastrService } from "@nebular/theme";
import { TranslateService } from "@ngx-translate/core";
import { GridsterItem } from "angular-gridster2";
import { ImageCroppedEvent } from "ngx-image-cropper";
import { Datalet } from "../../data-catalogue/model/datalet";
import { DataCatalogueAdminApiService } from "../../data-catalogue/services/data-catalogue-admin-api.service";

export interface DialogData {
  componentType: string;
  name: string;
}

@Component({
  selector: 'ngx-dashboard-component-dialog',
  templateUrl: './dashboard-component-dialog.component.html',
  styleUrls: ['./dashboard-component-dialog.component.scss']
})
export class DashboardComponentDialogComponent implements OnInit {
  @Input() selectedItemType: string;
  @Input() items: GridsterItem[];
  @Input() item: GridsterItem;

  itemIndex;
  datalets: Array<Datalet>;
  selectedItem: Datalet;
  dataletObject: Datalet = {};
  imageChangedEvent: any = '';
  imageFile: any = '';
  croppedImage: any = '';
  imageBackup;
  imageUrlBackup;
  selectedOption: string;
  maintainAspectRatioVariable: boolean = false;
  aspectRatioVariable: string = "";
  statisticsIcons: Array<string>;

  editorConfig: AngularEditorConfig = {
    editable: true,
    spellcheck: true,
    height: 'auto',
    minHeight: '0',
    maxHeight: 'auto',
    width: 'auto',
    minWidth: '0',
    translate: 'yes',
    enableToolbar: true,
    showToolbar: true,
    placeholder: 'Enter text here...',
    defaultParagraphSeparator: '',
    defaultFontName: '',
    defaultFontSize: '',
    fonts: [
      { class: 'arial', name: 'Arial' },
      { class: 'times-new-roman', name: 'Times New Roman' },
      { class: 'calibri', name: 'Calibri' },
      { class: 'comic-sans-ms', name: 'Comic Sans MS' }
    ],
    customClasses: [
      {
        name: 'quote',
        class: 'quote',
      },
      {
        name: 'redText',
        class: 'redText'
      },
      {
        name: 'titleText',
        class: 'titleText',
        tag: 'h1',
      },
    ],
    uploadUrl: 'v1/image',
    // upload: (file: File) => { ... }
    uploadWithCredentials: false,
    sanitize: false,
    toolbarPosition: 'top',
    toolbarHiddenButtons: [
      [
        'fontName',
        // 'undo',
        // 'redo',
        // 'bold',
        // 'italic',
        // 'underline',
        // 'strikeThrough',
        // 'subscript',
        // 'superscript',
        // 'justifyLeft',
        // 'justifyCenter',
        // 'justifyRight',
        // 'justifyFull',
        // 'indent',
        // 'outdent',
        // 'insertUnorderedList',
        // 'insertOrderedList',
        // 'heading',
      ],
      [
        'customClasses',
        'insertImage',
        'insertVideo',
        'removeFormat',
        'toggleEditorMode',
        // 'fontSize',
        // 'textColor',
        // 'backgroundColor',
        // 'link',
        // 'unlink',
        // 'insertHorizontalRule',
      ]
    ]
  };

  constructor(
    public dialogRef: NbDialogRef<DashboardComponentDialogComponent>,
    private dataCatalogueAdminApiservice: DataCatalogueAdminApiService,
    private toastrService: NbToastrService,
    private translateService: TranslateService
    ) {
  }

  ngOnInit() {
    this.itemIndex = this.items.indexOf(this.item);
    this.selectedItemType = this.item.type;
    this.statisticsIcons = this.getEvaIcon();

    this.dataCatalogueAdminApiservice.getAllDatalets().subscribe(
      (response: Array<Datalet>) => {
        if (response !== undefined  && response !== null) {
          try{
            this.datalets = response;
            if (this.selectedItemType == 'CHART'){
              this.dataletObject = this.datalets.find(x => x.datalet_html === this.item.chart);
              // console.log(this.dataletObject);
            }
          } catch(e) {
            console.log("# Error in Dashboard page content deserialization due to " + e);
            this.toastrService.danger(this.translateService.instant('dashboardComponent.catch_error'));
          }
        }
    },  
    (error: any) => {
      // console.log("# Data Catalogue Error => "+ error);
      this.toastrService.danger(error.message, this.translateService.instant('general.datacatalogue_error'))
    }
    );

    //Radio Button for Image upload
    this.selectedOption = (this.item.url == true) ? 'upload-url' : 'upload-file';
  }

  onCancelClick(): void {
    if (this.imageBackup !== undefined) {
      this.item.image = this.imageBackup;
    }
    if (this.imageUrlBackup !== undefined) {
      this.item.imageUrl = this.imageUrlBackup;
    }
    this.dialogRef.close();
  }

  onSaveClick( item, selectedItemType ): void {
    item.type = selectedItemType;

    if (selectedItemType == 'CHART') {
      // console.log(this.dataletObject);
      item.chart = this.dataletObject.datalet_html;
      item.chartTitle = this.dataletObject.title;
    
    } else if (selectedItemType == 'IMAGE') {
      if (this.croppedImage !== undefined && this.croppedImage !== null && (this.croppedImage.length > 3 || this.croppedImage.length == 0)){
        if (this.croppedImage.length > 3){
          item.image = this.croppedImage;
        }
      }
      if (this.selectedOption=='upload-url'){
        item.url = true;
        item.image = null;
      } else{
        item.url = false;
        item.imageUrl = null;
      }

    } else if (selectedItemType == 'MAP'){
      // console.log("# Here saved map: ", item);
    }
    
    this.items[this.itemIndex] = item;
    this.dialogRef.close(this.items);
  }

  // Move into a property file
  getEvaIcon() {
    return [
            'award-outline',
            'at-outline',
            'car-outline',
            'checkmark-circle-2-outline',
            'cloud-download-outline',
            'color-palette-outline',
            'email-outline', 
            'external-link-outline',
            'facebook-outline',
            'flag-outline', 
            'globe-outline', 
            'github-outline',
            'google-outline',
            'heart-outline',
            'linkedin-outline',
            'monitor-outline', 
            'message-square-outline',
            'percent-outline',
            'pie-chart-outline',
            'person-outline', 
            'pin-outline', 
            'sun-outline',
            'unlock-outline',
    ];
  }

  fileChangeEvent(event: any): void {
    this.imageChangedEvent = event;
  }

  imageCropped(event: ImageCroppedEvent) {
    this.croppedImage = event.base64;
  }

  imageLoaded() {
    this.imageBackup = this.item.image;
    this.imageUrlBackup = this.item.imageUrl;
    this.item.image = undefined; 
    this.item.imageUrl = undefined;
  }

  getHiddenUrlBlock(){
    return (this.selectedOption!=='upload-url');
  }

  getHiddenFileUploadBlock(){
    return (this.selectedOption!=='upload-file');
  }

}
