import { DatePipe } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { NbDialogService, NbToastrService } from '@nebular/theme';
import { TranslateService } from '@ngx-translate/core';
import { FORMERR } from 'dns';
import { LocalDataSource } from 'ng2-smart-table';
import { OidcUserInformationService } from '../../../auth/services/oidc-user-information.service';
import { DashboardPage } from '../../../model/dashboard-page.model';
import { MenuType } from '../../../model/enumeration/menu-type.model';
import { DialogComponent } from '../../shared/dialog/dialog.component';
import { TableActionsComponent } from '../../shared/table-actions/table-actions.component';
import { DashboardCloneWizardComponent } from '../dashboard-clone-wizard/dashboard-clone-wizard.component';
import { DashboardComponentDialogComponent } from '../dashboard-component-dialog/dashboard-component-dialog.component';
import { DashboardGuideComponent } from '../dashboard-guide/dashboard-guide.component';
import { DashboardInfoComponent } from '../dashboard-info/dashboard-info.component';
import { DashboardSharingComponent } from '../dashboard-sharing/dashboard-sharing.component';
import { DashboardPageCrudService } from '../services/dashboard-page-crud.service';
import { MenuBlockCrudService } from '../services/menu-block-crud.service';
import { PublicDashboardService } from '../services/public-dashboard.service';
import { TargetCrudService } from '../services/target-crud.service';
import { UserCrudService } from '../services/user-crud.service';

@Component({
  selector: 'ngx-manage-dashboard-page',
  templateUrl: './manage-dashboard-page.component.html',
  styleUrls: ['./manage-dashboard-page.component.scss']
})
export class ManageDashboardPageComponent implements OnInit {
  source: LocalDataSource;
  settings: any;
  pages: Array<DashboardPage> = [];

  constructor(private service: DashboardPageCrudService,
    private menuBlockService: MenuBlockCrudService,
    private router: Router,
    private datePipe: DatePipe,
    private dialogService: NbDialogService,
    private translateService: TranslateService,
    private toastrService: NbToastrService,
    private targetService: TargetCrudService,
    private userService: OidcUserInformationService,
    private dashboardCloneService: DashboardCloneWizardComponent, private userCrudService: UserCrudService,
    private publicDashboardService: PublicDashboardService
  ) {
    this.initTableSettings();
  }

  ngOnInit(): void {
    this.source = new LocalDataSource();
    this.fetchTableData();

  }

  fetchTableData() {
    this.service.getEditablePages().subscribe(
      (data) => {
        this.pages = data;
        this.source.load(data);
        console.log("Fetching editable pages");
        console.log(data)
      },
      error => (this.showError(error))
    );
    this.targetService.getAll().subscribe(
      (data) => {
        console.log("Fetching targets");
        console.log(data)
        for (let i = 0; i < data.length; i++) {
          // console.log("LOG-----> userService.user.name");
          // console.log(this.userService.user.preferred_username);
          // console.log("LOG-----> data[i].name");
          // console.log(data[i].name);
          if (data[i].createdBy !== this.userService.user.preferred_username && data[i].name === this.userService.user.preferred_username) {
            console.log("IS EQUAL");
            this.service.getPageById(data[i].dashboardPage.id.toString()).subscribe(res => {

              console.log("LOG-----> getPageById----> res");
              console.log(res);
              this.pages.push(res);
              this.source.empty();
              this.source.load(this.pages);
            });

          }
        }
      },
      error => (this.showError(error))
    );
    this.getSharedWithRole();
    this.getSharedWithGroup();
    this.getPublic();
  }

  getSharedWithRole() {
    this.targetService.getAll().subscribe(
      (data) => {
        for (let i = 0; i < data.length; i++) {
          if (data[i].createdBy !== this.userService.user.preferred_username && data[i].name === this.userService.user.realm_access.roles[0]) {
            // console.log("IS EQUAL");
            this.service.getPageById(data[i].dashboardPage.id.toString()).subscribe(res => {
              this.pages.push(res);
              this.source.empty();
              this.source.load(this.pages);
            });

          }
        }
      },
      error => (this.showError(error))
    );
  }

  getSharedWithGroup() {
    let groups: string[] = [];
    this.targetService.getAll().subscribe(
      (data) => {
        this.userCrudService.getUserGroups(this.userService.user.preferred_username)
          .subscribe(res => {
            groups = Object.keys(res);
            console.log(groups)
            for (let i = 0; i < data.length; i++) {
              for (let j = 0; j < groups.length; j++) {
                if (data[i].createdBy !== this.userService.user.preferred_username && data[i].name == groups[j]) {
                  this.service.getPageById(data[i].dashboardPage.id.toString()).subscribe(res => {
                    this.pages.push(res);
                    this.source.empty();
                    this.source.load(this.pages);
                  });
                }
              }

            }
          });
      },
      error => (this.showError(error))
    );
  }

  getPublic() {
    this.publicDashboardService.getAllPublicDashboard().subscribe(
      (data) => {
        // console.log("Fetching PUBLIC DASHBOARDS");
        // console.log(data)
        for (let i = 0; i < data.length; i++) {
          //   console.log("LOG-----> userService.user.name");
          //   console.log(this.userService.user.preferred_username);
          //   console.log("LOG-----> data[i].name");
          //   console.log(data[i].name);
          //     console.log("IS EQUAL");
          this.service.getPageById(data[i].id.toString()).subscribe(res => {
            // console.log("LOG-----> getPageById----> res");
            if(res.pageOwner !== null){
              this.pages.push(res);
              this.source.empty();
              this.source.load(this.pages);
            }
          });
        }
      },
      error => (this.showError(error))
    );
  }
  showSuccess() {
    this.toastrService.success(this.translateService.instant('dashboardPage.deleted'), this.translateService.instant('general.success'));
  }

  showError(error) {
    this.toastrService.danger(error.message, this.translateService.instant('general.error'))
  }

  initTableSettings(): void {
    let sharedFilter = [];
    let booleans = [true, false]
    for (let myShared of booleans) {
      sharedFilter.push({ value: myShared, title: myShared })
    }

    let menuTypeFilter = [];
    for (let mType of MenuType.values()) {
      menuTypeFilter.push({ value: mType.enumVal, title: this.translateService.instant("MenuType." + mType.enumVal) })
    }


    this.settings = {
      mode: 'external',
      columns: {
        shared: {
          title: this.translateService.instant('dashboardPage.status'),
          valuePrepareFunction: (cell, row) => {
            if (row.shared) {
              return this.translateService.instant('dashboardPage.published');
            }
            return this.translateService.instant('dashboardPage.draft');
          },
          // filter: {
          //   type: 'list',
          //   config: {
          //     selectText: '',
          //     list: sharedFilter
          //   },
          // }
        },
        type: {
          title: this.translateService.instant('dashboardPage.type'),
          filter: {
            type: 'list',
            config: {
              selectText: '',
              list: menuTypeFilter
            },
          }
        },
        name: {
          title: this.translateService.instant('dashboardPage.name')
        },
        description: {
          title: this.translateService.instant('dashboardPage.description')
        },
        note: {
          title: this.translateService.instant('dashboardPage.note')
        },
        createdBy: {
          title: this.translateService.instant('dashboardPage.createdBy')
        },
        createdDate: {
          title: this.translateService.instant('dashboardPage.createdDate'),
          valuePrepareFunction: (cell, row) => {
            return this.datePipe.transform(cell);
          }
        },

        /*content: {
          title: this.translateService.instant('dashboardPage.content'),
          filter: false
        },*/

        actions: {
          title: this.translateService.instant('general.actions'),
          filter: false,
          sort: false,
          type: 'custom',
          valuePrepareFunction: (cell, row) => {
            return [
              { action: 'edit', enabled: row.pageOwner, status: "info", icon: "edit-2", tooltip: this.translateService.instant('general.edit') },
              { action: 'delete', enabled: row.pageOwner, status: "danger", icon: "trash-2", tooltip: this.translateService.instant('general.delete') },
              { action: 'clone', enabled: true, status: "info", icon: "copy", tooltip: this.translateService.instant('dashboardClone.action_clone') },
              { action: 'preview', enabled: true, status: "info", icon: "eye-outline", tooltip: this.translateService.instant('dashboardClone.action_preview') },
              { action: 'edit_sharing', enabled: row.pageOwner, status: "info", icon: "share-outline", tooltip: this.translateService.instant('dashboardPage.edit_sharing') },
              { action: 'editDashboard', enabled: row.pageOwner, status: "info", icon: "edit", tooltip: this.translateService.instant('general.editContent') }

            ];
          },
          renderComponent: TableActionsComponent,
          onComponentInitFunction: (instance) => {
            instance.onCustom.subscribe(value => {
              this.onCustom(value, instance.rowData);
            });
          },
        }
      },

      actions: {
        add: false,
        edit: false,
        delete: false,
        clone: false,
        edit_sharing: false
      },

    };
  }

  onCustom(event, data) {
    if (event == 'edit') {
      const dialogRef = this.dialogService
        .open(DashboardInfoComponent,
          { context: { pageId: data.id } })
        .onClose.subscribe((res) => {
          console.log("RES SHOULD BE TRUE")
          console.log(res)
          this.fetchTableData();
        }, err => {
          // console.log(err);
        });
    }

    if (event == 'editDashboard') {
      this.router.navigate(['/pages/dashboard-management/edit-dashboard-page-content/' + data.id])
    }

    if (event == 'preview') {
      this.dashboardCloneService.previewDashboard(data.id);
      console.log("IN PREVIEW");
      console.log(data.id);
    }

    if (event == 'clone') {
      this.dashboardCloneService.cloneFromExternal(data).subscribe((res) => {
        console.log('## Dashboard has been succesfully cloned');
        //this.router.navigate(['/pages/dashboard-management/manage-dashboard-pages']);
        this.showSuccess();
        this.fetchTableData();
      }, err => {
        console.log("# Error during cloning due to e: " + err);
        this.showError(err);
      });
    }

    if (event == 'delete') {

      this.dialogService.open(DialogComponent, { context: { title: this.translateService.instant('general.delete') + " " + this.translateService.instant('general.dashboard'), message: this.translateService.instant('general.deleteConfirm') } }).onClose.subscribe(res => {
        if (res) {
          //this.service.deletePageById(data.id).subscribe(() => {
          this.service.deleteDeepPageById(data.id).subscribe(() => {
            this.fetchTableData();
            console.log(`Dashboard Page with id=${data.id} has been deleted.`),
              this.showSuccess();
          },
            error => (this.showError(error))
          );
        } else return;

      });
    }

    if (event == 'edit_sharing') {
      const dialogRef = this.dialogService
        .open(DashboardSharingComponent, { context: { pageId: data.id } })
        .onClose.subscribe((res) => {
          console.log("RES SHOULD BE TRUE")
          console.log(res)
          this.fetchTableData();
        }, err => {
          // console.log(err);
        });
    }
  }

  openGuide() {
    const dialogRef = this.dialogService
      .open(DashboardGuideComponent,
        { context: { info: "manage dashboards" } });
  }

}
