import { Component, Input, OnInit } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { NbToastrService } from '@nebular/theme';
import { TranslateService } from '@ngx-translate/core';
import { DashboardPage } from '../../../model/dashboard-page.model';
import { DashboardPageCrudService } from '../services/dashboard-page-crud.service';
import { Location } from '@angular/common';
import { FormManagerService } from '../services/form-manager.service';
import { MenuType } from '../../../model/enumeration/menu-type.model';
import { MenuItemCrudService } from '../services/menu-item-crud.service';


@Component({
  selector: 'ngx-manage-menu-blocks',
  templateUrl: './manage-menu-blocks.component.html',
  styleUrls: ['./manage-menu-blocks.component.scss']
})
export class ManageMenuBlocksComponent implements OnInit {
  @Input() page: FormGroup = new FormGroup({});
  sharedDomain = [true, false];
  sharedType = MenuType;
  sharedTypes():Array<string>{
    const keys = Object.keys(this.sharedType);
    return keys;
  }
  
  pageId: string; //For Targets
  pageName: string; //For MenuItem
  disabledTarget: boolean; //For Targets
 

  constructor( 
    private formService: FormManagerService,
    private service: DashboardPageCrudService,
    private menuItemService: MenuItemCrudService,
    private router: Router,
    private route: ActivatedRoute,
    private toastrService: NbToastrService,
    private translateService: TranslateService,
    private _location: Location
  ) { }

  ngOnInit(): void {
    this.page = this.formService.pageFormCreator(new DashboardPage());
    
    this.route.paramMap.subscribe(params => {
      this.pageId = params.get('id');
      if (this.pageId) {
        this.service.getPageById(this.pageId).subscribe(
          (event: DashboardPage) => { this.editPage(event); 
          },
          error => this.showError(error)
        );
      }
    });

  }

  editPage(page: DashboardPage) {
    this.page = this.formService.pageFormCreator(page);
    this.pageName = page.name;
    
    //this.disabledTarget = page.type!=undefined ? page.type != MenuType.SHARED : true;   
    this.menuItemService.getByPageId(this.pageId).subscribe(
        (data) => { this.disabledTarget = data!=null && data.length>0 ? data[0].type != MenuType.SHARED : true; }, 
        error => (this.showError(error)) 
      );

  }

  storePage(): void {
    if (this.page.value.id == null) {
      console.log('## CREATE DASHBOARD WITH ID:' + this.page.value.id);
      let tmp = <DashboardPage>this.page.getRawValue();
      this.pageName = tmp.name;
      //this.disabledTarget = tmp.type!=undefined ? tmp.type != MenuType.SHARED : true;
      this.service.createPage(tmp).subscribe(
        (data: DashboardPage) => {
          // console.log("# DASHBOARD CREATED,  DATA => " + data);
          data.shared= false;
          console.log(data)
          this.router.navigate(['/pages/dashboard-management/manage-dashboard-pages']);
          this.showSuccess();
        },
        error => (this.showError(error))
      );
      
    }
    else {
      console.log('# MODIFY DASHBOARD WITH ID:' + this.page.value.id);
      let tmp = <DashboardPage>this.page.getRawValue();
      this.pageName = tmp.name;
      //this.disabledTarget = tmp.type!=undefined ? tmp.type != MenuType.SHARED : true;
      this.service.updatePageById(tmp).subscribe(
        (data: DashboardPage) => {
          this.editPage(data);
          this.showSuccess();
        },
        error => (this.showError(error))
      );
      
    }
  }

  showSuccess() {
    this.page.value.id == null
      ? this.toastrService.success(this.translateService.instant('dashboardPage.created'), this.translateService.instant('Success'))
      : this.toastrService.success(this.translateService.instant('dashboardPage.updated'), this.translateService.instant('Success'));
  }

  showError(error) {
    this.toastrService.danger(error.error.message, this.translateService.instant('general.error'))
  }

  
  
  updateSharedPersonalTypology($event) {    
    this.menuItemService.getByPageId(this.pageId).subscribe(
      (data) => {
      this.disabledTarget = data!=null && data.length>0 ? data[0].type != MenuType.SHARED : true;
      // console.log("# Broadcast event updateSharedPersonalTypology disabledTarget => " + this.disabledTarget);
      if (this.disabledTarget){
        this.page.get("shared").disable();
      }else{
        this.page.get("shared").enable();  
      }
    },
    error => (this.showError(error))
    );
  }

  getMessageDisabledTarget(){
    return this.disabledTarget 
    ? this.translateService.instant('dashboardPage.not_mandatory_field')
    : this.translateService.instant('general.mandatory_field');
  }

}
