import { Component, Input, OnInit } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { NbToastrService } from '@nebular/theme';
import { TranslateService } from '@ngx-translate/core';
import { MenuBlock } from '../../../model/menu-block.model';
import { MenuBlockCrudService } from '../services/menu-block-crud.service';
import { Location } from '@angular/common';
import { MenuFormManagerService } from '../services/menu-form-manager.service';
import { MenuType } from '../../../model/enumeration/menu-type.model';

@Component({
  selector: 'ngx-menu-blocks-page',
  templateUrl: './menu-blocks-page.component.html',
  styleUrls: ['./menu-blocks-page.component.scss']
})
export class MenuBlocksPageComponent implements OnInit {

  @Input() block: FormGroup = new FormGroup({});
  typeDomain = MenuType.values(); //Fix
  blockId: string; //For targets

  constructor(
    private formService: MenuFormManagerService,
    private service: MenuBlockCrudService,
    private router: Router,
    private route: ActivatedRoute,
    private toastrService: NbToastrService,
    private translateService: TranslateService,
    private _location: Location
  ) { }

  ngOnInit(): void {
    this.block = this.formService.pageFormCreator(new MenuBlock());
    this.route.paramMap.subscribe(params => {
      this.blockId = params.get('id');
      if (this.blockId) {
        this.service.getById(this.blockId).subscribe(
          (event: MenuBlock) =>{ this.editBlock(event); },
          (err: any) => console.log(err)
        );;
      }
    });
  }

  editBlock(menuBlock: MenuBlock) {
    this.block = this.formService.pageFormCreator(menuBlock);
  }

  storeBlock(): void {
    if (this.block.value.id == null) {
      //console.log('## CREATE MENU-BLOCK WITH ID:' + this.block.value.id);
      let objToAdd = <MenuBlock> this.block.getRawValue();
      objToAdd.code = objToAdd.label!=null ? objToAdd.label.replace(/\s/g, "_").toUpperCase() : objToAdd.label;
      //console.log('objToAdd =>  ' , JSON.stringify(objToAdd));

      this.service.create(objToAdd).subscribe(
        (data: MenuBlock) => {
          console.log("Menu block created, data => " + data);
          console.log(data);
          this.router.navigate(['/pages/dashboard-management/manage-menu-blocks']);
          this.showSuccess();
        },
        error => (this.showError(error))
      );
      
    }
    else {
      //console.log('## EDIT MENU-BLOCK WITH ID:' + this.block.value.id);
      let objToModify = <MenuBlock>this.block.getRawValue()
      objToModify.code = objToModify.label!=null ? objToModify.label.replace(/\s/g, "_").toUpperCase() : objToModify.label;
      this.service.update(objToModify).subscribe(
        (data: MenuBlock) => {
          this.editBlock(data);
          this.showSuccess();
        },
        error => (this.showError(error))
      );
    }
  }

  showSuccess() {
    this.block.value.id == null ?
      this.toastrService.success(this.translateService.instant('menuBlock.created'), this.translateService.instant('Success')) :
      this.toastrService.success(this.translateService.instant('menuBlock.updated'), this.translateService.instant('Success'));
  }

  showError(error) {
    this.toastrService.danger(error.error.message, this.translateService.instant('general.error'))
  }

  backClicked() {
    this._location.back();
  }
  
}


