import { Injectable } from '@angular/core';
import { ConfigService } from '@ngx-config/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Observable, of } from 'rxjs';
import { IDashboardPage } from '../../../model/dashboard-page.model';
import { ICloneDashboard } from '../../../model/clone-dashboard.model';
import AMTram2020 from '../../../../assets/map/am_tram_2020.json';
import { CloningOptionType } from '../../../model/enumeration/cloning-option-type.model';

@Injectable({
  providedIn: 'root'
})
export class DashboardPageCrudService {
  private apiURL: any;

  constructor(configService: ConfigService, private http: HttpClient) {
    this.apiURL = configService.getSettings("dashboard-controller.api_base_url");
  }

  getAllPages(): Observable<Array<IDashboardPage>> {
    return this.http.get<Array<IDashboardPage>>(`${this.apiURL}/api/all-dashboard-pages`);
  }

  getSharedPages(): Observable<Array<IDashboardPage>> {
    return this.http.get<Array<IDashboardPage>>(`${this.apiURL}/api/shared-dashboard-pages`);
  }

  getPersonalPages(): Observable<Array<IDashboardPage>> {
    return this.http.get<Array<IDashboardPage>>(`${this.apiURL}/api/personal-dashboard-pages`);
  }

  getEditablePages(): Observable<Array<IDashboardPage>> {
    return this.http.get<Array<IDashboardPage>>(`${this.apiURL}/api/editable-dashboard-pages`);
  }

  getClonablePages(): Observable<Array<IDashboardPage>> {
    return this.http.get<Array<IDashboardPage>>(`${this.apiURL}/api/clonable-dashboard-pages`);
  }

  getPageById(id: string): Observable<IDashboardPage> {
    return this.http.get<IDashboardPage>(`${this.apiURL}/api/dashboard-pages/${id}`);
  }

  createPage(page: IDashboardPage): Observable<IDashboardPage> {
    let httpHeaders = new HttpHeaders({ 'Content-type': 'application/json' });
    return this.http.post<IDashboardPage>(`${this.apiURL}/api/dashboard-pages`, page, { headers: httpHeaders });
  }

  deletePageById(id: string): any {
    return this.http.delete<IDashboardPage>(`${this.apiURL}/api/dashboard-pages/${id}`);
  }

  deleteDeepPageById(id: string): any {
    return this.http.delete<IDashboardPage>(`${this.apiURL}/api/dashboard-pages-deep/${id}`);
  }

  updatePageById(page: IDashboardPage): Observable<IDashboardPage> {
    let httpHeaders = new HttpHeaders({ 'Content-type': 'application/json' });
    return this.http.put<IDashboardPage>(`${this.apiURL}/api/dashboard-pages/${page.id}`, page, {
      headers: httpHeaders,
    });
  }

  duplicatePage(id: string): Observable<IDashboardPage> {
    return this.http.get<IDashboardPage>(`${this.apiURL}/api/duplicate-dashboard-pages/${id}`);
  }

  duplicatePageWithParams(cloneDashboard: ICloneDashboard): Observable<IDashboardPage> {
    let httpHeaders = new HttpHeaders({ 'Content-type': 'application/json' });
    return this.http.post<IDashboardPage>(`${this.apiURL}/api/duplicate-by-params-dashboard-pages`, cloneDashboard, { headers: httpHeaders });
  }

  getGeoJsonMap(url: string): Observable<any> {
    //Mocked mode A => http://localhost:4200/assets/map/am_tram_2018.json
    //Mocked mode B => http://localhost:4200/assets/map/2020-March-heatgeo-5minutes.json
    if (url == undefined || url == null)
      return of(AMTram2020);
    else 
      return this.http.get<any>(`${url}`);
  }

}
