import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ConfigService } from '@ngx-config/core';
import { Observable } from 'rxjs';
import { IMenuBlock } from '../../../model/menu-block.model';

@Injectable({
  providedIn: 'root'
})
export class MenuBlockCrudService {
  private apiURL: any;

  constructor(configService: ConfigService, private http: HttpClient) {
    this.apiURL = configService.getSettings("dashboard-controller.api_base_url");
  }

  getAll(): Observable<Array<IMenuBlock>> {
    return this.http.get<Array<IMenuBlock>>(`${this.apiURL}/api/menu-blocks`);
  }

  getAllShared(): Observable<Array<IMenuBlock>> {
    return this.http.get<Array<IMenuBlock>>(`${this.apiURL}/api/shared-menu-blocks`);
  }

  getAllPublic(): Observable<Array<IMenuBlock>> {
    return this.http.get<Array<IMenuBlock>>(`${this.apiURL}/api/public-menu-blocks`);
  }

  getAllPersonal(): Observable<Array<IMenuBlock>> {
    return this.http.get<Array<IMenuBlock>>(`${this.apiURL}/api/personal-menu-blocks`);
  }

  getAllLinked(): Observable<Array<IMenuBlock>> {
    return this.http.get<Array<IMenuBlock>>(`${this.apiURL}/api/linked-menu-blocks`);
  }

  getAllEditable(): Observable<Array<IMenuBlock>> {
    return this.http.get<Array<IMenuBlock>>(`${this.apiURL}/api/editable-menu-blocks`);
  }

  getById(id: string): Observable<IMenuBlock> {
    return this.http.get<IMenuBlock>(`${this.apiURL}/api/menu-blocks/${id}`);
  }

  create(menuBlock: IMenuBlock): Observable<IMenuBlock> {
    let httpHeaders = new HttpHeaders({ 'Content-type': 'application/json' });
    return this.http.post<IMenuBlock>(`${this.apiURL}/api/menu-blocks`, menuBlock, { headers: httpHeaders });
  }

  deleteById(id: string): any {
    return this.http.delete<IMenuBlock>(`${this.apiURL}/api/menu-blocks/${id}`);
  }

  update(menuBlock: IMenuBlock): Observable<IMenuBlock> {
    let httpHeaders = new HttpHeaders({ 'Content-type': 'application/json' });
    return this.http.put<IMenuBlock>(`${this.apiURL}/api/menu-blocks/${menuBlock.id}`, menuBlock, {
      headers: httpHeaders,
    });
  }

}
