import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ConfigService } from '@ngx-config/core';
import { Observable } from 'rxjs';
import { IMenuItem } from '../../../model/menu-item.model';

@Injectable({
  providedIn: 'root'
})
export class MenuItemCrudService {
  private apiURL: any;

  constructor(configService: ConfigService, private http: HttpClient) {
    this.apiURL = configService.getSettings("dashboard-controller.api_base_url");
  }

  getAll(): Observable<Array<IMenuItem>> {
    return this.http.get<Array<IMenuItem>>(`${this.apiURL}/api/menu-items`);
  }

  getById(id: string): Observable<IMenuItem> {
    return this.http.get<IMenuItem>(`${this.apiURL}/api/menu-items/${id}`);
  }

  getByPageId(pageId: string): Observable<Array<IMenuItem>> {
    return this.http.get<Array<IMenuItem>>(`${this.apiURL}/api/menu-items-bypage/${pageId}`);
  }

  create(menuItem: IMenuItem): Observable<IMenuItem> {
    let httpHeaders = new HttpHeaders({ 'Content-type': 'application/json' });
    return this.http.post<IMenuItem>(`${this.apiURL}/api/menu-items`, menuItem, { headers: httpHeaders });
  }

  deleteById(id: string): any {
    return this.http.delete<IMenuItem>(`${this.apiURL}/api/menu-items/${id}`);
  }

  update(menuItem: IMenuItem): Observable<IMenuItem> {
    let httpHeaders = new HttpHeaders({ 'Content-type': 'application/json' });
    return this.http.put<IMenuItem>(`${this.apiURL}/api/menu-items/${menuItem.id}`, menuItem, {
      headers: httpHeaders,
    });
  }

}
