import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ConfigService } from '@ngx-config/core';
import { Subject, Observable } from 'rxjs';
import { ITargetAvailable } from '../../../model/target-available.model';

@Injectable({
  providedIn: 'root'
})
export class TargetAvailableCrudService {
  private apiURL: any;

  constructor(configService: ConfigService, private http: HttpClient) {
    this.apiURL = configService.getSettings("dashboard-controller.api_base_url");
  }

  getAll(): Observable<Array<ITargetAvailable>> {
    return this.http.get<Array<ITargetAvailable>>(`${this.apiURL}/api/target-availables`);
  }

  getAvailableTargetById(id: string): Observable<ITargetAvailable> {
    return this.http.get<ITargetAvailable>(`${this.apiURL}/api/target-availables/${id}`);
  }

  getAvailableTargetsByType(type: string): Observable<Array<ITargetAvailable>> {
    return this.http.get<Array<ITargetAvailable>>(`${this.apiURL}/api/target-availables-bytype/${type}`);
  }

  createAvailableTarget(target: ITargetAvailable): Observable<ITargetAvailable> {
    let httpHeaders = new HttpHeaders({ 'Content-type': 'application/json' });
    return this.http.post<ITargetAvailable>(`${this.apiURL}/api/target-availables`, target, { headers: httpHeaders });
  }

  deleteAvailableTargetById(id: string): any {
    return this.http.delete<ITargetAvailable>(`${this.apiURL}/api/target-availables/${id}`);
  }

  updateAvailableTarget(target: ITargetAvailable): Observable<ITargetAvailable> {
    let httpHeaders = new HttpHeaders({ 'Content-type': 'application/json' });
    return this.http.put<ITargetAvailable>(`${this.apiURL}/api/target-availables/${target.id}`, target, {
      headers: httpHeaders,
    });
  }
  
}
