import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ConfigService } from '@ngx-config/core';
import { Observable } from 'rxjs';
import { IGroupRepresentation } from '../../../model/group-representation.model';
import { IRoleRepresentation } from '../../../model/role-representation.model';
import { IUser } from '../../../model/user.model';

@Injectable({
  providedIn: 'root'
})
export class UserCrudService {
  private apiURL: any;

  constructor(configService: ConfigService, private http: HttpClient) {
    this.apiURL = configService.getSettings("dashboard-controller.api_base_url");
  }

  getIdmUsers(): Observable<Array<IUser>> {
    return this.http.get<Array<IUser>>(`${this.apiURL}/user/get-idm-users`);
  }

  getIdmRoles(): Observable<Array<IRoleRepresentation>> {
    return this.http.get<Array<IRoleRepresentation>>(`${this.apiURL}/user/get-idm-roles`);
  }
  
  getIdmGroups(): Observable<Array<IGroupRepresentation>> {
    return this.http.get<Array<IGroupRepresentation>>(`${this.apiURL}/user/get-idm-groups`);
  }

  getUserRoles(): Observable<Array<string>> {
    return this.http.get<Array<string>>(`${this.apiURL}/user/get-user-roles`);
  }

  getUserGroups(username:string): Observable<Array<string>> {
    return this.http.get<Array<string>>(`${this.apiURL}/user/get-user-groups/${username}`);
  }

}
