import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { NbTagComponent, NbTagInputAddEvent } from '@nebular/theme';
import { DCATDataset,FormatCount } from '../model/dcatdataset';
import { ODMSCatalogueInfo } from '../model/odmscatalogue-info';
import { SearchFacet } from '../model/search-facet';
import { SearchFilter } from '../model/search-filter';
import { SearchRequest } from '../model/search-request';
import { SearchResult } from '../model/search-result';
import { DataCataglogueAPIService } from '../services/data-cataglogue-api.service';
import { NbLayoutScrollService } from '@nebular/theme';
import {KeyrockUserInformationService} from '../../../auth/services/keyrock-user-information.service';
import { OidcUserInformationService } from '../../../auth/services/oidc-user-information.service';

@Component({
  selector: 'ngx-dataset-filtered',
  templateUrl: './dataset-filtered.component.html',
  styleUrls: ['./dataset-filtered.component.scss']
})
export class DatasetFilteredComponent implements OnInit {
    separatorKeys:number[] = [13, 188];
    searchResponse:SearchResult=new SearchResult();
    searchRequest:SearchRequest=new SearchRequest();
  
    cataloguesInfos: Array<ODMSCatalogueInfo>=[];
    user: any;
    tags: string[] = [];
  
    constructor(private restApi:DataCataglogueAPIService, private nbScroll: NbLayoutScrollService,private userService: OidcUserInformationService, private keyrockUserInformationService: KeyrockUserInformationService ) { }
  
    loading=false;
  
    facetLimits={};
    page: number = 1;
    
    @Input() 
    totalDatasets:number=0;
  
    filters: Array<string> = [];
    filtersTags: Array<string>= [];

  
    ngOnInit(): void {
      this.userService.onUserChange()
      .subscribe((user: any) => {
        this.user = user;
         if (this.user.hasOwnProperty('topics')) {
          this.tags = this.user.topics.topics;
         }
         if (this.tags.length > 0) {
          this.tags.forEach(element => {
            this.searchDatasetOnTagsValue(element);
          });
            
         }
        
        });
    }
  searchDatasetOnTagsValue(tag) {
    this.searchRequest.nodes.push(1);
    let index = this.searchRequest.filters.findIndex(x=> x.field==="ALL");
    if(index<0){
      this.searchRequest.filters.push(new SearchFilter("ALL",tag));
    }else{
      let filter=this.searchRequest.filters[index];
      filter.value=tag;
    }
    this.searchOnFilter();
  /*  tags.forEach(element => {
      console.log(element);
      this.searchRequest.filters.push(new SearchFilter('ALL',element));
      this.searchOnFilter();
    
    });*/

  }
searchOnFilter(){
  this.restApi.searchDatasets(this.searchRequest).subscribe(
    res=>{
      // this.searchResponse=res
      if (res.count > 0){
        console.log(res);
        console.log(this.searchResponse);
       
        this.totalDatasets = this.totalDatasets + res.count;
        let oldResult = this.searchResponse.results;
        if (oldResult === undefined){
          this.searchResponse=res;
          this.searchResponse.results.map((x:DCATDataset)=>{ this.processDataset(x) })
        } else {
          console.log(oldResult);
          let newResult = oldResult.concat(res.results);
          console.log(newResult);
          this.searchResponse.count= this.searchResponse.count + res.count;
          this.searchResponse.results= newResult;
          console.log(this.searchResponse);
          this.searchResponse.results.map((x:DCATDataset)=>{ this.processDataset(x) })
        }
        
        this.loading=false;
      }
     
    },
    err=>{
      console.log(err);
      this.loading=false;
    });
}
processDataset(dataset:DCATDataset):DCATDataset{
  let tmp=[];
  dataset.distributionFormats=[];
  for(let d of dataset.distributions){
    if(tmp.indexOf(d.format)<0){
      let fC=new FormatCount();
      fC.format=d.format;
      fC.count=1;
      dataset.distributionFormats.push(fC);
      tmp.push(d.format);
    }else{
      dataset.distributionFormats[tmp.indexOf(d.format)].count++;
    }
  }

  dataset.description = dataset.description.replace(/\*/g,'').replace(/\\n/g,'')
                                      .replace(/\(http.*\)/g,'').replace(/##\s*/g,'')
                                      .replace(/<.*>(.*)<\/.*>/g,'$1')
                                      .replace(/>/g,'').replace(/\[|\]/g,'')

  return dataset;
}
 getColor(format:string):string{
      switch(format.toLowerCase()){
        case 'csv':
          return '#dfb100';
        case 'html':
          return '#55a1ce';
        case 'json':
        case 'xml':
          return '#ef7100';
        case 'text':
        case 'txt':
          return '#74cbec';
        case 'xls':
        case 'xlsx':
          return '#2db55d';
        case 'zip':
          return '#686868';
        case 'api':
          return 'ec96be';
        case 'pdf':
          return '#e0051e';
        case 'rdf':
        case 'nquad':
        case 'turtle':
        case 'ntriples':
          return '#0b4498';
        case 'fiware':
        case 'ngsi':
        case 'ngsi-ld':
        case 'fiware-ngsi':
        case 'fiware-ngsi-ld':
          return '#65c3d1';
        default:
          return 'default';
      }
    }
  
  
  
  }
