import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { NbTagComponent, NbTagInputAddEvent } from '@nebular/theme';
import { DCATDataset,FormatCount } from '../model/dcatdataset';
import { ODMSCatalogueInfo } from '../model/odmscatalogue-info';
import { SearchFacet } from '../model/search-facet';
import { SearchFilter } from '../model/search-filter';
import { SearchRequest } from '../model/search-request';
import { SearchResult } from '../model/search-result';
import { DataCataglogueAPIService } from '../services/data-cataglogue-api.service';
import { NbLayoutScrollService } from '@nebular/theme';

@Component({
  selector: 'ngx-search',
  templateUrl: './search.component.html',
  styleUrls: ['./search.component.scss']
})
export class SearchComponent implements OnInit {

  separatorKeys:number[] = [13, 188];
  searchResponse:SearchResult=new SearchResult();
  searchRequest:SearchRequest=new SearchRequest();

  cataloguesInfos: Array<ODMSCatalogueInfo>=[]

  constructor(private restApi:DataCataglogueAPIService, private nbScroll: NbLayoutScrollService) { }

  loading=false;

  facetLimits={};
  page: number = 1;

  @Input() 
  totalDatasets:number=0;

  filters: Array<string> = [];
  filtersTags: Array<string>= [];

  ngOnInit(): void {
    this.getState();
    if(this.filters.length == 0 && this.filtersTags.length == 0){
      this.loading=true
      this.restApi.getCataloguesInfo().subscribe(infos =>{
        this.cataloguesInfos = infos;
        this.searchRequest.nodes = infos.map(x=>x.id)
        this.loading=false
        this.pageChanged(this.page);
      },err=>{
        console.log(err);
        this.loading=false;
      })
      console.log("First time")
    } else{
      console.log("Recharge");
      console.log(this.filters);
      console.log(this.filtersTags);
      this.pageChanged(this.page);
      console.log("HERE")
    }
  }

  pageChanged($event:number){
    console.log("Event is: " + $event)
    this.page=$event;
    this.searchRequest.start=($event-1)*this.searchRequest.rows;
    localStorage.setItem("page", JSON.stringify(this.page));
    this.searchDataset();
  }

  searchDataset(isFirst=false){
    this.loading=true
    this.filtersTags=[];

    this.searchRequest.filters.forEach(x=>{
      if(x.field!='ALL'){
        let values = x.value.split(',')
        let name=x.field;
        let index = this.searchResponse.facets.findIndex(x=> x.search_parameter===name)
        if(index>=0){
          name=this.searchResponse.facets[index].displayName;
        }
        values.forEach(y=> this.filtersTags.push(name+": "+y))
      }
    })

    this.restApi.searchDatasets(this.searchRequest).subscribe(
      res=>{
        this.searchResponse=res
        this.totalDatasets = this.searchResponse.count;
        this.searchResponse.results.map((x:DCATDataset)=>{ this.processDataset(x) })
        this.loading=false;
      },
      err=>{
        console.log(err);
        this.loading=false;
      });
      this.nbScroll.scrollTo(0,0);
      localStorage.setItem("cataloguesInfos", JSON.stringify(this.cataloguesInfos));
  }

  onTagRemove(tagToRemove: NbTagComponent): void {
    console.log(tagToRemove.text)
    this.filters = this.filters.filter(x => x!=tagToRemove.text);
    this.searchRequest.filters.map(x =>{ 
      if(x.field=='ALL'){
        let a = x.value.split(',');
        x.value = a.filter(b => b!=tagToRemove.text).join(',');
      }
    })
    this.saveState();
    this.resetPage();
    this.searchDataset()
  }

  onTagAdd({ value, input }: NbTagInputAddEvent): void {
    console.log("Value is " + value);
    console.log("Input is " + input)
    console.log("Filter is " + this.filters)
    console.log("FilterTag is " + this.filtersTags)
    console.log("PAGE NUMBER IS " + this.page);
    //added timeout since comma doesn't desapear from input
    setTimeout(()=>{
      input.nativeElement.value = ''
      if (value) {
        this.filters.push(value);
        this.searchRequest.filters.map(x => {
          if (x.field == 'ALL') {
            if (x.value != '') {
              let a = x.value.split(',');
              a.push(value)
              x.value = a.join(',');
            } else {
              x.value = value;
            }
          }
        })
        this.saveState();
        this.resetPage();
        this.searchDataset()
      }
    }, 50);
  }

  getFacetsLimit(facet){
    if(this.facetLimits[facet]==undefined){
      this.facetLimits[facet]=10;
    }
    return this.facetLimits[facet];
  }

  setFacetsLimit(facet,value){
    this.facetLimits[facet]=value;
  }


  processDataset(dataset:DCATDataset):DCATDataset{
    let tmp=[];
    dataset.distributionFormats=[];
    for(let d of dataset.distributions){
      if(tmp.indexOf(d.format)<0){
        let fC=new FormatCount();
        fC.format=d.format;
        fC.count=1;
        dataset.distributionFormats.push(fC);
        tmp.push(d.format);
      }else{
        dataset.distributionFormats[tmp.indexOf(d.format)].count++;
      }
    }

    dataset.description = dataset.description.replace(/\*/g,'').replace(/\\n/g,'')
																				.replace(/\(http.*\)/g,'').replace(/##\s*/g,'')
																				.replace(/<.*>(.*)<\/.*>/g,'$1')
																				.replace(/>/g,'').replace(/\[|\]/g,'')

    return dataset;
  }

  getColor(format:string):string{
    switch(format.toLowerCase()){
      case 'csv':
        return '#dfb100';
      case 'html':
        return '#55a1ce';
      case 'json':
      case 'xml':
        return '#ef7100';
      case 'text':
      case 'txt':
        return '#74cbec';
      case 'xls':
      case 'xlsx':
        return '#2db55d';
      case 'zip':
        return '#686868';
      case 'api':
        return 'ec96be';
      case 'pdf':
        return '#e0051e';
      case 'rdf':
      case 'nquad':
      case 'turtle':
      case 'ntriples':
        return '#0b4498';
      case 'fiware':
      case 'ngsi':
      case 'ngsi-ld':
      case 'fiware-ngsi':
      case 'fiware-ngsi-ld':
        return '#65c3d1';
      default:
        return 'default';
    }
  }

  onFilterRemove(filter: NbTagComponent): void {
    let tmp=filter.text.split(': ');
    let name = tmp[0];
    let facetIndex = this.searchResponse.facets.findIndex(x=>x.displayName==name)
    if(facetIndex>=0){
      name=this.searchResponse.facets[facetIndex].search_parameter;
    }
    let index = this.searchRequest.filters.findIndex(x=> x.field==name);
    let filterTag = this.searchRequest.filters[index];
    this.searchRequest.filters[index].value=filterTag.value.split(',').filter(x=> x!=tmp[1]).join(',');
    if(this.searchRequest.filters[index].value==''){
      this.searchRequest.filters.splice(index,1);
    }
    this.saveState();
    this.resetPage();
    this.searchDataset()
  }

  getDatasetByFacet(search_parameter,newValue){
    console.log("Called this")
    this.searchRequest.start=0;
    let index = this.searchRequest.filters.findIndex(x=> x.field===search_parameter);
    if(index<0){
      this.searchRequest.filters.push(new SearchFilter(search_parameter,newValue));
    }else{
      let filter=this.searchRequest.filters[index];
      this.searchRequest.filters.splice(index,1)
      let tmp=filter.value.split(',');
      tmp.push(newValue);
      filter.value=tmp.join(',');
      this.searchRequest.filters.push(filter);
    }
    this.saveState();
    this.resetPage();
    this.searchDataset()
  }

  displayFacet(search_parameter,value){
    let index = this.searchRequest.filters.findIndex(x=> x.field===search_parameter);
    if(index<0) return true;
    else{
      let values=this.searchRequest.filters[index].value.split(',');
      let vIndex = values.findIndex(x=>x===value)
      if(vIndex<0) return true;
    }
    return false;
  }

  filterFacets(search_parameter,values:SearchFacet[]){
    let index = this.searchRequest.filters.findIndex(x=> x.field===search_parameter);
    if(index<0) return values;
    else{
      let usedValues=this.searchRequest.filters[index].value.split(',');
      return values.filter( x=> usedValues.indexOf(x.search_value)<0 );
    }
  }

  resetPage(){
    localStorage.setItem("page", JSON.stringify(1));
    this.page = 1;
    this.searchRequest.start = 0;
  }

  saveState(){
    localStorage.setItem("filters", JSON.stringify(this.filters));
    localStorage.setItem("filterTags", JSON.stringify(this.filtersTags));
    localStorage.setItem("searchRequest", JSON.stringify(this.searchRequest));
    localStorage.setItem("cataloguesInfos", JSON.stringify(this.cataloguesInfos));
    localStorage.setItem("searchRequest.filters", JSON.stringify(this.searchRequest.filters));
    localStorage.setItem("searchResponse.facets", JSON.stringify(this.searchResponse.facets));
    localStorage.setItem("searchResponse.results", JSON.stringify(this.searchResponse.results));
    localStorage.setItem("searchResponse.count", JSON.stringify(this.searchResponse.count));
    localStorage.setItem("searchRequest.rows", JSON.stringify(this.searchRequest.rows));
  }

  getState(){
    if(JSON.parse(localStorage.getItem("filters")) != null){
      this.filters = JSON.parse(localStorage.getItem("filters"));
    }
    if(JSON.parse(localStorage.getItem("filtersTags")) != null){
      this.filtersTags =  JSON.parse(localStorage.getItem("filterTags"));
    }
    if(JSON.parse(localStorage.getItem("searchRequest")) != null){
      this.searchRequest = JSON.parse(localStorage.getItem("searchRequest"));
    }
    if(JSON.parse(localStorage.getItem("searchRequest.filters")) != null){
      this.searchRequest.filters =   JSON.parse(localStorage.getItem("searchRequest.filters"));
    }
    if(JSON.parse(localStorage.getItem("searchResponse.facets")) != null){
      this.searchResponse.facets =  JSON.parse(localStorage.getItem("searchResponse.facets"));
    }
    if(JSON.parse(localStorage.getItem("searchResponse.results")) != null){
      this.searchResponse.results =  JSON.parse(localStorage.getItem("searchResponse.results"));
    }
    if(JSON.parse(localStorage.getItem("searchResponse.count")) != null){
      this.searchResponse.count =  JSON.parse(localStorage.getItem("searchResponse.count"));
    }
    if(JSON.parse(localStorage.getItem("searchRequest.rows")) != null){
      this.searchRequest.rows =  JSON.parse(localStorage.getItem("searchRequest.rows"));
    }
    if(JSON.parse(localStorage.getItem("page")) == null || JSON.parse(localStorage.getItem("page")) == 1){
      this.page = 1;
    } else{
      this.page =  JSON.parse(localStorage.getItem("page"));
    }
    if(JSON.parse(localStorage.getItem("cataloguesInfos")) != null){
      this.cataloguesInfos =  JSON.parse(localStorage.getItem("cataloguesInfos"));
    }
  }

}
