import { Component, OnInit } from '@angular/core';
import { MatListOption } from '@angular/material/list';
import * as L from 'leaflet';
import 'leaflet.heat/dist/leaflet-heat.js';
import { DashboardPageCrudService } from '../../dashboard-management/services/dashboard-page-crud.service';
import { MessinaClientService } from '../../messina/service/messina-client.service';
import Amsterdam_bike from '../../../../assets/map/map-component-maps/amsterdam_bike_request.json';
import Messina_cycling_path from '../../../../assets/map/map-component-maps/messina_cycling_paths.json';
import Messina_ATM_Stops from '../../../../assets/map/map-component-maps/atmStops.json';
import Messina_Cameras from '../../../../assets/map/map-component-maps/cameras.json';
import Messina_POIs from '../../../../assets/map/map-component-maps/messina_pois.json';
import Messina_Incidenti_2018 from '../../../../assets/map/map-component-maps/incidenti_messina/street_result_2018.json';
import Messina_Incidenti_2019 from '../../../../assets/map/map-component-maps/incidenti_messina/street_result_2019.json';
import Messina_Incidenti_2020 from '../../../../assets/map/map-component-maps/incidenti_messina/street_result_2020.json';
import Messina_district from '../../../../assets/map/map-component-maps/out_district_messina.json';
import Bilbao_air_quality from '../../../../assets/map/map-component-maps/bilbao_air_quality_request.json';
import Bilbao_bike from '../../../../assets/map/map-component-maps/bilbao_bike_request.json';
import Bilbao_sound from '../../../../assets/map/map-component-maps/bilbao_sound_request.json';
import Bilbao_wifi from '../../../../assets/map/map-component-maps/bilbao_wifi_request.json';
import Bilbao_layers from '../../../../assets/map/map-component-maps/BilbaoLayers.json';
import Helsinki_layers from '../../../../assets/map/map-component-maps/Helsinki_layers.json';
import Helsinki_Bike from '../../../../assets/map/map-component-maps/HSL_n_BICYCLE_STATIONS_request.json';
import Helsinki_Cyclist from '../../../../assets/map/map-component-maps/HSL_n_cyclist_request.json';
import Helsinki_Terminalit from '../../../../assets/map/map-component-maps/HSL_n_terminalit_request.json';
import { ConfigService } from '@ngx-config/core';
import { DatastorageService } from '../../dashboard-management/services/datastorage.service';
import { TranslateService } from '@ngx-translate/core';
import { DashboardGuideComponent } from '../../dashboard-management/dashboard-guide/dashboard-guide.component';
import { NbDialogService } from '@nebular/theme';
import { MapModalComponent } from './map-modal/map-modal.component';
import { DataStorageEntity } from '../../../model/data-storage-entity.model';
import { Subtype } from '../../../model/enumeration/subtype.model';

export interface DataStorageLayer {
  id: string;
  name: string;
  description: string;
  alternateName: string;
  type: string;
  subtype: Subtype;
  map: L.GeoJSON;
  geometry: L.GeoJSON;
  properties?: {
    name: string;
    alternateName: string,
  }
}

export interface FeatureProperties {
  fill: string;
  fillOpacity: number;
}

@Component({
  selector: 'ngx-map',
  styleUrls: ['./map.component.scss'],
  templateUrl: './map.component.html',
})
export class MapComponent implements OnInit {
  constructor(
    public dialogService: NbDialogService,
    private dashboardPageService: DashboardPageCrudService,
    private configService: ConfigService,
    private dataStorageService: DatastorageService,
    private translateService: TranslateService,
    private messinaClientService: MessinaClientService,
    private modalService: NbDialogService) {
    //this.fillGeojesonlayer();

  }
  public map: L.Map;

  public legend = new (L.Control.extend({
    options: { position: 'bottomright' }
  }));

  options = {
    layers: [
      L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '...' }),
    ],
    zoom: 9,
  };

  heatOptions = {
    radius: 12,
    max: 1.0,
    blur: 15,
    maxZoom: 18,
    //gradient: { 0.5: 'green', 0.6: 'yellow', 1.0: 'red' },
    minOpacity: 0.7
  }
  pilot: string;

  layersByPilot: { [key: string]: string[] } = {
    MESSINA: [],
    BILBAO: [],
    AMSTERDAM: [],
    HELSINKI: [],
    URBANITE: [],
  }

  coordinatesByPilot: { [key: string]: L.LatLngExpression } = {
    MESSINA: [38.183333, 15.566667],
    BILBAO: [43.2627100, -2.9252800],
    AMSTERDAM: [52.3740300, 4.8896900],
    HELSINKI: [60.1695200, 24.9354500],
    URBANITE: [60.1695200, 24.9354500],
  }

  geojsonLayerContent: { [key: string]: L.GeoJSON } = {};
  layerDescription: { [key: string]: string } = {};

  heatlayerLayerContent: { [key: string]: L.HeatLayer } = {
  };


  ngOnInit(): void {
    this.pilot = this.configService.getSettings("default_pilot");
    this.dataStorageService.getMapLayers().subscribe((entities: DataStorageLayer[]) => {
      //console.log(entities);
      entities.forEach(entity => {
        console.log(entity);
        if (entity.subtype == Subtype.HeatLayer) {
          this.heatlayerLayerContent[entity.alternateName] = <L.HeatLayer>L.heatLayer(this.heatLayerConverter(<any>entity.map), this.heatOptions);
        } else {
          this.geojsonLayerContent[entity.alternateName] = <L.GeoJSON>this.getLayer(entity.alternateName, entity.map);
        }
        this.layerDescription[entity.alternateName] = entity.description;
        this.layersByPilot[this.pilot] = this.layersByPilot[this.pilot].concat(entity.alternateName);
      });
    }, err => { console.log(err) });
  }

  onChange(event: any) {
    let clickedLayerName = event.option.value;
    let geojsonLayer = <L.GeoJSON>this.geojsonLayerContent[clickedLayerName];
    let heatLayer = <L.HeatLayer>this.heatlayerLayerContent[clickedLayerName];
    console.log("geojsonlayer value ", geojsonLayer);
    console.log("heatLayer value ", heatLayer);
    if (event.option.selected === true) {
      if (geojsonLayer !== null || geojsonLayer !== undefined) {
        this.map.addLayer(geojsonLayer);
        //pointToLayer icon
        geojsonLayer.eachLayer((layer: L.Layer) => {
          if (layer instanceof L.Marker) {
            layer.setIcon(L.icon({
              iconUrl: 'assets/img/markers/marker-icon.png',
              iconSize: [25, 41],
              iconAnchor: [12, 41],
              popupAnchor: [1, -34],
              shadowSize: [41, 41]
            }));
          }
        });
        this.map.fitBounds(geojsonLayer.getBounds(), {
          padding: [50, 50]
        });
      } else {
        this.map.addLayer(heatLayer);
        this.map.addControl(this.legend);
      }
    } else {
      //console.log("DECHARGE LAYER-> " + clickedLayerName)
      if (geojsonLayer !== null || geojsonLayer !== undefined) {
        this.map = this.map.removeLayer(geojsonLayer);
      } else {
        this.map = this.map.removeLayer(heatLayer);
        this.map.removeControl(this.legend);
      }
    }



  }

  /**
   * This transform geoJSON into array managed by leaflet for heatmap
   * @param geoJson geoJSON 
   * @returns Points with intensity for leflet heatmap
   */
  private heatLayerConverter(geoJson) {
    var result = [];
    for (let r in geoJson['features']) {
      let coord = geoJson['features'][r].geometry.coordinates;
      result.push([coord[1], coord[0], 0.5]); // lat, lng, intensity 
    }
    // console.log("Points result.length =>" + result.length + "  result: " + result);
    return result;
  }

  onMapReady(map: L.Map) {
    this.map = map;
    setTimeout(() => {
      this.map.invalidateSize();
      this.map.setView(this.coordinatesByPilot[this.configService.getSettings("default_pilot")], 13.5);
    }, 50);

    this.legend.onAdd = (map) => {
      let div = L.DomUtil.create('div', 'info legend');
      let grades: string[] = ["Low level", "Mid level", "High level"]
      let labels = ["<strong>Sound Level</strong>"];
      for (let i = 0; i < grades.length; i++) {
        div.innerHTML += '<i style="background: ' + this.getLegendColor(i) + '">' + '&nbsp;' + '</i> ' + `<b>${grades[i]}</b>` + '<br>'
      }

      return div;
    };
  }

  icon = new L.Icon({
    iconSize: [25, 41],
    iconAnchor: [13, 41],
    iconUrl: 'assets/img/markers/marker-icon.png',
    iconRetinaUrl: 'assets/img/markers/marker-icon-2x.png',
    shadowUrl: 'assets/img/markers/marker-shadow.png'
  });

  public getLayer(title: string, l) {
    let layer = L.geoJSON(l, {
      onEachFeature: (feature, layer) => {
        //console.log(feature)
        layer.bindPopup(this.getPopupContent(title, feature))
      },
      style: (feature) => {
        let style = {
          color: 'blue'
        }

        if (feature['properties'] != null && feature['properties'] != undefined) {

          if (feature.properties.hasOwnProperty('fill')) {
            style['fillColor'] = feature.properties['fill'];
          }

          if (feature.properties.hasOwnProperty('fill-opacity')) {
            style['fillOpacity'] = feature.properties['fill-opacity'];
          }

          if (feature.properties.hasOwnProperty('stroke')) {
            style['color'] = feature.properties['stroke']
          }

          if (feature.properties.hasOwnProperty('stroke-width')) {
            style['weight'] = feature.properties['stroke-width']
          }

          if (feature.properties.hasOwnProperty('stroke-opacity')) {
            style['opacity'] = feature.properties['stroke-opacity']
          }

          // if(feature.properties.hasOwnProperty('fill')){
          //   return { color: feature.properties['fill'],
          //   fillColor: feature.properties['fillColor'],
          // };
          // } else if (feature.properties.hasOwnProperty('stroke')){
          //   console.log("HERE ");
          //   return {
          //     color: feature.properties['stroke'],
          //     weight: feature.properties['stroke-width']+3,
          //     opacity: feature.properties['stroke-opacity']
          // };
          // }
        }
        return style;
      },
    });
    return layer;
  }




  public getPopupContent(title: string, feature) {
    let content = "<h3>" + title + "</h3>";

    if (feature.properties) {
      for (var p in feature.properties) {
        content += "<p>" + p + ": " + feature.properties[p] + "</p>";
      }
    }

    // if (feature.coordinates) {
    //   content += "<p>Coordinates: " + feature.coordinates + "</p>";
    // }

    return content;
  }

  getLegendColor(v: number) {
    switch (v) {
      case 1:
        return 'yellow'
      case 2:
        return 'red';
      default:
        return 'green';
    }
  }

  openGuide() {
    const dialogRef = this.dialogService
      .open(DashboardGuideComponent,
        { context: { info: "Map Layers Guide" } });
  }

  private getIcon(type): string {
    switch (type) {
      case "Retail_Services":
        return "store";
      case "Restaurant":
        return "utensils";
      case "Medical_Facility":
        return "heartbeat";
      case "School":
        return "school";
      case "Sport_Recreation":
        return "running";
      case "City_Town_Center":
        return "university";
      case "Hotel":
        return "h-square";
      case "Enclosed_Carpark":
        return "parking";
      case "Automobile_Facility":
        return "car-side";
      case "Bus_Station":
        return "bus";
      case "Religious_Facility":
        return "pray";
      case "Open_Carpark":
        return "parking";
      case "Forest":
        return "tree";
      case "Cultural_Buildings":
        return "building";
      case "River":
        return "water";
      case "Rail_Station":
        return "train";
      case "Ferry":
        return "ship";
      case "Lake":
        return "water";
      case "Public_Buildings":
        return "building";
      case "WaterElements":
        return "water";
      case "Regio Nat State Park":
        return "tree";
      case "lndustrial_Area":
        return "industry";
      case "Industry":
        return "industry";
      case "Sand_Area":
        return "umbrella-beach";
      case "Tunnel":
        return "archway";
      case "Rail":
        return "train";
    }
    return "map-marker"
  }

  buildGeoJSON(data: any) {
    let result = [];
    var geojsonModel = {
      "type": "Feature",
      "geometry": {
        "type": "Point",
        "coordinates": []
      },
      "properties": {
      }
    };
    console.log("HERE");
    console.log(Object.keys(data));
    console.log(data['results'])
    console.log(Object.keys(data['results']));
    console.log(data['results']['features'])
    console.log(Object.keys(data['results']['features']));
    for (let feature in data['results']) {
      console.log("R is");
      console.log(feature);
    }
  }

  public invokeModal(event, layer: string) {
    console.log(event);

    const dialogRef = this.dialogService.open(MapModalComponent, {
      context: {
        layer: layer,
        description: this.layerDescription[layer],
      },
    });
  }
}