import { Component, OnInit, ViewChild } from "@angular/core";

import {
  ApexAxisChartSeries,
  ApexTitleSubtitle,
  ApexDataLabels,
  ApexChart,
  ApexPlotOptions,
  ApexLegend,
  ApexXAxis,
  ApexYAxis,
  ApexNoData
} from "ng-apexcharts";
import { MessinaClientService } from "../service/messina-client.service";

export type ChartOptions = {
  legend: ApexLegend,
  series: ApexAxisChartSeries;
  chart: ApexChart;
  dataLabels: ApexDataLabels;
  title: ApexTitleSubtitle;
  plotOptions: ApexPlotOptions;
  xaxis: ApexXAxis;
  yaxis: ApexYAxis;
  noData: ApexNoData;
};

@Component({
  selector: 'ngx-martrix-batch',
  templateUrl: './martrix-batch.component.html',
  styleUrls: ['./martrix-batch.component.scss']
})
export class MartrixBatchComponent implements OnInit {

  @ViewChild("chart") chart: any;
  public chartOptions: Partial<ChartOptions>;

  serie_plus1=[];
  serie_minus1=[];
  selectedRoad1 = "A20 - Messina-Boccetta";
  date1 = "2021-06-08T00:00:00Z";
  
  serie_plus2=[];
  serie_minus2=[];
  selectedRoad2 = "Diramazione Settentrionale Sicula - Messina - SS113 Settentrionale Sicula";
  date2 = "2021-06-08T00:00:00Z"

  roads: any;

  constructor(private messinaClient: MessinaClientService) {
    this.chartOptions = {
      legend:{
        position:'bottom',
        show: true
      },
      series: [

      ],
      chart: {
        height:300,
        type: "heatmap",
        toolbar: {
          show: false
        }
      },
      plotOptions: {
        heatmap: {
          shadeIntensity: 0.2,
          enableShades: false,
          colorScale: {
            ranges: [
              {
                from: 0,
                to: 1,
                color: "#009688",

              },
              {
                from: 1,
                to: 2,
                color: "#4caf50"
              },
              {
                from: 2,
                to: 3,
                color: "#8bc34a"
              },
              {
                from: 3,
                to: 4,
                color: "#cddc39"
              },
              {
                from: 4,
                to: 5,
                color: "#ffeb3b"
              },
              {
                from: 5,
                to: 6,
                color: "#ffc107"
              },
              {
                from: 6,
                to: 7,
                color: "#ff9800"
              },
              {
                from: 7,
                to: 8,
                color: "#ff5722"
              },
              {
                from: 8,
                to: 9,
                color: "#f44336"
              },
              {
                from: 9,
                to: 10,
                color: "#970e0e"
              }
            ]
          }
        }
      },
      dataLabels: {
        enabled: false
      },
      title: {
        text: "Jam factor for a specific road over a week"
      },
      xaxis: {
        type: "category",
        categories: [ "00:00", "01:00", "02:00", "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00", "11:00", "12:00", "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00", "21:00", "22:00", "23:00"]
      },

      yaxis: {
        title: {
          text: 'Week',
        }
      },

      noData: {
        text: "No data. Road with only one direction.",
        align: 'center',
        verticalAlign: 'middle',
        offsetX: 0,
        offsetY: 0,
        style: {
          color: "#ffc107",
          fontSize: '15px',
          fontFamily: undefined
        }
      },


    };
  }

  ngOnInit(): void {
    //Call APIs and update UI with inizial parameters
    this.getDataHeatmap(this.date1, this.selectedRoad1, this.date2, this.selectedRoad2);
  }

  updateHeatMaps($event) {
    //Read parameters
    var obsDate1 = $event['obsDate1'];
    var obsDate2 = $event['obsDate2'];
    var tzoffset = (new Date()).getTimezoneOffset() * 60000; //offset in milliseconds
    var obsDateTZ1 = (new Date(obsDate1 - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';
    var obsDateTZ2 = (new Date(obsDate2 - tzoffset)).toISOString().slice(0, -1).split('.').shift() + 'Z';
    var selectedRoad1 = $event['selectedRoad1'];
    var selectedRoad2 = $event['selectedRoad2'];

    // Update global roads
    this.selectedRoad1 = selectedRoad1;
    this.selectedRoad2 = selectedRoad2;

    //Call APIs and update UI
    this.getDataHeatmap(obsDateTZ1,selectedRoad1,obsDateTZ2,selectedRoad2);
  }

/**
 * Call APIs and refresh Maps on UI based on resulr and passed parameters
 * @param date1  date one
 * @param road1  road one
 * @param date2  date two
 * @param road2  road two
 */
  private getDataHeatmap(date1, road1, date2, road2) {
    this.messinaClient.getBatchData(date1, road1).subscribe(res => {
      this.chartOptions.xaxis.categories = res.labels;
      this.serie_plus1 = res.results["+"];
      this.serie_minus1 = res.results["-"];
    });

    this.messinaClient.getBatchData(date2, road2).subscribe(res => {
      this.chartOptions.xaxis.categories = res.labels;
      this.serie_plus2 = res.results["+"];
      this.serie_minus2 = res.results["-"];
    });
  }

}
