import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ConfigService } from '@ngx-config/core';
import { Observable, of } from 'rxjs';
import March_2017_5min from '../../../../assets/map/2017-March-heatgeo-5minutes.json';
import March_2017_10min from '../../../../assets/map/2017-March-heatgeo-10minutes.json';
import November_2017_5min from '../../../../assets/map/2017-November-heatgeo-5minutes.json';
import November_2017_10min from '../../../../assets/map/2017-November-heatgeo-10minutes.json';
import August_2017_10min from '../../../../assets/map/2017-August-heatgeo-10minutes.json';
import August_2017_5min from '../../../../assets/map/2017-August-heatgeo-5minutes.json';
import March_2018_5min from '../../../../assets/map/2018-March-heatgeo-5minutes.json';
import March_2018_10min from '../../../../assets/map/2018-March-heatgeo-10minutes.json';
import November_2018_5min from '../../../../assets/map/2018-November-heatgeo-5minutes.json';
import November_2018_10min from '../../../../assets/map/2018-November-heatgeo-10minutes.json';
import August_2018_10min from '../../../../assets/map/2018-August-heatgeo-10minutes.json';
import August_2018_5min from '../../../../assets/map/2018-August-heatgeo-5minutes.json';
import March_2019_5min from '../../../../assets/map/2019-March-heatgeo-5minutes.json';
import March_2019_10min from '../../../../assets/map/2019-March-heatgeo-10minutes.json';
import November_2019_5min from '../../../../assets/map/2019-November-heatgeo-5minutes.json';
import November_2019_10min from '../../../../assets/map/2019-November-heatgeo-10minutes.json';
import August_2019_10min from '../../../../assets/map/2019-August-heatgeo-10minutes.json';
import August_2019_5min from '../../../../assets/map/2019-August-heatgeo-5minutes.json';
import March_2020_5min from '../../../../assets/map/2020-March-heatgeo-5minutes.json';
import March_2020_10min from '../../../../assets/map/2020-March-heatgeo-10minutes.json';

@Injectable({
  providedIn: 'root'
})
export class MessinaClientService {
  private apiURL: any;
  private token: any;

  constructor(configService: ConfigService, private http: HttpClient) {
    this.apiURL = configService.getSettings("messina.api_base_url");
    this.token = configService.getSettings("messina.token");
  }

  getTrafficByDateTime(startDateTime: string, endDateTime: string, page: number = 0, pageSize: number = 40000): Observable<any> {
    return this.http.get<any>(`${this.apiURL}/api/v1/timeSeries/HereAPI_Traffic?page=${page}&pageSize=${pageSize}&groupBy=datetime&token=${this.token}&startDatetime=${startDateTime}&endDatetime=${endDateTime}`);
  }

  getPOI(page: number = 0, pageSize: number = 40000): Observable<any> {
    return this.http.get<any>(`${this.apiURL}/api/v2/static/collection/POIs?page=${page}&pageSize=${pageSize}&format=geojson&token=${this.token}`);
  }
  getDataStoragePOIs(): Observable<any> {
    return this.http.get<any>(`https://messina.urbanite.esilab.org/data/getTData/pointOfInterest/messina?limit=1000`);

  }
  getStops(page: number = 0, pageSize: number = 40000): Observable<any> {
    return this.http.get<any>(`${this.apiURL}/api/v2/static/collection/atmStops?page=${page}&pageSize=${pageSize}&format=geojson&token=${this.token}&filter=${JSON.stringify({ "mainStop": true })}`);
  }

  getCameras(page: number = 0, pageSize: number = 40000): Observable<any> {
    return this.http.get<any>(`${this.apiURL}/api/v2/static/collection/cameras?page=${page}&pageSize=${pageSize}&format=geojson&token=${this.token}`);
  }

  getCyclingPaths(page: number = 0, pageSize: number = 40000): Observable<any> {
    return this.http.get<any>(`${this.apiURL}/api/v2/static/collection/cyclingPaths?page=${page}&pageSize=${pageSize}&format=json&token=${this.token}`);
  }

  getBatchData(date: string, roads:string=undefined): Observable<any>{
    if(roads==undefined || roads==null){
      // roads="Messina-Boccetta";
      roads="A20 - Messina-Boccetta";
    }
    return this.http.get<any>(`${this.apiURL}/api/v1/trafficBatch?date=${date}&roads=${roads}&token=${this.token}`);
  }

  getRoads(): Observable<any>{
    return this.http.get<any>(`${this.apiURL}/api/v1/roads?token=${this.token}`);
  }

  getHeatMap(selectedPeriod: string, startDateTime: string, endDateTime: string, factorMinutesPerStop: number = 10, page: number = 0, pageSize: number = 40000): Observable<any> {
    //TODO DEVELOP THIS WHEN HEATMAP APIs ARE AVAILABLE
    //return this.http.get<any>(`${this.apiURL}/api/v1/timeSeries/heatMap?page=${page}&pageSize=${pageSize}&groupBy=datetime&token=${this.token}&factorMinutesPerStop=${factorMinutesPerStop}&startDatetime=${startDateTime}&endDatetime=${endDateTime}`);
    
    //Mocked mode due to unavailability of API
    let res = null;
    // Year 2017
    if (selectedPeriod == '2017_03_F10') {
      res = March_2017_10min;
    } else if (selectedPeriod == '2017_03_F05') {
      res = March_2017_5min;
    } else if (selectedPeriod == '2017_08_F10') {
      res = August_2017_10min;
    } else if (selectedPeriod == '2017_08_F05') {
      res = August_2017_5min;
    } else if (selectedPeriod == '2017_11_F10') {
      res = November_2017_10min;
    } else if (selectedPeriod == '2017_11_F05') {
      res = November_2017_5min;
    // Year 2018
    } else if (selectedPeriod == '2018_03_F10') {
        res = March_2018_10min;
    } else if (selectedPeriod == '2018_03_F05') {
      res = March_2018_5min;
    } else if (selectedPeriod == '2018_08_F10') {
      res = August_2018_10min;
    } else if (selectedPeriod == '2018_08_F05') {
      res = August_2018_5min;
    } else if (selectedPeriod == '2018_11_F10') {
      res = November_2018_10min;
    } else if (selectedPeriod == '2018_11_F05') {
      res = November_2018_5min;
    // Year 2019
    } else if (selectedPeriod == '2019_03_F10') {
      res = March_2019_10min;
    } else if (selectedPeriod == '2019_03_F05') {
      res = March_2019_5min;
    } else if (selectedPeriod == '2019_08_F10') {
      res = August_2019_10min;
    } else if (selectedPeriod == '2019_08_F05') {
      res = August_2019_5min;
    } else if (selectedPeriod == '2019_11_F10') {
      res = November_2019_10min;
    } else if (selectedPeriod == '2019_11_F05') {
      res = November_2019_5min;
    // Year 2020
    } else if (selectedPeriod == '2020_03_F10') {
      res = March_2020_10min;
    } else if (selectedPeriod == '2020_03_F05') {
      res = March_2020_5min;
    } 

    return of(res);
  }

}
