import { Component, HostListener, OnInit } from '@angular/core';
import { NbAccessChecker } from '@nebular/security';
import { NbMenuItem } from '@nebular/theme';
import { ConfigLoader, ConfigService } from '@ngx-config/core';
import { IMenuBlock } from '../model/menu-block.model';
import { MenuBlockCrudService } from './dashboard-management/services/menu-block-crud.service';
import { MenuMapperService } from './dashboard-management/services/menu-mapper.service';
import { PublicDashboardService } from './dashboard-management/services/public-dashboard.service';
import { MENU_ITEMS } from './pages-menu';

@Component({
  selector: 'ngx-pages',
  styleUrls: ['pages.component.scss'],
  template: `
  <ngx-loader></ngx-loader>
    <ngx-one-column-layout>
      <nb-menu [items]="menu"></nb-menu>
      <router-outlet></router-outlet>
    </ngx-one-column-layout>
  `,
})
export class PagesComponent implements OnInit {
  menu = MENU_ITEMS;
  userRoles: string[];
  demoEnabled=false;
  extPages=[];
  default_pilot="URBANITE";
  json_data: any;

  constructor(
    private accessChecker: NbAccessChecker,
    private configService: ConfigService,
    private menuBlockService: MenuBlockCrudService,
    private menuMapperService: MenuMapperService,
    private publicDashboardService: PublicDashboardService
    ) {
  }

  ngOnInit() {
    this.publicDashboardService.getAllPublicDashboard().subscribe(res => {
      // console.log("HITTED")
      // console.log(res);
    });
    this.default_pilot = this.configService.getSettings("default_pilot").toUpperCase();
    this.json_data = this.configService.getSettings(`${this.default_pilot.toLowerCase()}_menu`);
    
    this.extPages = this.configService.getSettings("external-application-cards");
    this.demoEnabled = this.configService.getSettings("enable_demo_pages");
    this.menu.map( x=> {
      if( x.data!=undefined && x.data['name']=='administration'){
        x.children.map( y =>{
          if(y.data['name']=='data-catalogue-administration'){
            y.url = `${this.configService.getSettings('idra_base_url')}/${this.configService.getSettings('idra_portal_base_href')}/#/catalogues`
          }

          if(y.data['name']=='idm-administration'){
            y.url = `${this.configService.getSettings('idmBaseURL')}/auth/admin/${this.configService.getSettings('idmRealmName')}/console`
          }
        })
      }
    })

    /** JMA - Add SHARED Menu to the Dashboard */
    // this.menuBlockService.getAll().subscribe((response: Array<IMenuBlock>) => {
    //   // console.log('block response: ', JSON.stringify(response));
    //   if (response !== undefined  && response !== null){
    //     try{
    //       this.menu = this.menu.concat(response.map(item => this.menuMapperService.mapBlockToNbMenuItem(item))) ;
    //     } catch(e) {
    //       console.log("# Error in menu content loading due to " + e);
    //     }
    //   }
    //   // console.log("# Menu SHARED => "+this.menu);
    // });
    
    /** JMA - Add SHARED Menu to the Dashboard */
    this.menuBlockService.getAll().subscribe((response: Array<IMenuBlock>) => {
      // console.log('block response: ', JSON.stringify(response));
      if (response !== undefined  && response !== null && response){
        try{
          console.log(response)
          /**************** */
          for(let i = 0; i< response.length; i++){
            if(response[i].linkedItems == null){
              let tmp = this.menuMapperService.mapBlockToNbMenuItem(response[i])
              this.authMenuItem(tmp)
              this.menu = this.menu.concat(tmp);
            }
          }
          /**************** */
        } catch(e) {
          console.log("# Error in menu content loading due to " + e);
        }
      }
      // console.log("# Menu SHARED => "+this.menu);
    });
    this.authMenuItems();
  }

  authMenuItems() {
    this.menu.forEach(item => {
      this.authMenuItem(item);
    });
  }

  authMenuItem(menuItem: NbMenuItem) {
    if (this.configService.getSettings('enableAuthentication')){
      if (menuItem.data && menuItem.data['name']) {
        this.accessChecker.isGranted('view', menuItem.data['name']).subscribe(res =>
          menuItem.hidden = !res
        )
      } else {
        menuItem.hidden = true;
      }
    }
    /**
     * MMA - Useful to enable a specific menuItem only for a specific Pilot. 
     * NOTE: "URBANITE" is visible for all pilots
     * H
     */
    if(menuItem.data && menuItem.data['key'] != undefined && menuItem.data['key'] == "data_analysis"){
      menuItem.children.forEach(x => {
        if(x.data && x.data['key'] != undefined && this.json_data['data_analisys'][x.data.key] != undefined){
          x.hidden = !this.json_data['data_analisys'][x.data.key]
        } else{
          x.hidden = true;
        }

        // When the city is AMSTERDAM the Bike Trajectories option is called Bike Data
        if (this.default_pilot === "AMSTERDAM" && x.title === "Bike Trajectories") {
          x.title = "Bike Data";
        }
      })
    }
  }

  @HostListener('window:beforeunload', ['$event'])
  beforeunloadHandler(event) {
      this.clearState();
  }
  clearState(){
    console.log("Clearing state");
    localStorage.removeItem("filters");
    localStorage.removeItem("filtersTags");
    localStorage.removeItem("searchRequest");
    localStorage.removeItem("searchRequest.filters");
    localStorage.removeItem("searchRequest.rows");
    localStorage.removeItem("searchRequest.count");
    localStorage.removeItem("searchResponse.facets");
    localStorage.removeItem("searchResponse.results");
    localStorage.removeItem("cataloguesInfos");
    localStorage.removeItem("page");
   } 

}
