import { Component } from "@angular/core";
import { environment } from "../../../../environments/environment";
import { ConfigService } from "@ngx-config/core";

@Component({
  selector: "ngx-ml-module",
  templateUrl: "./ml-module.component.html",
  styleUrls: ["./ml-module.component.scss"],
})
export class MlModuleComponent {
  constructor(configService: ConfigService) {
    this.storage_url =
      configService.getSettings("traffic_simulation_storage_url") + "/storage";
    this.file_url =
      configService.getSettings("traffic_simulation_storage_url") +
      "/storage/ml-experiments";
  }
  storage_url: any;
  file_url: any;
  visualizationBtnDisabled: boolean = true;
  mlBtnDisabled: boolean = true;
  recommendationBtnDisabled: boolean = true;
  selectedExperiment: any = undefined;
  experiments = [
    {
      name: "Experiment 1",
      goal: "The goal of this experiment is to analyze the impact on CO2 emissions when closing the Moyua square in conjunction with increased number of cyclists /decreased number of cars.  By doing so, different scenarios can be compared and the best one can be implemented.",
      data: "Each row in the data represents one simulation run. Each of the 14 simulation runs is a variation of the number of cyclists when the Moyua square is closed. The number of cyclists varies from 1500 to 19000 while changing the number of inhabitants from 2.000 to 20.000, respectively. The input features are related to: surface of closed area, capacity of closed road, number of lanes of closed road, number of incoming roads, type of district, number of bus stops nearby closed area, number of cars, pt, bicycles, avg travel time, number of cars/pt/bicycles nearby closed road, CO2 nearby closed area while the target variable represents the amount of CO2 in the closed area.",
      orangeInteractions:
        "• Load data     • Visualize data     • Apply ML algorithms     • Analyze ML results",
    },
    {
      name: "Experiment 2",
      goal: "The goal of this experiment is to help decision-makers in proposing a mobility policy based on previously defined city-objectives.  The decision-makers input the required city changes, while the system outputs a mobility policy that satisfies that specification. The mobility policies are related to closing of the Moyua square in the city center in specific time and for different duration, while the city changes are related to air pollution and usage of different means of transport.",
      data: "40 variations of closing the Moyua square in different time (from 8am to 17 pm) and duration (1,2,3, or 4 hours) of the day were simulated.  The input features are represented by some KPIs and other simulation results such as: air pollutants (CO2, NOx, PM), usage of different modes of transport (car, bicycle, pt, walk), bike safety and bikeability.  The target variables that represents the mobility policy are related to the starting time and duration of closure. These variables are discretized into 30 mins and 15 mins respectively.",
      orangeInteractions: "• Load data     • Visualize data",
    },
    {
      name: "Experiment 3",
      goal: "The goal of this experiment is to improve experiment 2 by adding additional target variables related to closing of different areas around the Moyua square. This allows the decision-makers to consider more factors in order to satisfy the city goals.",
      data: "192 simulations were executed that differ in start, duration and area of closure. Overall 33 combinations of 10 different areas were simulated, where the start hour of closure is from 7am to 5pm with duration of 1, 2, 3 or 4 hours. The input features are related to the simulation results and predefined KPIs as well. The same set of KPIs are calculated for 3 different geographical levels: local (in Moyua square), local-extended (Moyua + surrounding areas), city-wide. The target variables include start and duration of closure, area that is closed and length of streets included in the closed area.",
      orangeInteractions: "• Load data     • Visualize data",
    },
  ];

  onExperiment(experiment) {
    this.selectedExperiment = experiment;
    this.visualizationBtnDisabled = false;
    this.mlBtnDisabled = false;
    this.recommendationBtnDisabled = false;
  }

  onVisualization() {}

  onMl() {}

  onRecommendation() {}
}
