import { HttpClientModule } from "@angular/common/http";
import { NgModule } from "@angular/core";
import { FormsModule, ReactiveFormsModule } from "@angular/forms";
import { LeafletModule } from "@asymmetrik/ngx-leaflet";
import {
  NbAccordionModule,
  NbActionsModule,
  NbButtonGroupModule,
  NbButtonModule,
  NbCardModule,
  NbCheckboxModule,
  NbIconModule,
  NbInputModule,
  NbListModule,
  NbRadioModule,
  NbSelectModule,
  NbTableModule,
  NbTabsetModule,
  NbTooltipModule,
  NbToggleModule,
  NbToastrModule,
  NbDatepickerModule,
  NbProgressBarModule,
  NbDialogModule,
  NbSpinnerComponent,
  NbSpinnerModule,
} from "@nebular/theme";
import {
  InjectableRxStompConfig,
  RxStompService,
  rxStompServiceFactory,
} from "@stomp/ng2-stompjs";
import { ChartModule } from "angular2-chartjs";
import { NgxEchartsModule } from "ngx-echarts";
import { environment } from "../../../environments/environment";
import { ThemeModule } from "../../@theme/theme.module";
import { ChartsModule } from "../charts/charts.module";
import { NetworkCreationComponent } from "./network-creation/network-creation.component";
import { ScenariosComponent } from "./scenarios/scenarios.component";
import { SimulationWizardRoutingModule } from "./simulation-wizard-routing.module";
import { ViewContainerDirective } from "./utils/directives/view-container.directive";
import SelectArea from "leaflet-area-select";
import { ResultsComponent } from "./results/results.component";
import { ResultsMapComponent } from "./results-map/results-map.component";
SelectArea; // Dirty trick to load leaflet area select
import { OurSpiderChartComponent } from "./our-spider-chart/our-spider-chart.component";
import { SimulationCreationComponent } from "./simulation-creation/simulation-creation.component";
import { ExistingSimulationsComponent } from "./existing-simulations/existing-simulations.component";
import { MlModuleComponent } from "./ml-module/ml-module.component";
import { RecommenderPageComponent } from "./recommender-page/recommender-page.component";
import { EditNetworkMapComponent } from "./edit-network-map/edit-network-map.component";
import { ConfigService } from '@ngx-config/core';

export class MyRxStompConfig extends InjectableRxStompConfig {
  constructor(private configService: ConfigService) {
      super();
      this.brokerURL = `${configService.getSettings("traffic_simulation_stomp_url")}`;
      this.heartbeatIncoming = 0;
      this.heartbeatOutgoing = 10000;
      this.reconnectDelay = 500;
      this.debug = (msg: string): void => {
        environment.showWebSocketDebug && console.debug(msg);
    };
  }
}

@NgModule({
  imports: [
    HttpClientModule,
    ThemeModule,
    NbCardModule,
    NbActionsModule,
    NbAccordionModule,
    NbListModule,
    NbTabsetModule,
    NbTableModule,
    ChartsModule,
    LeafletModule,
    NbButtonGroupModule,
    NbButtonModule,
    FormsModule,
    NbInputModule,
    ReactiveFormsModule,
    NbCheckboxModule,
    NbRadioModule,
    ChartModule,
    NgxEchartsModule,
    NbSelectModule,
    NbTooltipModule,
    NbIconModule,
    SimulationWizardRoutingModule,
    NbToggleModule,
    NbToastrModule,
    NbDatepickerModule,
    NbProgressBarModule,
    NbDialogModule.forRoot(),
    NbSpinnerModule,
  ],
  declarations: [
    NetworkCreationComponent,
    ViewContainerDirective,
    ScenariosComponent,
    ResultsComponent,
    ResultsMapComponent,
    OurSpiderChartComponent,
    SimulationCreationComponent,
    ExistingSimulationsComponent,
    MlModuleComponent,
    RecommenderPageComponent,
    EditNetworkMapComponent,
  ],
  providers: [
    {
      provide: InjectableRxStompConfig, 
      useClass: MyRxStompConfig, 
      deps: [ConfigService] 
    },
    {
      provide: RxStompService,
      useFactory: rxStompServiceFactory,
      deps: [InjectableRxStompConfig],
    },
  ],
})
export class SimulationWizardModule {}
