import { HttpClient } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BehaviorSubject, Observable, of } from "rxjs";
import { map, tap } from "rxjs/operators";
import { City } from "../data/city";
import { Network } from "../data/network";
import { Scenario } from "../data/scenario";
import { tapDebugApi } from "./tap-debug";
import { ConfigService } from "@ngx-config/core";

@Injectable({ providedIn: "root" })
export class CityService {
  private url: any;
  private selectedCityIndex: number;
  private selectedCity$ = new BehaviorSubject<City | null>(null);
  cities: City[];

  constructor(private httpClient: HttpClient, configService: ConfigService) {
    this.url =
      configService.getSettings("traffic_simulation_base_url") + "/city";
  }

  public setSelectedCity(cityId: string) {
    this.selectedCityIndex = this.cities.findIndex(
      (city) => city.cityId == cityId
    );
    if (this.selectedCityIndex != -1) {
      localStorage.setItem(
        "selectedCityId",
        this.cities[this.selectedCityIndex].cityId
      );
      this.selectedCity$.next(this.cities[this.selectedCityIndex]);
    } else {
      this.selectedCity$.next(null);
    }
  }

  getSelectedCity(): Observable<City> {
    return this.selectedCity$.asObservable();
  }

  getSelectedCityNotBehavior(): Observable<City> {
    return this.httpClient.get<City>(`${this.url}/selected`).pipe(tapDebugApi);
  }

  getCities(): Observable<City[]> {
    return this.httpClient.get<City[]>(this.url).pipe(
      map<any[], City[]>((cities) =>
        cities.map((city) => City.deserialize(city))
      ),
      tapDebugApi
    );
  }

  getCityScenarios(cityId: String): Observable<Scenario[]> {
    return this.httpClient
      .get<Scenario[]>(`${this.url}/${cityId}/scenarios`)
      .pipe(
        map<any[], Scenario[]>((scenarios) =>
          scenarios.map((scenario) => Scenario.deserialize(scenario))
        ),
        tapDebugApi
      );
  }

  getCityNetworks(cityId: String): Observable<Network[]> {
    return this.httpClient
      .get<Network[]>(`${this.url}/${cityId}/networks`)
      .pipe(
        map<any, Network[]>((networks) =>
          networks.map((network) => Network.deserialize(network))
        ),
        tapDebugApi
      );
  }

  public async initialize() {
    localStorage.clear();
    this.cities = await this.getCities().toPromise();
  }
}
