import { Injectable } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import { Observable } from "rxjs";
import { tap } from "rxjs/operators";
import { Simulation } from "../data/simulation";
import { environment } from "../../../../../environments/environment";
import { ConfigService } from "@ngx-config/core";

@Injectable({
  providedIn: "root",
})
export class ComService {
  private traffic_sim_server: any;

  constructor(private http: HttpClient, configService: ConfigService) {
    this.traffic_sim_server =
      configService.getSettings("traffic_simulation_base_url") + "/network";
  }

  sendGetResultsVis(simId: number): Observable<any> {
    const url: string = `${this.traffic_sim_server}/simulation/${simId}/results/vis-trips`;
    return this.http.get(url).pipe(tap(console.debug));
  }

  // Simulation
  getSimulations(scenarioId: number): Observable<Simulation[]> {
    const url: string = `${this.traffic_sim_server}/scenario/simulations/${scenarioId}`;
    return this.http.get<Simulation[]>(url).pipe(tap(console.debug));
  }

  runSimulation(id: number): Observable<any> {
    const url: string = `${this.traffic_sim_server}/simulation/${id}/run`;
    console.debug(url);
    return this.http.get(url).pipe(tap(console.debug));
  }
  // Subpopulation
  createSubpopulation(formData: any) {
    console.log(formData);
    // TODO on backend
  }

  getVehicles() {
    const url: string = `${this.traffic_sim_server}/vehicle`;
    return this.http.get(url);
  }

  getHbefaCategories() {
    const url: string = `${this.traffic_sim_server}/vehicle/hbefaCategories`;
    return this.http.get(url);
  }

  getHbefaFuels(hbefaCategory: string) {
    const url: string = `${this.traffic_sim_server}/vehicle/hbefaFuels/${hbefaCategory}`;
    return this.http.get(url);
  }

  getHbefaSizes(hbefaCategory: string) {
    const url: string = `${this.traffic_sim_server}/vehicle/hbefaSizes/${hbefaCategory}`;
    return this.http.get(url);
  }

  getHbefaEmissionConcepts(hbefaCategory: string) {
    const url: string = `${this.traffic_sim_server}/vehicle/hbefaEmissionConcepts/${hbefaCategory}`;
    return this.http.get(url);
  }
}

export class Population {
  id: number;
  name: string;
  description?: string;
}
