import {
  HttpClient,
  HttpErrorResponse,
  HttpHeaders,
} from "@angular/common/http";
import { Injectable } from "@angular/core";
import { Observable, throwError } from "rxjs";
import { map, catchError } from "rxjs/operators";
import { Network } from "../data/network";
import { tapDebugApi } from "./tap-debug";
import { ConfigService } from "@ngx-config/core";

@Injectable({
  providedIn: "root",
})
export class NetworkService {
  private baseAPIUrl: any;
  private dssUrl: any;

  constructor(private httpClient: HttpClient, configService: ConfigService) {
    this.baseAPIUrl =
      configService.getSettings("traffic_simulation_base_url") + "/network";
    this.dssUrl =
      configService.getSettings("traffic_simulation_dss_url") + "/dss";
  }

  handleError(error: HttpErrorResponse) {
    let errorMessage = "Some sort of error.";
    if (error.error instanceof ErrorEvent) {
      errorMessage = `Error: ${error.error.message}`;
    } else {
      errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
    }
    return throwError(errorMessage);
  }

  createNetwork(network: Network) {
    return this.httpClient.post(this.baseAPIUrl, network).pipe(
      map((network) => Network.deserialize(network)),
      tapDebugApi
    );
  }

  getNetworks(): Observable<Network[]> {
    return this.httpClient.get<Network[]>(this.baseAPIUrl).pipe(
      map<any[], Network[]>((networks) =>
        networks.map((network) => Network.deserialize(network))
      ),
      tapDebugApi,
      catchError(this.handleError)
    );
  }

  getNetwork(id: number): Observable<Network> {
    return this.httpClient
      .get<Network>(`${this.baseAPIUrl}/${id}`)
      .pipe(tapDebugApi, catchError(this.handleError));
  }

  postGenerateBikeLanes(
    geometry: JSON,
    twoLane: boolean,
    laneSeparation: number
  ): Observable<any> {
    const headers = new HttpHeaders().append("Accept", "application/json");
    return this.httpClient
      .post(
        `${this.dssUrl}/generate_bike_lanes/`,
        {
          geometry: geometry,
          twoLane: twoLane,
          laneSeparation: laneSeparation,
        },
        {
          headers: headers,
        }
      )
      .pipe(
        map((res) => {
          return res;
        })
      );
  }

  getMapInfo(): Observable<any> {
    return this.httpClient.get(`${this.dssUrl}/map-info`).pipe(
      map((res) => {
        return res;
      }),
      tapDebugApi,
      catchError(this.handleError)
    );
  }

  getKpiVisInfo(): Observable<any> {
    return this.httpClient.get(`${this.dssUrl}/kpi-vis-info`).pipe(
      map((res) => {
        return res;
      }),
      tapDebugApi,
      catchError(this.handleError)
    );
  }
}
