import { HttpClient, HttpHeaders } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { Observable } from "rxjs";
import { map, tap } from "rxjs/operators";
import { Scenario } from "../data/scenario";
import { Simulation } from "../data/simulation";
import { tapDebugApi } from "./tap-debug";
import { ConfigService } from "@ngx-config/core";

@Injectable({
  providedIn: "root",
})
export class ScenarioService {
  private baseApiUrl: any;

  constructor(private httpClient: HttpClient, configService: ConfigService) {
    this.baseApiUrl =
      configService.getSettings("traffic_simulation_base_url") + "/scenario";
  }

  createScenario(scenario: Scenario) {
    return this.httpClient.post(this.baseApiUrl, scenario).pipe(
      map((scenario) => Scenario.deserialize(scenario)),
      tapDebugApi
    );
  }

  getScenarios(): Observable<Scenario[]> {
    return this.httpClient.get<Scenario[]>(this.baseApiUrl).pipe(
      map<any[], Scenario[]>((scenarios) =>
        scenarios.map((scenario) => Scenario.deserialize(scenario))
      ),
      tapDebugApi
    );
  }

  getScenarioSimulations(id: number): Observable<Simulation[]> {
    return this.httpClient
      .get<Simulation[]>(`${this.baseApiUrl}/${id}/simulations`)
      .pipe(
        map<any[], Simulation[]>((simulations) =>
          simulations.map((simulation) => Simulation.deserialize(simulation))
        ),
        tapDebugApi
      );
  }
}
