import { Component, OnInit } from '@angular/core';
import * as L from 'leaflet';
import { ConfigService } from '@ngx-config/core';
import { PutVoronoiService } from '../services/put-voronoi.service';
import { GetBikemodelsService } from '../services/get-bikemodels.service';
import { GetBikedataService } from '../services/get-bikedata.service';
import { CreateNewbikemodelService } from '../services/create-newbikemodel.service';
import { GetOdmatrixService } from '../services/get-odmatrix.service';
/*import { GetOdmatrixcustomService } from '../services/get-odmatrixcustom.service';*/

@Component({
  selector: 'ngx-bike-analysis',
  templateUrl: './bike-analysis.component.html',
  styleUrls: ['./bike-analysis.component.scss']
})

export class BikeAnalysisComponent implements OnInit {
  constructor(
    private putVoronoiService: PutVoronoiService,
    private getBikemodelsService: GetBikemodelsService,
    private getBikedataService: GetBikedataService,
    private createNewbikemodelService: CreateNewbikemodelService,
    private getOdmatrixService: GetOdmatrixService,
    private configService: ConfigService
  ) {  this.apiURL = configService.getSettings("bike_analysis_api_base_url"); }
  

  private apiURL: any;

  chart_config: any = {}

  private dateS = null;
  private intensidadJA: any = {};
  private temperaturaJA: any = {};
  private calendarInfo: any = {};
  private city: string = "0";
  private map: L.Map;
  
  public lat:number = 0;
  public lon:number = 0;
  public zoom:number = 12;
  private static BIO_LON: number = -2.9356732032001926;
  private static BIO_LAT: number = 43.26201529732467;
  private static AMS_LON: number = 4.895325492686641;
  private static AMS_LAT: number = 52.375147927015995;
  private static HEL_LON: number = 24.925176054721625;
  private static HEL_LAT: number = 60.22470512386342;
  private static MES_LON: number = 15.553426081599502;
  private static MES_LAT: number = 38.18445422457372;

  public nfecha: Date = new Date();
  public ofecha: Date = new Date();
  public sfecha: Date = new Date();
  public fecha: Date = new Date();
  public rfecha: Date = new Date();
  public rDate: Date = new Date();
  public lDate: Date = new Date();

  private areas: any = null;
  private points: any = null;

  private mats: any[];
  private series: any;
  private ts: any[];
  private ele_series: any[];
  private selected_key: any;
  public sfeature_combo: number = -1;
  public calendarRestriction = [];
  public allowedDay = true;
  public slider_max: number = 24;
  public slider_val: number = 0;
  public resMat = null;
  public markers;

  public options = {
    layers: [
      L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' }),
    ],
    zoom: 12,
    center: L.latLng({ lat: this.lat, lng: this.lon }),
  };

  onMapReady(map: L.Map) {
    this.map = map;
    setTimeout(() => { }, 50);
    this.map.createPane("paneLow").style.zIndex = '250'; // between tiles and overlays
    this.map.createPane("paneHigh").style.zIndex = '450'; // between overlays and shadows
  }

  addDays(dateTime, count_days = 0){
    return new Date(new Date(dateTime).setDate(dateTime.getDate() + count_days));
  }

  ngOnInit(): void {
    this.calendarRestriction = [];
    let dd = new Date();
    for(let i=0;i<4;i++){
      dd = this.addDays(dd,1);
      let mS = (dd.getMonth()+1)+'';
      if ((dd.getMonth()+1) < 10){
        mS = '0'+mS;
      }
      let dS = dd.getDate()+'';
      if ( dd.getDate() < 10 ){
        dS = '0'+dS;
      }
      let ll = dd.getFullYear()+"-"+mS+"-"+dS;
      this.calendarRestriction.push(ll);
    }
    console.log('ngOnInit::: calendarRestriction='+this.calendarRestriction);    
    let pilotName = this.configService.getSettings("default_pilot");
    if      ( pilotName === "BILBAO" || pilotName === "URBANITE" ) { this.city = "0"; } 
    else if ( pilotName === "AMSTERDAM" ) { this.city = "1"; }
    else if ( pilotName === "HELSINKI" )  { this.city = "2"; document.getElementById("last_option").innerHTML = "5: HoD, DoW, SaH, MET and Ferry Arrival/Departure"; }
    else if ( pilotName === "MESSINA" )   { this.city = "3"; }       
    this.initMapOptions();
    this.getAreas0();
    this.createCalendar(this.fecha);
  }

  openInstructions() {
    document.getElementById('instructionModal').style.display = 'block';
  }
  
  closeInstructions() {
    document.getElementById('instructionModal').style.display = 'none';
  }

  clickModelTable(e: any) {
    console.log('mtable clicked...');
    this.sfeature_combo = -1;
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById("models_table");
    for (let i = 0; i < ttable.rows.length; i++) {
      let row = ttable.rows[i];
      for (let j = 0; j < row.cells.length; j++) {
        let ccell = row.cells[j];
        ccell.style.backgroundColor = "white";
        ccell.style.border = 'solid';
        ccell.style.borderWidth = '1px';
      }
    }
    let et: HTMLElement = <HTMLElement>e.target;
    let row: HTMLTableRowElement = <HTMLTableRowElement>et.parentNode;
    for (let j = 0; j < row.cells.length; j++) {
      let ccell = row.cells[j];
      ccell.style.backgroundColor = 'rgba(236,206,61,0.4)'; 
      ccell.style.border = 'solid';
      ccell.style.borderWidth = '1px';
      if ( j == 1 ){
        this.sfeature_combo = parseInt(row.cells[j].innerHTML);
      }
    }
    let firstRow = ttable.rows[0];
    for (let j = 0; j < firstRow.cells.length; j++) {
      let ccell = firstRow.cells[j];
      ccell.style.backgroundColor = "white";
      ccell.style.border = 'solid';
      ccell.style.borderWidth = '1px';
    }
    console.log('clickModelTable::: sfeature_combo selected='+this.sfeature_combo);
    this.createCalendar(this.fecha);
  }

  initMapOptions(){
    this.zoom = 14;
    let pointsFile = '/assets/geojsons/bike_stations_bilbao.geojson';
    let areasFile  = '/assets/geojsons/distritos_bilbao.geojson';
    if (this.city === "0") {
      this.lat = BikeAnalysisComponent.BIO_LAT;
      this.lon = BikeAnalysisComponent.BIO_LON;
    }
    else if (this.city === "1") {
      this.lat = BikeAnalysisComponent.AMS_LAT;
      this.lon = BikeAnalysisComponent.AMS_LON;
    }
    else if (this.city === "2") {
      this.lat = BikeAnalysisComponent.HEL_LAT;
      this.lon = BikeAnalysisComponent.HEL_LON;
      this.zoom = 11;
      pointsFile = '/assets/geojsons/bike_stations_helsinki.geojson';
      areasFile = '/assets/geojsons/helsinki_regions.geojson';
      this.zoom = 11;      
    }
    else if (this.city === "3") {
      this.lat = BikeAnalysisComponent.MES_LAT;
      this.lon = BikeAnalysisComponent.MES_LON;
      pointsFile = '/assets/geojsons/points_messina.geojson';
      areasFile = '/assets/geojsons/circoscriziones_messina.geojson';
      this.zoom = 13;
    }
    
    fetch(pointsFile).then(res => res.text()).then(data => {
      document.getElementById('inputVoronoiID').innerHTML = data;
    });
  
    fetch(areasFile).then(res => res.text()).then(data => {
      document.getElementById('inputAreasID').innerHTML = data;
    });  

    this.options = {
      layers: [
        L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '...' }),
      ],
      zoom: this.zoom,
      center: L.latLng({ lat: this.lat, lng: this.lon }),
    };    
  }
  
  getBikeData0() {
    console.log('bike-analysis.component.ts.getBikeData0::: before calling getBikeData');
    this.getBikedataService.getValues().subscribe(res => this.getBikeData(res), err => console.log(err));
  }

  getBikeData(bikeData: any[]) {
    let iniTs = bikeData[0][0];
    var ini = new Date(iniTs);
    let endTs = bikeData[bikeData.length - 1][0];
    var end = new Date(endTs);

    console.log('ini=' + ini);
    console.log('end=' + end);
    this.printBikeData(bikeData);
  }

  printBikeData(data: any[]) {
    console.log("bike_data.js.printBikeData:::  data.length=" + data.length);
    console.log("bike_data.js.printBikeData:::    data[0][0] =" + new Date(data[0][0]));
    console.log("bike_data.js.printBikeData::: data[data.length-1][0] =" + new Date(data[data.length - 1][0]));
    let xs = [];
    let ys = [];
    let bikes = [];
    let temps = [];
    let precs = [];
    let max_x = data[data.length - 1][0];
    let min_x = data[0][0];
    for (let k = 0; k < data.length; k++) {
      xs.push(data[k][0]);
      ys.push(data[k][1]);
      bikes.push({ x: data[k][0], y: data[k][1] });
    }
    for (let k = 0; k < this.temperaturaJA.length; k++) {
      if (this.temperaturaJA[k][0] > min_x && this.temperaturaJA[k][0] < max_x) {
        temps.push({ x: this.temperaturaJA[k][0], y: this.temperaturaJA[k][1] });
      }
    }
    for (let k = 0; k < this.intensidadJA.length; k++) {
      if (this.intensidadJA[k][0] > min_x && this.intensidadJA[k][0] < max_x) {
        precs.push({ x: this.intensidadJA[k][0], y: this.intensidadJA[k][1] });
      }
    }

    this.chart_config = {
      type: 'line',
      data: {
        labels: xs,
        datasets: [
          {
            label: 'Bike Trips',
            backgroundColor: 'rgb(0, 0, 255)',
            type: 'line',
            borderColor: 'rgb(0, 0, 255)',
            borderWidth: 0.1,
            pointRadius: 0,
            yAxisID: 'y-axis-1',
            data: bikes
          }]
      },
      options: {
        responsive: true,
        maintainAspectRatio: false,
        scales: {
          yAxes: [
            {
              id: 'y-axis-1',
              scaleLabel: {
                display: true,
                labelString: '# Trips'
              },
              type: 'linear',
              display: true,
              position: 'right',
              gridlines: { display: true },
              labels: { show: true },
              ticks: { beginAtZero: true }
            }],
          xAxes: [{
            type: 'time',
            time: {
              max: data[data.length - 1][0],
              min: data[0][0],
              // unit: 'millisecond',
              displayFormats: {
                'millisecond': 'DD-MM-YY',
                'second': 'DD-MM-YY',
                'minute': 'DD-MM-YY',
                'hour': 'DD-MM-YY',
                'day': 'DD-MM-YY',
                'week': 'DD-MM-YY',
                'month': 'DD-MM-YY',
                'quarter': 'DD-MM-YY',
                'year': 'DD-MM-YY'
              }
            }
          }]
        },
        pan: {
          enabled: true,
          mode: 'x',
          speed: 1
        },
        zoom: {
          enabled: true,
          mode: 'x',
          speed: 100
        }
      }
    };
  }

  evaluate() {
    // this.getValues()
  }

  ////////////////////////////////////////////////////////////////////////
  // calendar
  ////////////////////////////////////////////////////////////////////////
  monthToWord(val: number) {
    var lmonth = [];
    lmonth[0] = "January";
    lmonth[1] = "February";
    lmonth[2] = "March";
    lmonth[3] = "April";
    lmonth[4] = "May";
    lmonth[5] = "June";
    lmonth[6] = "July";
    lmonth[7] = "August";
    lmonth[8] = "September";
    lmonth[9] = "October";
    lmonth[10] = "November";
    lmonth[11] = "December";
    return lmonth[val];
  }

  populateYearCB(yyear: number) {
    var mItem: HTMLOptionElement = <HTMLOptionElement>document.createElement("option");
    let ySel: HTMLSelectElement = <HTMLSelectElement>document.getElementById("yCBI");
    mItem.text = "2016";
    mItem.value = 2016 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2017";
    mItem.value = 2017 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2018";
    mItem.value = 2018 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2019";
    mItem.value = 2019 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2020";
    mItem.value = 2020 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2021";
    mItem.value = 2021 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2022";
    mItem.value = 2022 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2023";
    mItem.value = 2023 + "";
    ySel.options.add(mItem);
    var idx = 0;
    if (yyear === 2016) { idx = 0; }
    else if (yyear === 2017) { idx = 1; }
    else if (yyear === 2018) { idx = 2; }
    else if (yyear === 2019) { idx = 3; }
    else if (yyear === 2020) { idx = 4; }
    else if (yyear === 2021) { idx = 5; }
    else if (yyear === 2022) { idx = 6; }
    else if (yyear === 2023) { idx = 7; }

    ySel.selectedIndex = idx;
  }

  populateMonthCB(yyear: number, monthI: number) {
    var mItem: HTMLOptionElement = <HTMLOptionElement>document.createElement("option");
    let mSel: HTMLSelectElement = <HTMLSelectElement>document.getElementById("mCBI");
    mItem.text = "January";
    mItem.value = 0 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "February";
    mItem.value = 1 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "March";
    mItem.value = 2 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "April";
    mItem.value = 3 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "May";
    mItem.value = 4 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "June";
    mItem.value = 5 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "July";
    mItem.value = 6 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "August";
    mItem.value = 7 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "September";
    mItem.value = 8 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "October";
    mItem.value = 9 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "November";
    mItem.value = 10 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "December";
    mItem.value = 11 + "";
    mSel.options.add(mItem);
    mSel.selectedIndex = monthI - 1;
  }

  findMaxDaysInMonth(ofecha: Date) {
    var temp: Date = new Date(ofecha);
    var nowMonth = ofecha.getMonth();
    var lastDay = temp.getDate();
    for (var j = 1; j < 32; j++) {
      temp.setDate(temp.getDate() + 1);
      var lMonth = temp.getMonth();
      if (lMonth !== nowMonth) { break; }
      lastDay = temp.getDate();
    }
    return lastDay + 1;
  }

  findNumOfEmptyDays(ofecha: Date) {
    var temp = new Date(ofecha);
    temp.setDate(1);
    var out = temp.getDay() - 1;
    if (temp.getDay() === 0) out = 6;
    return out;
  }

  createCalendar(local_fecha: Date) {
    this.allowedDay = true;  
    console.log('calendar.js.createCalendar::: local_fecha=' + local_fecha);
    this.sfecha = local_fecha;
    console.log('calendar.js.createCalendar::: calendarRestriction='+this.calendarRestriction);
    document.getElementById("calendar").innerHTML = '';
    var btn: HTMLDivElement = <HTMLDivElement>document.createElement("div");
    btn.style.boxSizing = "border-box";
    btn.style.fontFamily = "Verdana, sans-serinf";
    //btn.className        = "month";
    //btn.style.position   = 'absolute';
    btn.style.width = "100%";
    btn.style.background = '#cfb391';
    btn.style.border = '0px';
    btn.style.boxSizing = 'border-box';
    btn.style.marginTop = '-16px';
    var monthI       = local_fecha.getMonth() + 1;
    var maxDaysMonth = this.findMaxDaysInMonth(local_fecha);
    var edays        = this.findNumOfEmptyDays(local_fecha);
    var temp: Date   = new Date(local_fecha);
    this.lDate       = new Date(temp.setMonth(temp.getMonth() - 1));
    temp             = new Date(local_fecha);
    this.rDate       = new Date(temp.setMonth(temp.getMonth() + 1));
    var year         = local_fecha.getFullYear();
    btn.innerHTML    = "<ul id=\"parentUL\" style=\"height:100%;box-sizing:border-box;display:block;text-align:center\"><li id=\"prev\" style=\"padding-top:3px;list-style:none;float:left;\" class=\"prev\">&#10094;</li><li id=\"next\" style=\"list-style:none;float:right;padding-top:3px\" class=\"next\">&#10095;</li>";
    btn.innerHTML    = btn.innerHTML + "<li style=\"text-align:center;display:block\"><select style=\"margin-left:-40px;\" id=\"mCBI\"></select><br><span style=\"font-size:18;px\"><select id=\"yCBI\"></select></span> </li></ul>";
    // week
    var weekBtn: HTMLUListElement = <HTMLUListElement>document.createElement("ul");
    weekBtn.className = "weekdays";
    weekBtn.style.color = 'rgb(102,102,102)';
    weekBtn.style.backgroundColor = '#ddd';
    weekBtn.style.margin = '0px';
    weekBtn.style.padding = '0px';
    weekBtn.style.fontSize = '13px';
    weekBtn.style.listStyle = 'disc';
    weekBtn.style.verticalAlign = 'baseline';
    weekBtn.style.boxSizing = 'border-box';
    weekBtn.style.paddingInlineStart = '40px';
    weekBtn.style.paddingInlineEnd = '40px';
    weekBtn.style.paddingLeft = '58px';
    weekBtn.innerHTML = "  <li style=\"display:inline-block;text-transform:uppercase;color:#666;width: 14.1%;\">Mo</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Tu</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.6%;\">We</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Th</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Fr</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Sa</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Su</li>";
    // weekBtn.style.display = "inline-block";
    weekBtn.style.color = "#666";
    btn.appendChild(weekBtn);
    // days
    var daysBtn: HTMLUListElement = <HTMLUListElement>document.createElement("ul");
    daysBtn.className = "days";
    daysBtn.style.padding = "2px";
    daysBtn.style.display = 'block';
    daysBtn.style.verticalAlign = 'baseline';
    daysBtn.style.fontSize = '13px';
    daysBtn.style.listStyleType = "none";
    daysBtn.style.color = "#58595b";
    daysBtn.style.paddingInlineStart = '40px';
    daysBtn.innerHTML = "";
    daysBtn.id = "daysUL";
    daysBtn.style.width = '100%'
    var llimit = 32;
    for (var i = 0; i < edays; i++) {
      var dayBtn = document.createElement("li");
      dayBtn.style.width = '12.6%';
      dayBtn.style.listStyleType = "none";
      dayBtn.style.color = "#777";
      dayBtn.style.marginBottom = '5px';
      dayBtn.style.fontSize = '100%';
      dayBtn.style.margin = '0';
      dayBtn.style.outline = '0';
      dayBtn.style.boxSizing = 'border-box';
      dayBtn.style.lineHeight = '1.35';
      dayBtn.style.listStyleType = "none";
      dayBtn.style.display = "inline-block";
      dayBtn.style.textAlign = "center";
      dayBtn.style.listStyle = 'disc';
      dayBtn.style.verticalAlign = 'baseline';
      dayBtn.id = -i + "";
      daysBtn.append(dayBtn);
    }
    for (var i = 1; i < maxDaysMonth; i++) {
      var dayBtn = document.createElement("li");
      dayBtn.style.width = '12.6%'
      dayBtn.style.listStyleType = "none";
      dayBtn.style.color = "#777";
      dayBtn.style.marginBottom = '5px';
      dayBtn.style.fontSize = '100%';
      dayBtn.style.margin = '0';
      dayBtn.style.outline = '0';
      dayBtn.style.boxSizing = 'border-box';
      dayBtn.style.lineHeight = '1.35';
      dayBtn.style.listStyleType = "none";
      dayBtn.style.display = "inline-block";
      dayBtn.style.textAlign = "center";
      dayBtn.style.listStyle = 'disc';
      dayBtn.style.verticalAlign = 'baseline';
      dayBtn.id = i + "";
      var mI = monthI + "";
      if (monthI < 10) mI = '0' + monthI;
      var iI = i + '';
      if (i < 10) iI = '0' + i;
      var dd = year + "-" + mI + "-" + iI;

      if (this.calendarInfo !== null) {
        if (dd in this.calendarInfo) {

          if (this.calendarInfo[dd]["lect"] === 0) {
            dayBtn.style.backgroundColor = "#0f0";
          }
          if (this.calendarInfo[dd]["fest"] === 1) {
            dayBtn.style.backgroundColor = "#f00";
          }
          else if (this.calendarInfo[dd]["fest"] === 2) {
            dayBtn.style.backgroundColor = "#f0f";
          }
          else if (this.calendarInfo[dd]["fest"] === 3) {
            dayBtn.style.backgroundColor = "#ff0";
          }
        } else {

        }
      }
      if ( this.sfeature_combo > 2 ){
        if ( this.calendarRestriction !== null ){
          if ( this.calendarRestriction.includes(dd) ){
            dayBtn.style.background = "rgb(238,238,238)";
          }else{
            daysBtn.style.background = "rgb(70,70,70)";
          }
        }
      }else{
        dayBtn.style.background = "rgb(238,238,238)";
      }
      if (i === this.sfecha.getDate() && (monthI - 1) === this.sfecha.getMonth() && year === this.sfecha.getFullYear()) {
        let sel_color = 'red';
        console.log('dd='+dd);
        if ( this.sfeature_combo > 2 ){
          if ( this.calendarRestriction !== null ){
            if ( !this.calendarRestriction.includes(dd) ){
              sel_color = 'black';
              this.allowedDay = false;
            }
          }
        }
        dayBtn.style.padding = '0px';
        dayBtn.style.border = 'solid';
        dayBtn.style.borderWidth = '2px';
        dayBtn.style.borderColor = sel_color;
        dayBtn.style.color = 'blue !important';
        dayBtn.className = "active";
        var sp: HTMLSpanElement = <HTMLSpanElement>document.createElement("span");
        sp.className = "active";
        sp.innerHTML = i + "";
        dayBtn.appendChild(sp);

      } else {
        dayBtn.innerHTML = i + "";
      }
      daysBtn.append(dayBtn);
    }
    btn.appendChild(daysBtn);
    document.getElementById("calendar").appendChild(btn);
    document.getElementById("parentUL")['component'] = this;
    document.getElementById("parentUL").addEventListener("click", function (e) {
      let targ: HTMLLIElement = <HTMLLIElement>e.target;
      if (targ && targ.nodeName === "LI") {
        if (targ.id === "prev") {
          this['component'].createCalendar(this['component'].lDate);
        }
        else if (targ.id === "next") {
          this['component'].createCalendar(this['component'].rDate);
        }
      }
    });

    document.getElementById("daysUL")['component'] = this;
    document.getElementById("daysUL").addEventListener("click", function (e) {
      let targ: HTMLLIElement = <HTMLLIElement>e.target;
      if (targ && targ.nodeName === "LI") {
        if (+targ.id > 0) {
          this['component'].sfecha = new Date(local_fecha.setDate(+targ.id));
          //changeChartZoomAndDate();
          this['component'].createCalendar(local_fecha);
        }
      }
    });
    this.populateMonthCB(year, monthI);
    this.populateYearCB(year);
    console.log('traffic-prediction.component.ts.createCalendar::: this.sfecha='+this.sfecha+" feature_combo="+this.sfeature_combo+' this.allowedFecha='+this.allowedDay);
  }

  lformatDate(local_date: Date) {
    //var d = new Date(date),
    let month: string = '' + (local_date.getMonth() + 1);
    let day: string = '' + local_date.getDate();
    let year: string = '' + local_date.getFullYear();

    if (month.length < 2)
      month = '0' + month;
    if (day.length < 2)
      day = '0' + day;
    let out = [year, month, day].join('-');
    return out;
  }

  lformatDateTime(datetime: Date) {
    //var d = new Date(date),
    let month = '' + (datetime.getMonth() + 1);
    let day = '' + datetime.getDate();
    let year = datetime.getFullYear();

    if (month.length < 2)
      month = '0' + month;
    if (day.length < 2)
      day = '0' + day;
    let out = [year, month, day].join('-');
    //console.log('espiras.js.lformatDate::: in='+date+" out="+out);

    out = out + " " +
      ("0" + datetime.getUTCHours()).slice(-2) + ":" +
      ("0" + datetime.getUTCMinutes()).slice(-2);
    return out;
  }

  ///////////////////////////////////////////////////////////////////////////
  openNewModelModal() {
    document.getElementById('createModelModal').style.display = 'block';
  }
  closeNewModelModal(res: any) {
    console.log('closeNewModelModal:::' + res);
    document.getElementById('createModelModal').style.display = 'none';
  }

  createNewModel() {
    console.log('bikes.js.createNewModel()::: in function');
    let areasS: HTMLSelectElement = <HTMLSelectElement>document.getElementById('areasNM');
    let areas = areasS.value;
    let feature_comboS: HTMLSelectElement = <HTMLSelectElement>document.getElementById('numNM');
    let feature_combo: number = Number(feature_comboS.value);
    console.log('bikes.js.createNewModel():::\t         areas=\t' + areas);
    console.log('bikes.js.createNewModel():::\t feature_combo=\t' + feature_combo);
    this.createNewbikemodelService.createModel(areas,feature_combo).subscribe(res => this.closeNewModelModal(res), err => console.log(err));
  }
  odMatrix(period: number) {
    if ( !this.allowedDay ){
      document.getElementById('wrongDay').style.display = 'block';
    }else{  
      if ( period == 2 &&  ( this.lformatDate(this.sfecha) != this.calendarRestriction[0] ) ){
        this.sfecha = new Date(this.calendarRestriction[0]);
        this.createCalendar(this.sfecha);
        document.getElementById('correctDay').style.display = 'block';
      }      
      document.getElementById('container_1').innerHTML = "";
      console.log('bikes.ts.odMatrix::: period=' + period);
      let model_id = this.getSelectedModel();
      let feature_combo = this.getSelectedNum();
      let areas = this.getSelectedAreas();
      let dateS = this.lformatDate(this.sfecha);
      console.log("odMatrix::: model_id=" + model_id);
      console.log("odMatrix::: areas=" + areas);
      console.log("odMatrix::: feature_combo=" + feature_combo);
      this.getOdmatrixService.getMatrix(feature_combo, areas, model_id, dateS, period).
        subscribe(res => this.printODMatrix(res), err => console.log(err));
    }
  }

  closeWrongDay(){
    document.getElementById('wrongDay').style.display = 'none';
  }

  closeCorrectDay(){
    document.getElementById('correctDay').style.display = 'none';
  }
  
  odMatrixCustom() {
    console.log('660:bikes.ts:odMatrixCustom::: ');
  }

  getBikeModels0() {
    this.getBikemodelsService.getValues().subscribe(res => this.dataToModelsTable(res), err => console.log(err));
    this.getBikeData0();
    this.createCalendar(this.fecha);
  }

  getAreas0() {
    this.getBikemodelsService.getAreas().subscribe(res => this.dataToCombo(res), err => console.log(err));
    this.getBikeModels0();
  }

  dataToCombo(data){
    let sel:HTMLSelectElement = <HTMLSelectElement>  document.getElementById('areasNM');
    let L = sel.options.length - 1;
    for(let i = L; i >= 0; i--) {
       sel.remove(i);
    }
    for(let ele in data){
      //console.log(JSON.stringify(data[ele]));
      let jj = data[ele]['geojson'];
      var opt = document.createElement('option');
      opt.value = jj['name'];
      opt.innerHTML = jj['name'];
      sel.appendChild(opt);
    }
  }

  printODMatrix(data: any) {
    console.log('666:bikes.ts:printODMatrix::: data=' + JSON.stringify(data));
    this.resMat = data;
    this.mats = data['result']['instants'];
    this.series = data['result']['series'];
    this.ts = data['ts'];
    let ini = new Date(this.ts[0]);
    let end = new Date(this.ts[this.ts.length - 1]);
    this.printResultODMatrix();
  }

  printResultODMatrix() {
    this.chart_config = {}
    this.slider_max = (this.mats.length - 1);
    this.slider_val = 0;
    this.showMat(0);
  }

  showMat(val: number) {
    let container: HTMLDivElement = <HTMLDivElement>document.getElementById('container_1');
    let dateLabel: HTMLLabelElement = <HTMLLabelElement>document.getElementById('dateLabel');
    let mat = this.mats[val];
    console.log("mat="+JSON.stringify(mat));
    let datetimeS = this.lformatDateTime(new Date(this.ts[val]));
    dateLabel.innerHTML = datetimeS;

    let mm = {};
    let uu = {};
    let ll = {};
    let ss = {};
    let num_areas = Math.sqrt(Object.keys(mat).length);
    let labels = [];
    for (let k = 0; k < num_areas; k++) {
      labels.push(k + 1);
    }
    for (let i = 0; i < num_areas; i++) {
      let o = i+1;
      mm[o] = [];
      uu[o] = [];
      ll[o] = [];
      ss[o] = [];
      for(let j = 0;j< num_areas;j++){
        let d = j+1;
        mm[o][d] = 0;
        uu[o][d] = 0;
        ll[o][d] = 0;
        ss[o][d] = 0;
      }
    }
    for (let i = 0; i < num_areas; i++) {
      for(let j = 0;j< num_areas;j++){
        let o = i+1;
        let d = j+1;
        let key = (i+1)+'#'+(j+1);
        mm[o][d] = mat[key]['y'].toFixed(2);
        uu[o][d] = mat[key]['yup'].toFixed(2);
        ll[o][d] = mat[key]['ylow'].toFixed(2);
        ss[o][d] = 0;
      }
    }
    let table = this.buildTable(labels, mm, uu, ll, ss);
    container.innerHTML = "";
    container.appendChild(table);
    if (this.selected_key !== 'undefined') {
      let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('odTable');
      for (let i = 0; i < ttable.rows.length; i++) {
        let row: HTMLTableRowElement = <HTMLTableRowElement>ttable.rows[i];
        for (let j = 0; j < row.cells.length; j++) {
          let ccell = row.cells[j];
          ccell.style.backgroundColor = "white";
          if (ccell.id === this.selected_key) {
            ccell.style.backgroundColor = '#ECCE3D66';
          }
          //ccell.style.borderWidth = '1px';
          //ccell.style.border      = 'solid';
        }
      }
      ttable['component'] = this;
    }

    document.querySelectorAll('#odTable td').forEach(e => e.addEventListener("click", function () {
      console.log("clicked");
      let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('odTable');
      let component = ttable['component'];
      for (let i = 0; i < ttable.rows.length; i++) {
        let row = ttable.rows[i];
        for (let j = 0; j < row.cells.length; j++) {
          let ccell = row.cells[j];
          ccell.style.backgroundColor = "white";
          //ccell.style.borderWidth = '1px';
          //ccell.style.border      = 'solid';          
        }
      }
      if (this.id.includes("#")) {
        let o = this.id.split("#")[0];
        let d = this.id.split("#")[1];
        //console.log("clicked::: " + o + "-" + d);
        this.style.backgroundColor = '#ECCE3D66';//'#FED8B1';
        component.selected_key = o + '#' + d; 
        component.chartMatrix();
      }
    }));
  }

  buildTable(labels, objects, uu, ll, ss) {
    let table = document.createElement('table');
    let thead = document.createElement('thead');
    let tbody = document.createElement('tbody');

    let theadTr = document.createElement('tr');
    let theadTh = document.createElement('th');
    theadTh.innerHTML = "O&#92;D";
    theadTr.appendChild(theadTh);
    for (let k = 0; k < labels.length; k++) {
      let theadTh = document.createElement('th');
      let d = labels[k];
      theadTh.innerHTML = d;
      theadTh.style.border = 'solid';
      theadTh.style.borderWidth = '1px';
      theadTr.appendChild(theadTh);
    }
    thead.appendChild(theadTr);
    table.appendChild(thead);
    for (let j = 0; j < labels.length; j++) {
      let tbodyTr = document.createElement('tr');
      let theadTh = document.createElement('th');
      theadTh.style.background = '#eaeaea';
      theadTh.style.border = 'solid';
      theadTh.style.borderWidth = '1px';
      let o = labels[j];
      theadTh.innerHTML = o;

      tbodyTr.appendChild(theadTh);
      for (let k = 0; k < labels.length; k++) {
        let d = labels[k];
        let tbodyTd = document.createElement('td');
        tbodyTd.innerHTML = "<div style='position:relative;bottom:-15px'><small>" + objects[o][d] + "</small>" + " <p style='font-size:8px'>[" + ll[o][d] + "," + uu[o][d] + "]</p></div>";
        tbodyTd.id = o + "#" + d;
        tbodyTd.style.border = 'solid';
        tbodyTd.style.borderWidth = '1px';
        tbodyTd.style.lineHeight = '0';
        tbodyTd.style.verticalAlign = 'bottom';
        tbodyTr.appendChild(tbodyTd);

      }
      tbody.appendChild(tbodyTr);
    }
    table.appendChild(tbody);
    table.id = 'odTable';
    return table;
  }

  chartMatrix() {
    console.log("odmatrix.js.chartMatrix::: data.length =" + this.mats.length);
    console.log("odmatrix.js.chartMatrix:::       min_t =" + new Date(this.ts[0] * 1000));
    console.log("odmatrix.js.chartMatrix:::       max_t =" + new Date(this.ts[this.ts.length - 1] * 1000));
    console.log('odmatrix.js.chartMatrix:::         key =' + this.selected_key);
    this.ele_series = this.series[this.selected_key];
    console.log('odmatrix.js.chartMatrix::: ele_series='+JSON.stringify(this.ele_series));
    console.log('odmatrix.js.chartMatrix::: ele_series.length=' + this.ele_series.length);
    console.log('odmatrix.js.chartMatrix:::         ts.length=' + this.ts.length);
    
    let chart_data:any = {
      labels: this.ts,
      datasets: [
        {
          label: 'Pred',
          backgroundColor: 'rgb(255, 165, 0,0.1)',
          type: 'line',
          fill: false,
          borderColor: 'rgb(255, 0, 0,1)',
          borderWidth: 1,
          pointRadius: 0,
          lineTension: 0,
          yAxisID: 'y-axis-1',
          data: this.ele_series['ys']  
        },
        {
          label: "BandTop",
          type: "line",
          backgroundColor: "rgb(255, 165, 0,0.3)",
          borderColor: "transparent",
          pointRadius: 0,
          legend: { display: false },
          fill: 0,
          lineTension: 0,
          tension: 0,
          yAxisID: 'y-axis-1',
          data: this.ele_series['yups']  
        },
        {
          label: "BandBottom",
          type: "line",
          backgroundColor: "rgb(255, 165, 0,0.3)",
          borderColor: "transparent",
          pointRadius: 0,
          lineTension: 0,
          legend: { display: false },
          fill: 0,
          tension: 0,
          yAxisID: 'y-axis-1',
          data: this.ele_series['ydowns'] 
        }]
    };
    
    this.chart_config = {
      data: chart_data, 
      options: {
        responsive: true,
        maintainAspectRatio: false,         
        legend: {
          labels: {
            filter: function (legendItem, chartData) {
              if (legendItem.datasetIndex == 1 || legendItem.datasetIndex == 2) {
                return false;
              } else {
                return true;
              }
            }
          }
        },
        scales: {
          xAxes: [{
            type: 'time',
            min: this.ts[0],
            max: this.ts[this.ts.length-1],
            barPercentage: 1.0,
            time: {
              displayFormats: {
                'hour': 'HH:mm DD-MM',
                'day': 'HH:mm DD-MM'
              }
            },
            ticks:{ 
              maxTicksLimit: 12,
              autoSkip: true, 
            }
          }],          
          yAxes: [
            {
              id: 'y-axis-1',
              scaleLabel: {
                display: true,
                labelString: '# Trips'
              },
              type: 'linear',
              display: true,
              position: 'right',
              gridlines: { display: true },
              labels: { show: true },
              ticks: { min:0, beginAtZero: true },
              min: 0,
            }
          ],
          y:{ min:0 }
        },
        plugins:{
          zoom: {
            pan: {
              enabled: true,
              mode: 'x',
            },
            zoom: {
              enabled: true,
              drag: false,
              mode: 'x',
            }
          }
        },
      }
    };
  };

  dataToModelsTable(data: any[]) {
    console.log('espiras.js.dataToModelsTable::: models=' + JSON.stringify(data));
    var ttable: HTMLTableElement = <HTMLTableElement>document.getElementById("models_table");
    ttable.innerHTML = "<tr><th style=\"border:solid;border-width:1px\"><b>Model</b></th>\n\
                            <th style=\"border:solid;border-width:1px\"><b>#</b></th>\n\
                            <th style=\"border:solid;border-width:1px\"><b>Areas</b></th>\n\
                            <th style=\"border:solid;border-width:1px\"><b>Status</b></th>\n\
                            <th style=\"border:solid;border-width:1px\"><b>Score</b></th></tr>";
                      
    for (let k = 0; k < data.length; k++) {
      let ele = data[k];
      let row: HTMLTableRowElement = <HTMLTableRowElement>ttable.insertRow(ttable.rows.length);
      let cell0: HTMLTableCellElement = <HTMLTableCellElement>row.insertCell(0);
      let cell1: HTMLTableCellElement = <HTMLTableCellElement>row.insertCell(1);
      let cell2: HTMLTableCellElement = <HTMLTableCellElement>row.insertCell(2);
      let cell3: HTMLTableCellElement = <HTMLTableCellElement>row.insertCell(3);
      let cell4: HTMLTableCellElement = <HTMLTableCellElement>row.insertCell(4);
      cell0.innerHTML = ele['model_id'];
      cell0.style.border = 'solid';
      cell0.style.borderWidth = '1px';
      cell1.innerHTML = ele['feature_combo'];
      cell1.style.border = 'solid';
      cell1.style.borderWidth = '1px';
      cell2.innerHTML = ele['areas'];
      cell2.style.border = 'solid';
      cell2.style.borderWidth = '1px';
      cell3.innerHTML = ele['status'];
      cell3.style.border = 'solid';
      cell3.style.borderWidth = '1px';
      cell4.innerHTML = ele['performance'];
      cell4.style.border = 'solid';
      cell4.style.borderWidth = '1px';
    }
  }

  /////////////////////////////////////////////////
  openLoadPointsModal() {
    document.getElementById('loadPointsModal').style.display = 'block';
  }
  openLoadAreasModal() {
    document.getElementById('loadAreasModal').style.display = 'block';
  }

  closeCustomODMatrixModal() {
    document.getElementById('customOdMatrixModal').style.display = 'none';
  }
  closeLoadPointsModal() {
    document.getElementById('loadPointsModal').style.display = 'none';
  }
  /////////////////////////////////////////////////
  // Voronoi
  /////////////////////////////////////////////////
  style(feature) {
    return {
      fillColor: '#E31A1C',
      weight: 2,
      opacity: 1,
      color: 'white',
      dashArray: '3',
      fillOpacity: 0.7
    };
  }

  loadAreas() {

    if (this.areas !== undefined && this.areas !== null) {
      this.map.removeLayer(this.areas);
    }
    if (this.points !== undefined && this.points !== null) {
      this.map.removeLayer(this.points);
    }
    let ttext = document.getElementById('inputAreasID').textContent;
    let ttextJSON = JSON.parse(ttext);
    this.markers = [];
    this.areas = L.geoJSON(ttextJSON, {
      style: this.style,
      onEachFeature: function (feature, layer) {
        layer.bindTooltip(feature.properties.zone_id.toString(), { permanent: true });
      }
    }).addTo(this.map);

    document.getElementById('loadAreasModal').style.display = 'none';
  }

  loadPoints() {
    let ttext = document.getElementById('inputVoronoiID').textContent;
    let ttextJSON = JSON.parse(ttext);
    console.log("loadPoints::: ttextJSON=" + JSON.stringify(ttextJSON));
    if (this.areas !== undefined && this.areas !== null) {
      this.map.removeLayer(this.areas);
    }
    if (this.points !== undefined && this.points !== null) {
      this.map.removeLayer(this.points);
    }
    this.markers = [];
    var geojsonMarkerOptions = {
      radius: 3,
      fillColor: "#00f",
      color: "#00f",
      weight: 1,
      opacity: 1,
      fillOpacity: 1,
      zIndex: '10000'
    };
    this.points = L.geoJSON(ttextJSON, {
      pointToLayer: function (feature, latlng) {
        return L.circleMarker(latlng, geojsonMarkerOptions);
      },
      pane: 'paneHigh'
    }).addTo(this.map);

    document.getElementById('loadPointsModal').style.display = 'none';
  }

  createVoronoi() {
    console.log('createVoronoi::: in routine');
    let ttext = document.getElementById('inputVoronoiID').textContent;
    let ttextJSON = JSON.parse(ttext);
    if (this.areas !== undefined && this.areas !== null) {
      this.map.removeLayer(this.areas);
    }
    console.log('createVoronoi before calling services');
    this.putVoronoiService.putPoints(ttextJSON).subscribe(data => this.printVoronoi(JSON.stringify(data)));
  }

  printVoronoi(data: any) {
    console.log('printVoronoi::: data=' + data);
    console.log('bike.ts:::printVoronoi');
    // this.areas = L.geoJSON(JSON.parse(data)['polis']).addTo(this.map);
    this.areas = L.geoJSON(JSON.parse(data)['polis'], {
      pane: "paneLow",
      style: this.style,
      onEachFeature: function (feature, layer) {
        layer.bindTooltip(feature.properties.zone_id.toString(), { permanent: true });
      }
    }).addTo(this.map);

  }

  resetZoom() {
    console.log('resetZoom::: in function');
  }
  
  getSelectedRow() {
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('models_table');
    for (let i = 0; i < ttable.rows.length; i++) {
      let row = ttable.rows[i];
      if (row.cells[0].style.backgroundColor !== 'white') {
        return i;
      }
    }
    return -1;
  }

  getSelectedModel() {
    let sel = this.getSelectedRow();
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('models_table');
    let row = ttable.rows[sel];
    return Number(row.cells[0].innerHTML);
  }

  getSelectedNum() {
    let sel = this.getSelectedRow();
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('models_table');
    let row = ttable.rows[sel];
    return Number(row.cells[1].innerHTML);
  }

  getSelectedAreas() {
    let sel = this.getSelectedRow();
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('models_table');
    let row = ttable.rows[sel];
    return row.cells[2].innerHTML;
  }
}


