import { Component, OnInit } from '@angular/core';
import * as L from 'leaflet';
import { ConfigService } from '@ngx-config/core';
import { Options, LabelType } from '@angular-slider/ngx-slider';
import { BikeAnalysisAMSService } from '../services/bike-analysis_AMS.service';
import { PutGeoJsonService } from '../services/put-geojson.service';
import { LeafletLayerDirective } from '@asymmetrik/ngx-leaflet';
@Component({
  selector: 'ngx-od-from-counts',
  templateUrl: './bike-analysis_AMS.component.html',
  styleUrls: ['./bike-analysis_AMS.component.scss']
})
export class BikeAnalysisAMSComponent implements OnInit {

  constructor(
    private configService: ConfigService,
    private putGeoJSONService: PutGeoJsonService,
    private bikeAnalysisAMSService: BikeAnalysisAMSService) { }

  public R_EARTH: number = 6371e3;
  private map: L.Map;
  public static map2: L.Map;
  private city: string = "0";
  public lat: number = 0;
  public lon: number = 0;
  public zoom: number = 12;
  private static AMS_LON: number = 4.929861; //4.895325492686641;
  private static AMS_LAT: number = 52.400065; //52.375147927015995; 
  public options = {};
  public options2 = {};
  private areas: any = {};
  private areas0: any = {};
  private areas_name: string = "";
  private matrix0: any = {};
  private matrix_name: string = "";
  private num_areas: any = {};
  private area;
  private layerGroup;
  private points: any = {};
  private models: any = {};
  private legend;
  private legend2;
  public static circles = [];
  public toggleMap = false;
  public static marker;
  public static clrs: any = ['#fef5f0', '#fbd7c4', '#f6a585', '#f37048', '#e23218', '#b40b02', '#630000'];
  toggleLbls: boolean = false;
  toggleTTs: boolean = false;
  public sliderValue = 5;
  public static sliderValue2 = 5;
  public static center;
  public static maxRadio:number = 4000;
  public static overlays = {};
  
  sliderOptions:Options = { 
    floor: 0,
    ceil: 10,
    translate: (value: number, label: LabelType): string => {
      switch (label) {
        case LabelType.Low:
          return ''+value;
        case LabelType.High:
          return 10+'';
        default:
          return  ""+value;
      }
    }
  };

  ngOnInit(): void {
    this.city = "1";
    this.getAreas0();
    this.initMapOptions();
    this.initMap2Options();
    (<HTMLInputElement>document.getElementById('newMaxRadio')).value = BikeAnalysisAMSComponent.maxRadio+"";
  }
  changeRadio(){
    let val = (<HTMLInputElement> document.getElementById('newMaxRadio')).value;
    BikeAnalysisAMSComponent.maxRadio = parseInt(val);
  }

  iniCombo(num):void{
    let sel = (document.getElementById("ini_combo") as HTMLSelectElement);
    sel.innerText = null;
    for (let i = 1; i < num+1; i++) {
      let option = document.createElement("option");
      option.text = i + "";
      option.value = i + "";
      sel.add(option);
    }
    let option = document.createElement("option");
    option.text = "Ext";
    option.value = -999 + "";
    sel.add(option);
    sel.selectedIndex = -1;
  }

  initMapOptions() {
    this.zoom = 13;
    if (this.city === "1") {
      this.lat = BikeAnalysisAMSComponent.AMS_LAT;
      this.lon = BikeAnalysisAMSComponent.AMS_LON;
    }
   // console.log('bikeAnalysis_AMS.initMapOptions::: (lat,lon)='+this.lat+","+this.lon);
    this.options = {
      layers: [
        L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' }),
      ],
      zoom: this.zoom,
      center: L.latLng({ lat: this.lat, lng: this.lon }),
    };
  }

  initMap2Options() {
    this.zoom = 13;
    if (this.city === "1") {
      this.lat = BikeAnalysisAMSComponent.AMS_LAT;
      this.lon = BikeAnalysisAMSComponent.AMS_LON;
    }
   // console.log('bikeAnalysis_AMS.initMapOptions::: (lat,lon)='+this.lat+","+this.lon);
    this.options2 = {
      layers: [
        L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' }),
      ],
      zoom: this.zoom,
      center: L.latLng({ lat: this.lat, lng: this.lon }),
    };
  }

  onMapReady(map: L.Map) {
    this.map = map;
    setTimeout(() => { }, 50);
    this.map.createPane("paneLow").style.zIndex = '250'; // between tiles and overlays
    this.map.createPane("paneHigh").style.zIndex = '450'; // between overlays and shadows
    this.map.on('click', function (evt: any) {
      console.log('latLon=' + evt.latlng);
    });
  }


  onMap2Ready(map: L.Map) {
    BikeAnalysisAMSComponent.map2 = map;
    setTimeout(() => { }, 50);
    BikeAnalysisAMSComponent.map2.createPane("paneLow").style.zIndex = '250';  
    BikeAnalysisAMSComponent.map2.createPane("paneHigh").style.zIndex = '450';  
    BikeAnalysisAMSComponent.map2.on('click', function (evt: any) {
      console.log('latLon=' + evt.latlng);
      if ( BikeAnalysisAMSComponent.marker != null ){ BikeAnalysisAMSComponent.marker.remove(); }

      BikeAnalysisAMSComponent.center = evt.latlng;
      BikeAnalysisAMSComponent.marker = new L.Marker(BikeAnalysisAMSComponent.center);
      BikeAnalysisAMSComponent.marker.addTo(this);
      BikeAnalysisAMSComponent.overlays['marker'] = BikeAnalysisAMSComponent.marker;
      for(let k=0;k<BikeAnalysisAMSComponent.circles.length;k++){
        BikeAnalysisAMSComponent.circles[k].remove();
      }
      BikeAnalysisAMSComponent.circles = [];
      for(let k=0;k<BikeAnalysisAMSComponent.sliderValue2;k++){
        let radio = (k+1)*BikeAnalysisAMSComponent.maxRadio/(BikeAnalysisAMSComponent.sliderValue2*1.0);
        console.log('k='+k+'\tradio='+radio);
        let circle = L.circle(BikeAnalysisAMSComponent.center, radio).addTo( BikeAnalysisAMSComponent.map2);
        circle.addTo(this);
        BikeAnalysisAMSComponent.overlays['circle_'+k] = circle;
        BikeAnalysisAMSComponent.circles.push(circle);
      }
    });
  }

  openLoadAreasModal() {
    document.getElementById('loadAreasModal').style.display = 'block';
  }

  style(feature) {
    return {
      fillColor: '#E31A1C',
      weight: 2,
      opacity: 1,
      color: 'white',
      dashArray: '3',
      fillOpacity: 0.7
    };
  }

  clearAllData() {
    for (var key in BikeAnalysisAMSComponent.overlays) {
      this.map.removeLayer(BikeAnalysisAMSComponent.overlays[key]);
    }
  }

  loadAreas0() {
    let areasText = (<HTMLTextAreaElement>document.getElementById('inputAreasID')).value;
    console.log('bikeAnalysis.loadAreas::: areasText='+areasText);
    let areasJSON = JSON.parse(areasText);
    let areasName = (<HTMLInputElement>document.getElementById("newAreasName")).value;
    let input = {};
    input["areas_id"] = areasName;
    input["areas_json"] = areasJSON;
    this.printAreas(areasJSON);
    document.getElementById('loadAreasModal').style.display = 'none';
    this.bikeAnalysisAMSService.putAreas(input).subscribe(res => this.getAreas0()), err => console.log("loadAreas0::: ERROR: err=" + JSON.stringify(err));
  }

  getAreas0() {
    this.bikeAnalysisAMSService.getAreas().subscribe(res => this.getAreas(res)), err => console.log("getAreas0::: ERROR: err=" + JSON.stringify(err));
  }

  getAreas(res) {
    var select = document.getElementById("areas_combo");
    select.innerHTML = "";
    for (let i = 0; i < res.length; i++) {
      let opt = res[i];
      console.log(res[i]);
      this.areas[opt["areas"]] = opt["geojson"];
      this.num_areas[opt["areas"]] = opt["num_area"];
      let el = document.createElement("option");
      el.textContent = opt["areas"];
      el.value = opt["areas"];
      select.appendChild(el);
    }
    this.getModels0();
  }

  getModels0() {
    this.bikeAnalysisAMSService.getModels().subscribe(res => this.getModels(res)), err => console.log("getModels0::: ERROR: err=" + JSON.stringify(err));
  }

  createModel(){
    let dataset = 'ringring';
    let areas_id = (<HTMLSelectElement>document.getElementById('areas_combo')).value;
    let input = {};
    input['areas_id'] = areas_id;
    input['dataset'] = dataset;
    this.bikeAnalysisAMSService.createModel(input).subscribe(res => this.getAreas0()), err => console.log("createModel::: ERROR: err=" + JSON.stringify(err));

  }

  getModels(res) {
    console.log('bikeAnalysis_AMS.getModels::: res='+JSON.stringify(res));
    var select = document.getElementById("models_combo");
    select.innerHTML = "";
    for (let i = 0; i < res.length; i++) {
      let opt = res[i];
      let size = opt["size"];
      let key = opt["dataset"] + "#" + opt["areas"];
      this.models[key] = opt["result"]
      let el = document.createElement("option");
      el.textContent = opt["dataset"] + "\xA0\xA0("+size+")\xA0\xA0\xA0\xA0\xA0\xA0" + opt["areas"] + "\xA0\xA0[" + opt["status"] + "]";
      el.value = key;
      select.appendChild(el);
    }
  }

  printAreas0() {
    let val = (document.getElementById("areas_combo") as HTMLSelectElement).value;
    this.areas0 = this.areas[val];
    this.areas_name = val;
    this.printPoints(this.areas[val]);
  }

  printPoints(pointsJSON) {
    if (this.points !== undefined && this.points !== null) {
      this.map.removeLayer(this.points);
    }
    var geojsonMarkerOptions = {
      radius: 3,
      fillColor: "#00f",
      color: "#00f",
      weight: 1,
      opacity: 1,
      fillOpacity: 1,
      zIndex: '10000'
    };
    this.points = L.geoJSON(pointsJSON, {
      pointToLayer: function (feature, latlng) {
        return L.circleMarker(latlng, geojsonMarkerOptions);
      },
      pane: 'paneHigh'
    }).addTo(this.map);
    BikeAnalysisAMSComponent.overlays['points'] = this.points;
  }

  printAreas(areasJSON) {
    if (this.layerGroup !== undefined && this.layerGroup !== null) {
      this.map.removeLayer(this.layerGroup);
    }
    this.layerGroup = L.layerGroup().addTo(this.map);
    BikeAnalysisAMSComponent.overlays['lGG'] = this.layerGroup;
    let regions = [...areasJSON.features];
    regions.forEach((region) => {
      L.geoJSON(region).addTo(this.layerGroup);
     
    });

  }

  closeLoadPointsModal() {
    document.getElementById('loadPointsModal').style.display = 'none';
  }

  erase_run() {
    console.log("NOT YET");
  }

  printSolution0(){
    let sol = (document.getElementById("models_combo") as HTMLSelectElement).value;
    this.matrix_name = sol;
    this.matrix0 = this.models[sol];
    console.log("bikeAnalysisAMS.printSolution0::: sol="+sol);
    let areas_id = sol.split("#")[1];
    this.area = this.areas[areas_id];
    console.log("bikeAnalysisAMS.printSolution0::: sol="+sol);
    console.log("bikeAnalysisAMS.printSolution0::: areas_id="+areas_id);
    console.log("bikeAnalysisAMS.printSolution0::: this.num_areas["+areas_id+"]="+this.num_areas[areas_id]);
    this.iniCombo(this.num_areas[areas_id]);
    let iniend_zone_id = (document.getElementById("ini_combo") as HTMLSelectElement).value;
    console.log('bikeAnalysisAMS.printSolution0::: iniend_zone_id='+iniend_zone_id);
    this.printAreas(this.area);
    let res = this.models[sol];
    console.log('bikeAnalysisAMS.printSolution0::: res='+JSON.stringify(res));

  }

  printSolution() {
    let sol = (document.getElementById("models_combo") as HTMLSelectElement).value;
    let iniend_zone_id = (document.getElementById("ini_combo") as HTMLSelectElement).value;
    let res = this.models[sol];
    let vec = {};
    if (this.toggleLbls) {
      for (let k in res){
        let v = res[k][iniend_zone_id];
        vec[k] = v;
      }

    } else {
      vec = res[iniend_zone_id];
    }
    let maxval = 0;
    for (let i in vec){ if  ( vec[i] > maxval ) { maxval = vec[i]; } }
    console.log("printSolution::: maxval="+maxval);
    this.layerGroup.eachLayer((layer) => {
      let zone_id = parseInt((Object.values(layer._layers)[0] as L.Polygon).feature.properties.zone_id);
      let clr = this.fromValueToClr(vec[zone_id], maxval);
      layer.setStyle(() => ({ color: clr, fillColor: clr, fillOpacity: 0.6 }));
      layer.closeTooltip();
      let pnt = new L.Point(0,0);
      if ( this.toggleTTs ){
        pnt = new L.Point(50*(zone_id-1),50*(zone_id-1));

      }
      console.log('pnt='+pnt+' vec['+zone_id+']='+vec[zone_id]);
      layer.bindTooltip("<p><strong>"+zone_id+"</strong><br>val:" + vec[zone_id]+"</p>", { offset: pnt , permanent: true, direction: "right", opacity: 0.7 }).openTooltip();
    });
    let extValue = vec['-999'];
    let extClr = this.fromValueToClr(extValue, maxval);
    if ( this.legend instanceof L.Control ){
      let ddiv = document.getElementById('legend_div');
      ddiv.parentNode.removeChild(ddiv);
      ddiv.innerHTML = '';
      this.legend.remove;
    }    
    this.legend = new L.Control({position: 'bottomleft'});
    this.legend.onAdd = function () {
        var div = L.DomUtil.create('div', 'info legend');
        div.id = 'legend_div';
        div.style.background = '#C0C0C0DD';
        div.style.width = '150px';     
        div.style.paddingTop = '10px';
        div.style.paddingBottom = '20px';
        let labels = [];        
        //div.innerHTML +=
        //        labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:#303030"></div><label style="position:absolute;left:45px;top:12px"> <= 0</label></div>');
        let v0 = 0;
        let delta = maxval/(BikeAnalysisAMSComponent.clrs.length);
        console.log('legend::: maxval='+maxval+' delta='+delta);
        for (let i = 0; i < BikeAnalysisAMSComponent.clrs.length; i++) {
            let vnext = v0 + delta;
            div.innerHTML +=
                labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:' + BikeAnalysisAMSComponent.clrs[i] + '"></div><label style="position:absolute;left:45px;top:' + (12 + 30 * (i)) + 'px">' + v0.toFixed(0) +'-'+vnext.toFixed(0)+ '</label></div>');
            v0 = vnext;
        }
        div.innerHTML = labels.join('<br>');
        return div;         
    };
    this.legend.addTo(this.map);  
    BikeAnalysisAMSComponent.overlays['legend'] = this.legend;

    if ( this.legend2 instanceof L.Control ){
      let ddiv = document.getElementById('legend2_div');
      ddiv.parentNode.removeChild(ddiv);
      ddiv.innerHTML = '';
      this.legend2.remove;
    }    
    //<div class="" style="opacity: 0.7; transform: translate3d(503px, 226px, 0px);"><p><strong>2</strong><br>val:308</p></div>


    this.legend2 = new L.Control({position: 'topright'});
    this.legend2.onAdd = function () {
        var div = L.DomUtil.create('div', 'info legend');
        div.id = 'legend2_div';
        div.style.background = extClr;
        div.style.width = '150px';     
        div.style.paddingTop = '10px';
        div.style.paddingLeft = '10px';
        div.style.paddingBottom = '0px';
        div.style.height = '90px';
        div.style.opacity = '0.7';
        var div2 = L.DomUtil.create('div','leaflet-tooltip leaflet-zoom-animated leaflet-tooltip-center');
        div2.style.opacity = '0.7';
        div2.innerHTML = "<p><strong>Ext</strong><br>val:"+extValue+"</p>";
        div.append(div2);
        return div;         
    };
    this.legend2.addTo(this.map);  
    BikeAnalysisAMSComponent.overlays['legend2'] = this.legend2;
  }

  fromValueToClr(val, maxval) {
    let idx = Math.floor(val / maxval * 6);
    return BikeAnalysisAMSComponent.clrs[idx];
  }

  changeTooltips(){
    this.toggleTTs = !this.toggleTTs;
    this.printSolution();
  }

  oriDestChange() {
    console.log("oriDestChange::: this.toggleLbls=" + this.toggleLbls);
    this.toggleLbls = !this.toggleLbls;
    if (this.toggleLbls) {
      document.getElementById("odLabel").innerHTML = "Des";
    } else {
      document.getElementById("odLabel").innerHTML = "Ori";
    }
  }

  openInstructions() {
    document.getElementById('instructionModal').style.display = 'block';
  }

  closeInstructions() {
    document.getElementById('instructionModal').style.display = 'none';
  }

  mapToggle(){
    console.log('mapToggle::: this.toggleMap='+this.toggleMap);
    this.toggleMap = ! this.toggleMap;
    if ( this.toggleMap ){
      /*
      document.getElementById('inputAreasID').style.width='40%';
      document.getElementById('map2').style.display= 'block';
      document.getElementById('slider').style.display='block';
      document.getElementById('radioInput').style.display='block';*/
      document.getElementById('slider').style.display='block';
      document.getElementById('map2').style.display= 'block';
      document.getElementById('radioInput').style.display='block';
      document.getElementById('inputAreasID').style.display= 'none';
      document.getElementById('toggleMap').style.display='none';
      document.getElementById('areas_name').style.display='none';

      this.initMap2Options();
      BikeAnalysisAMSComponent.map2.invalidateSize();
    }else{
      document.getElementById('inputAreasID').style.width='90%';
      document.getElementById('map2').style.display= 'none';
      document.getElementById('slider').style.display='none';
      document.getElementById('radioInput').style.display='none';
      document.getElementById('inputAreasID').style.display= 'block';
      document.getElementById('toggleMap').style.display='block';   
      document.getElementById('areas_name').style.display='block';   
    }
  }

  slider_change(){
    console.log('sliderValue='+this.sliderValue);
    BikeAnalysisAMSComponent.sliderValue2 = this.sliderValue;
    for(let k=0;k<BikeAnalysisAMSComponent.circles.length;k++){
      BikeAnalysisAMSComponent.circles[k].remove();
    }
    BikeAnalysisAMSComponent.circles = [];
    for(let k=0;k<BikeAnalysisAMSComponent.sliderValue2;k++){
      let radio = (k+1)*BikeAnalysisAMSComponent.maxRadio/(BikeAnalysisAMSComponent.sliderValue2*1.0);
      console.log('k='+k+'\tradio='+radio);
      let circle = L.circle(BikeAnalysisAMSComponent.center, radio).addTo( BikeAnalysisAMSComponent.map2);
      circle.addTo(BikeAnalysisAMSComponent.map2);
      BikeAnalysisAMSComponent.circles.push(circle);
    }
  }

  generateJSON(){
    let geojson = {};
    geojson['type']='FeatureCollection';
    let name = (<HTMLInputElement>document.getElementById('newAreasName')).value;
    geojson['name']=name;
    geojson['crs'] = {};
    geojson['crs']['type'] = 'name';
    geojson['crs']['properties'] = {};
    geojson['crs']['properties']['name'] = 'urn:ogc:def:crs:OGC:1.3:CRS84';
    geojson['features'] = [];
    let valis = [];
    for(let i=0;i<this.sliderValue;i++){
      valis[i] = this.generate_circle_points(i);
    }
    for(let i=0;i<this.sliderValue;i++){
      let feature = {};
      feature['type'] = 'Feature';
      feature['properties'] = {};
      feature['properties']['name'] = 'A_zone_'+i;
      feature['properties']['zone_id'] = i+1;
      feature['geometry'] = {};
      feature['geometry']['type'] = 'Polygon';
      feature['geometry']['coordinates'] = [];
      let vols = [];
      for (let k=0;k<valis[i].length;k++){
        vols.push(valis[i][k]);
      }
      if ( i > 0 ){
          for(let k=valis[i-1].length-1;k>-1;k--){
            vols.push(valis[i-1][k]);
          }
      }
      console.log('vols.length='+vols.length);
      feature['geometry']['coordinates'].push(vols);
      geojson['features'].push(feature);
    }
    this.mapToggle();
    document.getElementById('inputAreasID').textContent = JSON.stringify(geojson);
  }

  generate_circle_points(i:number){
    let center:L.LatLng = BikeAnalysisAMSComponent.center;
    let lat_c = center.lat;
    let lon_c = center.lng;
    let radio = (i+1)*BikeAnalysisAMSComponent.maxRadio/(BikeAnalysisAMSComponent.sliderValue2*1.0);
    let steps = 100;
    let out = [];  
    for (let j = 0; j < steps+1; j++) {
      let degrees = (j/steps)*360.0;
      let radians = (Math.PI/180.0)*degrees
      let lon = (radio/this.R_EARTH) * Math.sin(radians) / Math.cos(lat_c * Math.PI/180.0)*180/Math.PI + lon_c;
      let lat = (radio/this.R_EARTH) * Math.cos(radians)*180/Math.PI + lat_c;
     // let d = this.haversine(lat_c, lon_c, lat, lon);
     // let d2 = this.haversine(lat_c, lon_c, lat, lon);
     // console.log('\t\ti='+i+'\tradio='+radio+'\tj='+j+'\td='+d+'\td2='+d2);
      out.push([lon,lat])
    }
    return out;
  }
/*
  haversine(lat1:number, lon1:number, lat2:number, lon2:number){
    let φ1 = lat1 * Math.PI/180; // φ, λ in radians
    let φ2 = lat2 * Math.PI/180;
    let Δφ = (lat2-lat1) * Math.PI/180;
    let Δλ = (lon2-lon1) * Math.PI/180;
    let a = Math.sin(Δφ/2) * Math.sin(Δφ/2) +
            Math.cos(φ1) * Math.cos(φ2) *
            Math.sin(Δλ/2) * Math.sin(Δλ/2);
    let c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
    let d = this.R_EARTH * c; // in metres
    return d;
  }

  haversine2(lat1:number, lon1:number, lat2:number, lon2:number){
    let φ1 = lat1 * Math.PI/180; // φ, λ in radians
    let φ2 = lat2 * Math.PI/180;
    let Δφ = (lat2-lat1) * Math.PI/180;
    let Δλ = (lon2-lon1) * Math.PI/180;
    let x = Δφ * Math.cos(φ1);
    let y = Δλ
    let c = Math.sqrt( x*x + y*y );
    let d = this.R_EARTH * c; // in metres
    return d;
  }
*/
  download_matrix(){
    let out = this.matrix0;
    const blob = new Blob([JSON.stringify(out)], { type: "text/json" });
    const link = document.createElement("a");
    link.download = this.matrix_name+".geojson";
    link.href = window.URL.createObjectURL(blob);
    link.dataset.downloadurl = ["text/json", link.download, link.href].join(":");
    const evt = new MouseEvent("click", {
      view: window,
      bubbles: true,
      cancelable: true,
    });
    console.log();
    link.dispatchEvent(evt);
    link.remove();
  }

  download_geojson(){
    let out = this.areas0;
    const blob = new Blob([JSON.stringify(out)], { type: "text/json" });
    const link = document.createElement("a");
    link.download = this.areas_name+".geojson";
    link.href = window.URL.createObjectURL(blob);
    link.dataset.downloadurl = ["text/json", link.download, link.href].join(":");
    const evt = new MouseEvent("click", {
      view: window,
      bubbles: true,
      cancelable: true,
    });
    console.log();
    link.dispatchEvent(evt);
    link.remove();
  }

  upload_geojson(){
    console.log("upload_geojson::: clicked");
    var fg = L.featureGroup();
    this.map.eachLayer((layer)=>{
      if(layer instanceof L.Path || layer instanceof L.Marker){
        fg.addLayer(layer);
      }
    });
    console.log(fg.toGeoJSON());
    let geojson = fg.toGeoJSON();
    let dt = new Date();
    let city = 'amsterdam';
    let name = "bike_analysis_"+dt.getTime();
    console.log("upload_geojson::: name="+name);
    this.putGeoJSONService.putGeoJSON(geojson,name,city,name, name).subscribe(
          res => {alert('geojson '+name+' added to storage.'); console.log("putGeoJSONService: SUCESS::: res="+JSON.stringify(res))}, 
          err => {alert('ERROR!!!\ngeojson '+name+' NOT added to storage.');console.log("putGeoJSONService: ERROR::: err="+JSON.stringify(err));}
    );
  }
}
