import { Component, OnInit } from '@angular/core';
import * as L from 'leaflet';
import { ConfigService } from '@ngx-config/core';
import { Options, LabelType } from '@angular-slider/ngx-slider';
import { GetPredictionService } from '../services/get-prediction.service';
//import { ChartjsLineComponent } from '../../charts/chartjs/chartjs-line.component';
import { Chart } from 'chart.js';
@Component({
  selector: 'ngx-gltraffic-prediction',
  templateUrl: './gltraffic-prediction.component.html',
  styleUrls: ['./gltraffic-prediction.component.scss']
})

export class GlTrafficPredictionComponent implements OnInit {
  constructor(
    private getPredictionService: GetPredictionService,
    private configService: ConfigService
  ) {  }

  chart_config: any = {}
  private map: L.Map;
  private city: string = "0";
  public lat:number = 0;
  public lon:number = 0;
  public zoom:number = 12;
  private static BIO_LON: number = -2.9356732032001926;
  private static BIO_LAT: number = 43.26201529732467;
  private static AMS_LON: number =  4.895325492686641;
  private static AMS_LAT: number = 52.375147927015995;
  private static HEL_LON: number = 24.925176054721625;
  private static HEL_LAT: number = 60.22470512386342;
  private static MES_LON: number = 15.553426081599502;
  private static MES_LAT: number = 38.18445422457372;
  private pointsFile = '/assets/geojsons/aforadores_bilbao.json';
  private plys: any = [];
  public sliderValue = 144;
  toggleLbls:boolean = false;

  public sCombo;
  public sWeekday;
  public sHoliday;
  public sSchool;
  public sTemp;
  public sPrecp;
  public sFutbol;
  //public colors = ["#030001","#1C000B","#350016","#4E0021","#68002C","#810037","#9A0042","#B3004E","#CD005A","#E60067","#FF0073"];
  public colors = ["#FF0000",// <-- red
    "#FF1100",
    "#FF2200",
    "#FF3300",
    "#FF4400",
    "#FF5500",
    "#FF6600",
    "#FF7700",
    "#FF8800",
    "#FF9900",
    "#FFAA00",
    "#FFBB00",
    "#FFCC00",
    "#FFDD00",
    "#FFEE00",
    "#FFFF00",// <-- yellow
    "#EEFF00",
    "#DDFF00",
    "#CCFF00",
    "#BBFF00",
    "#AAFF00",
    "#99FF00",
    "#88FF00",
    "#77FF00",
    "#66FF00",
    "#55FF00",
    "#44FF00",
    "#33FF00",
    "#22FF00",
    "#11FF00",
    "#00FF00"]// <-- green]
  public predictions = {};
  public win_pred = {};
  public maxs = {};

  options = {
    layers: [
      L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18 })//,
    ],
    zoom: 15,
    center: L.latLng({ lat: GlTrafficPredictionComponent.BIO_LAT, lng: GlTrafficPredictionComponent.BIO_LON }),
  };

  fromValtoHour(val:number){
    let hours:number = Math.floor(val*5/60);
    let mins:number = val*5-hours*60;
    let minsS = mins+"";
    if ( mins < 10 ) { minsS = "0"+minsS;}
    return hours+":"+minsS;

  }

  sliderOptions:Options = { 
    floor: 0,
    ceil: 287,
    translate: (value: number, label: LabelType): string => {
      switch (label) {
        case LabelType.Low:
          return this.fromValtoHour(value);
        case LabelType.High:
          return  this.fromValtoHour(value);
        default:
          return  ""+this.fromValtoHour(value);
      }
    }
  };

  tooltipChange(){
    this.toggleLbls = !this.toggleLbls;
    console.log('toogleLabel='+this.toggleLbls);
    if ( this.toggleLbls ){
      for(let p in this.plys){
        let ply = this.plys[p];
        ply.bindTooltip(ply.id+"", { permanent: true, offset: ply.offset });
      }
    }else{
      for(let p in this.plys){
        let ply = this.plys[p];
        let tooltip = ply.getTooltip();
        ply.unbindTooltip().bindTooltip(tooltip, { permanent: false});
      }      
    }
  }

  onMapReady(map: L.Map) {
    this.map = map;
    setTimeout(() => { }, 50);
    this.map.createPane("paneLow").style.zIndex = '250'; // between tiles and overlays
    this.map.createPane("paneHigh").style.zIndex = '450'; // between overlays and shadows
    this.load_polys();
    this.map.on('click',function(evt:any){
      console.log('latLon='+evt.latlng);
    });
  }
  
  ngOnInit(): void {
    let pilotName = this.configService.getSettings("default_pilot");
    if (pilotName === "BILBAO" || pilotName === "URBANITE") { this.city = "0"; this.pointsFile='/assets/geojsons/aforadores_bilbao.json'; }
    else if ( pilotName === "AMSTERDAM") { this.city = "1"; }
    else if ( pilotName === "HELSINKI" ) { this.city = "2"; this.pointsFile='/assets/geojsons/aforadores_helsinki.json'; }
    else if ( pilotName === "MESSINA"  ) { this.city = "3"; }  
    this.initMapOptions();
    this.load_from_win();
  }

  initMapOptions(){
    this.zoom = 15;
    if (this.city === "0") {
      this.lat = GlTrafficPredictionComponent.BIO_LAT;
      this.lon = GlTrafficPredictionComponent.BIO_LON;
    }
    else if (this.city === "1") {
      this.lat = GlTrafficPredictionComponent.AMS_LAT;
      this.lon = GlTrafficPredictionComponent.AMS_LON;
    }
    else if (this.city === "2") {
      this.lat = GlTrafficPredictionComponent.HEL_LAT;
      this.lon = GlTrafficPredictionComponent.HEL_LON;
      this.zoom = 11;
    }
    else if (this.city === "3") {
      this.lat = GlTrafficPredictionComponent.MES_LAT;
      this.lon = GlTrafficPredictionComponent.MES_LON;
      this.zoom = 13;
    }
  
    this.options = {
      layers: [
        L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' }),     
      ],
      zoom: this.zoom,
      center: L.latLng({ lat: this.lat, lng: this.lon }),
    };    
  }

  load_polys(){
   // let pointsFile = '/assets/geojsons/aforadores_bilbao.json';
    fetch(this.pointsFile).then(res => res.text()).then(data => {
      let locs:any = JSON.parse(data);
      this.plys = [];
      for(let e=0;e<locs.length;e++){
        
        let ele:object = locs[e];
        let id:string  = ele['id'];
        //console.log('gltraffic.load_polys::: id='+id);
        let coords: any = ele['coor'];
        let offset: any = ele['offset'];
        let direction: string = ele['direction'];
        let ply = L.polygon(coords,{color:'gray'}).addTo(this.map);
        ply['id'] = id;
        ply['gtcomp'] = this;
        ply['offset'] = offset;
        this.plys[id]= ply;
        let win = "<b>Sensor:</b  > "+id;
        win = win + "<div id='win_"+id+"' style='border-width:0.5px;border:solid;width:300px;height:200px'>";
        win = win + "<canvas id='chartDiv_"+id+"' style='display:none'>"
        win = win + "</canvas>";
        win = win+"</div>"
        ply.bindPopup(win).addTo(this.map);
        let main_object = this;
        ply.on("click", function (event) {
          for  (const [key, value] of Object.entries(event.target.gtcomp.plys)){
            let pl = event.target.gtcomp.plys[key];
            pl.setStyle({color:'gray'});
          }
          event.target.setStyle({color:'red'});
          let ctx = (document.getElementById('chartDiv_'+id) as HTMLCanvasElement).getContext("2d");
          document.getElementById('chartDiv_'+id).style.display = 'block';
          new Chart(ctx, {
            type: "line",
            data: main_object.chart_config[id].data,
            options: main_object.chart_config[id].options
          });
        });
      }
    });
  }
  load_from_win(){
    if ( window.localStorage.getItem('win_pred') != null &&    window.localStorage.getItem('win_pred')  != undefined ){
      console.log("gltraffic.onInit::: windowStorage HAS predictions");
      this.win_pred = JSON.parse(window.localStorage.getItem('win_pred') );
      console.log("gltraffic.onInit::: win_pred.size="+Object.keys(this.win_pred).length);
      for (let id in this.win_pred){
         this.addPrediction(id, this.win_pred[id]);
      }
      console.log("gltraffic.onInit::: predictions.size="+Object.keys(this.predictions).length);
    }else{
      console.log("gltraffic.onInit::: windowStorage does not have predictions");
    }   
  }
  slider_change(){
    //console.log("gltraffic.slider val="+this.sliderValue);
    //console.log("gltraffic.slider_change:: this.predictions="+Object.keys(this.predictions).length);
    if ( this.predictions == undefined || this.predictions == {} || this.predictions == null || Object.keys(this.predictions).length == 0){
      this.load_from_win();
    }
    for  (const [key, value] of Object.entries(this.plys)){
      let pl = this.plys[key];
      //console.log('slider_change::: key='+key+' sliderValue='+this.sliderValue+' val='+this.predictions[key][this.sliderValue]+' maxval='+this.maxs[key]);
      let clr = this.getColor(this.predictions[key][this.sliderValue],this.maxs[key]);
      pl.setStyle({color:clr});
    }    
  }

  getColor(val,maxval){
    let idx = Math.floor((1.0-val/maxval)*(this.colors.length-1));
    //console.log('getColor::: val='+val+' maxval='+maxval+' idx='+idx);
    return this.colors[idx];
  }

  getRandomColor() {
    var letters = '0123456789ABCDEF';
    var color = '#';
    for (var i = 0; i < 6; i++) {
      color += letters[Math.floor(Math.random() * 16)];
    }
    return color;
  }
  
  addPrediction(id:string,result:any){
    this.predictions[id] = {};
    let lmax = 0;
    for(let k=0;k<result['preds'].length;k++){
      this.predictions[id][k] = result['preds'][k][1];
      if ( result['preds'][k][1] > lmax ){
        lmax = result['preds'][k][1];
      }
    }
    
    this.maxs[id] = lmax;
    this.createChartConfig(id+"",result);
  }

  processResult(id:string,result:any){  
    this.win_pred[id] = result;
    window.localStorage.setItem("win_pred", JSON.stringify(this.win_pred));
    this.addPrediction(id,result);
  }

  createChartConfig(id:string, data:any) {
    let xs = [];
    let ys = [];
    let yups = [];
    let ydowns = [];
    let preds = [];
    let max_y = 0;
    for (let k = 0; k < data['preds'].length; k++) {
      xs.push(data['preds'][k][0]);
      preds.push({
        "name": data['preds'][k][0],
        "value": data['preds'][k][1],
        "min": data['limits'][k][1],
        "max": data['limits'][k][2]
      });
      ys.push({ "x": data['preds'][k][0], "y": data['preds'][k][1] });
      yups.push({ "x": data['limits'][k][0], "y": data['limits'][k][1] });
      ydowns.push({ "x": data['limits'][k][0], "y": data['limits'][k][2] });
      max_y = Math.max(max_y, data['preds'][k][1]);
    }
    max_y = 1.2 * max_y;

    let chart_data:any = {
      labels: xs,
      datasets: [
        {
          label: 'Pred',
          backgroundColor: 'rgb(255, 165, 0,0.1)',
          type: 'line',
          fill: false,
          borderColor: 'rgb(255, 0, 0,1)',
          borderWidth: 1,
          pointRadius: 0,
          yAxisID: 'y-axis-1',
          data: ys
        },
        {
          label: "BandTop",
          type: "line",
          backgroundColor: "rgb(255, 165, 0,0.3)",
          borderColor: "transparent",
          pointRadius: 0,
          legend: { display: false },
          fill: 0,
          tension: 0,
          yAxisID: 'y-axis-1',
          data: yups
        },
        {
          label: "BandBottom",
          type: "line",
          backgroundColor: "rgb(255, 165, 0,0.3)",
          borderColor: "transparent",
          pointRadius: 0,
          legend: { display: false },
          fill: 0,
          tension: 0,
          yAxisID: 'y-axis-1',
          data: ydowns
        }]
    };
    
    this.chart_config[id] = {
      data: chart_data, 
      options: {
        responsive: true,
        maintainAspectRatio: false,
        legend: {
          display: false
        },
        scales: {
          xAxes: [{
            type: 'time',
            barPercentage: 1.0,
            time: {
              displayFormats: {
                'hour': 'HH:mm',
                'day': 'HH:mm'
              }
            },
            ticks:{ 
              maxTicksLimit: 5,
              autoSkip: true, 
            }
          }],          
          yAxes: [
            {
              id: 'y-axis-1',
              scaleLabel: {
                display: false,
                labelString: ''
              },
              type: 'linear',
              display: true,
              position: 'right',
              gridlines: { display: true },
              labels: { show: true },
              ticks: { min:0, beginAtZero: true },
              min: 0,
            }
          ],
          y:{ min:0 }
        },
        plugins:{
          zoom: {
            pan: {
              enabled: true,
              mode: 'x',
            },
            zoom: {
              enabled: true,
              drag: false,
              mode: 'x',
            }
          }
        },
      }
    };
  }

  load_data(){
    this.sCombo   = (document.getElementById("combo_combo") as HTMLSelectElement).value;
    this.sWeekday = (document.getElementById("weekday_combo") as HTMLSelectElement).value;
    this.sHoliday = (document.getElementById("holiday_combo") as HTMLSelectElement).value;
    this.sSchool  = (document.getElementById("school_combo") as HTMLSelectElement).value;
    this.sTemp    = (document.getElementById("temp_combo") as HTMLSelectElement).value;
    this.sPrecp   = (document.getElementById("precp_combo") as HTMLSelectElement).value;
    this.sFutbol  = (document.getElementById("event_combo") as HTMLSelectElement).value;
    this.predictions = {};
    for(let id in this.plys){
      //console.log("load_data::: id="+id);
      this.getPredictionService.getValuesFromFeatures(id+"",this.sCombo,this.sWeekday,this.sTemp,this.sPrecp,this.sHoliday,this.sSchool,this.sFutbol).subscribe(res => this.processResult(id,res['result']), err => console.log(err));;
    }
  }
  openInstructions() {
    document.getElementById('instructionModal').style.display = 'block';
  }
  closeInstructions() {
    document.getElementById('instructionModal').style.display = 'none';
  }
}


