import { Component, OnInit } from '@angular/core';
import { HttpHeaders, HttpClient } from '@angular/common/http';
import * as L from 'leaflet';
import { ConfigService } from '@ngx-config/core';
import { GetNoiseComputationService } from '../services/get-noisecomputation.service';
import { PutGeoJsonService } from '../services/put-geojson.service';
@Component({
  selector: 'ngx-noise-comp',
  templateUrl: './noise-comp.component.html',
  styleUrls: ['./noise-comp.component.scss']
})

export class NoiseCompComponent implements OnInit {
  constructor(
    private configService: ConfigService, 
    private getNoiseComputationService: GetNoiseComputationService,
    private putGeoJSONService: PutGeoJsonService
  ) { this.apiURL = configService.getSettings("noisecomputation_api_base_url"); }

  private apiURL: any;
  private map: L.Map;
  private static newmap: L.Map;
  private city: string = "0";
  public lat:number = 0;
  public lon:number = 0;
  public zoom:number = 12;
  private static BIO_LON: number = -2.9356732032001926;
  private static BIO_LAT: number = 43.26201529732467;
  private static AMS_LON: number = 4.929861;  
  private static AMS_LAT: number = 52.400065; 
  private static HEL_LON: number = 24.925176054721625;
  private static HEL_LAT: number = 60.22470512386342;
  private static MES_LON: number = 15.553426081599502;
  private static MES_LAT: number = 38.18445422457372;
  public options = {};
  public newMapOptions = {};
  public drawControl: L.Control.Draw;
  public drawnLayer: L.DrawEvents.Created;
  public drawOptions: any;
  public drawnItems;
  public coords = [];

  public pnts = [];
  public vals63 = [];
  public vals125 = [];
  public vals250  = [];
  public vals500  = [];
  public vals1000 = [];
  public vals2000 = [];
  public vals4000 = [];
  public vals8000 = [];
  public valsleq = [];
  public valslaeq = [];

  public gidx;
  public legend;
  public marks = [];
  public static colors = ["#030001","#1C000B","#350016","#4E0021","#68002C","#810037","#9A0042","#B3004E","#CD005A","#E60067","#FF0073"];
  public formData:FormData;
  public filename:String;
  public trafficFilename:String;
  public map_names = [];
  public newgeojson_names = [];
  public newgeojsons = [];
  public model_names = [];
  public traffic_names = [];
  public static newSquareLayer;
  public geojson = {};
  ngOnInit(): void {
    let pilotName = this.configService.getSettings("default_pilot");
    if (pilotName === "BILBAO" || pilotName === "URBANITE") { this.city = "0"; }
    else if (pilotName === "AMSTERDAM") { this.city = "1"; }
    else if (pilotName === "HELSINKI") { this.city = "2"; }
    else if (pilotName === "MESSINA") { this.city = "3"; }    
    this.initMapOptions();
    this.update_runs();  
  }

  initMapOptions(){
    this.zoom = 15;
    if (this.city === "0") {
      this.lat = NoiseCompComponent.BIO_LAT;
      this.lon = NoiseCompComponent.BIO_LON;
    }
    else if (this.city === "1") {
      this.lat = NoiseCompComponent.AMS_LAT;
      this.lon = NoiseCompComponent.AMS_LON;
      this.zoom = 13;
    }
    else if (this.city === "2") {
      this.lat = NoiseCompComponent.HEL_LAT;
      this.lon = NoiseCompComponent.HEL_LON;
      this.zoom = 11;
    }
    else if (this.city === "3") {
      this.lat = NoiseCompComponent.MES_LAT;
      this.lon = NoiseCompComponent.MES_LON;
      this.zoom = 13;
    }
    this.drawnItems= L.featureGroup();
    this.drawOptions = {
      position: 'topright',
      draw: {
        marker: false,
        polyline: false,
        circle: false,
        circlemarker: false,
        polygon: false,
        rectangle: {
          shapeOptions: {
            color: '#85bb65'
          }
        }
      },
      edit: {
        featureGroup: this.drawnItems
      }
    };      
    this.options = {
      layers: [
        L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' }),     
      ],
      zoom: this.zoom,
      center: L.latLng({ lat: this.lat, lng: this.lon }),
    };   

    this.newMapOptions = {
      layers: [
        L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' }),     
      ],
      zoom: this.zoom,
      center: L.latLng({ lat: this.lat, lng: this.lon }),
    };  
  }

  onMapReady(map: L.Map) {
    this.map = map;
    setTimeout(() => { }, 50);
    this.map.createPane("paneLow").style.zIndex = '250'; // between tiles and overlays
    this.map.createPane("paneHigh").style.zIndex = '450'; // between overlays and shadows
    this.map.on('click',function(evt:any){
      console.log('latLon='+evt.latlng);
    });
  }

  onNewMapReady(map: L.Map) {
    NoiseCompComponent.newmap = map;
    setTimeout(() => { }, 50);
    NoiseCompComponent.newmap.createPane("paneLow").style.zIndex = '250'; // between tiles and overlays
    NoiseCompComponent.newmap.createPane("paneHigh").style.zIndex = '450'; // between overlays and shadows
    NoiseCompComponent.newmap.on('click',function(evt:any){
      console.log('latLon='+evt.latlng);
    });
  }

  public onDrawReady(dc: L.Control.Draw): void {
    this.drawControl = dc;
  }

  public onDrawEnd(draw: L.DrawEvents.Created): void {
    let layer = draw.layer;
    NoiseCompComponent.newmap.addLayer(layer);
    this.coords = layer.toGeoJSON()['geometry']['coordinates'];
    NoiseCompComponent.newSquareLayer = layer;
  }

  loadMap(){
    console.log('loadMap::: this.coords='+this.coords);
    if (  this.coords !== undefined ){
      if ( this.coords.length === 1 ){
        let eles:any = this.coords[0];
        console.log('loadMap:::eles.length='+eles.length);
        if ( eles.length === 5 ){
          console.log('loadMap:::eles.length='+eles.length);
          var maxLat = -10000.0;
          var minLat =  10000.0;
          var maxLon = -10000.0;
          var minLon =  10000.0;
          for(let i=0;i<5;i++){
            let ele = eles[i];
            let lat = ele[1];
            let lon = ele[0];
            if ( lat > maxLat ) maxLat = lat;
            if ( lat < minLat ) minLat = lat;
            if ( lon > maxLon ) maxLon = lon;
            if ( lon < minLon ) minLon = lon;
          }
          var bbox = [];
          bbox[0] = minLon;
          bbox[1] = minLat;
          bbox[2] = maxLon;
          bbox[3] = maxLat;
          console.log('loadMap::: bbox='+bbox);
          let ele:HTMLInputElement = document.getElementById("newMapName") as HTMLInputElement;
          let name = ele.value;
          if ( this.map_names.includes(name) ){
            alert("Name:"+name+" is already used in the platform.\nPlease choose different name.");
            return;
          }
          if ( name == "0" ){
            alert("Name: 0 is NOT a valid name.\nPlease choose different name.");
            return;           
          }
          if ( name == "" ){
            alert("Name: \"\" is NOT a valid name.\nPlease choose different name.");
            return;           
          }
          var data = {};
          data["name"] = name;
          data["bbox"] = bbox;
          console.log("loadMap()::: data="+JSON.stringify(data));
          this.createNewMap(data);
        }
      }
    }
    this.getMapsFromDB(0);
    document.getElementById('newMapModal').style.display="none";   
  }

  loadNewTraffic(){
    console.log('loadNewTraffic::: this.coords='+this.coords);;
    let ole:HTMLInputElement = document.getElementById("newTrafficInput") as HTMLInputElement;
    let path = ole.value;
    console.log('path='+path+"\ttrafficFilename="+this.trafficFilename);
    //let ele:HTMLInputElement = document.getElementById("newTrafficName")  as HTMLInputElement;
    //let name = ele.value;
    var data = {};
    data["path"] = this.trafficFilename;
    console.log("loadNewTraffic()::: data="+JSON.stringify(data));  
    this.getNoiseComputationService.createNewTraffic(data,this.formData).subscribe(res => document.getElementById('newTrafficModal').style.display = "none", err => console.log("loadNewTraffic::: ERROR: err="+err));
  }

  createNewMap(input){
    console.log('createNewMap::: input='+input);
    this.getNoiseComputationService.createNewMap(input).subscribe(res => console.log("createNewMap::: success data="+JSON.stringify(res)), err => console.log("createNewMap::: ERROR: err="+err));
  }

  getMapsFromDB0(data,ttype){
    var select = document.getElementById("map_combo");
    select.innerHTML = "";
    console.log('getMapsFromDB0::: data='+JSON.stringify(data));
    //var dataJ = JSON.parse(data);
    var el = document.createElement("option");
    el.textContent = "0: do not use";
    el.value = "0#NONE";
    select.appendChild(el);
    this.map_names = [];
    for(var i = 0; i < data.length; i++) {
        console.log("getMapsFromDB0::: data["+i+"]="+JSON.stringify(data[i]));
        var opt = data[i];
        var el = document.createElement("option");
        el.textContent = opt["id"]+": "+opt["name"];
        this.map_names.push(opt["name"]);
        el.value = opt["loc"]+"#"+opt["bbox"];
        console.log("getMapsFromDB0::: value="+el.value);
        select.appendChild(el);
    };
    if ( ttype === 1 ){
      this.getModelsFromDB(ttype);
    }
    console.log("getMapsFromDB0::: success data="+JSON.stringify(data));
    //document.getElementById("mapasCB").value = -1;
  }

  getTrafficFromDB0(data){
    var select = document.getElementById("traffic_combo");
    select.innerHTML = "";
    console.log('getTrafficFromDB0::: data='+JSON.stringify(data));
    //var dataJ = JSON.parse(data);
    var el = document.createElement("option");
    el.textContent = "0: do not use";
    el.value = "0#NONE";
    console.log("getTrafficFromDB0::: value="+el.value);
    select.appendChild(el);
    this.traffic_names = [];
    for(var i = 0; i < data.length; i++) {
        console.log("getTrafficFromDB0::: data["+i+"]="+JSON.stringify(data[i]));
        var opt = data[i];
        var el = document.createElement("option");
        el.textContent = opt["id"]+": "+opt["name"];
        this.traffic_names.push(opt['name']);
        el.value = opt["id"]+"#"+opt["name"];
        console.log("getTrafficFromDB0::: value="+el.value);
        select.appendChild(el);
    };
    console.log("getTrafficFromDB0::: success data="+JSON.stringify(data));
    console.log("getTrafficFromDB0::: traffic_names="+JSON.stringify(this.traffic_names));
    //document.getElementById("mapasCB").value = -1;
  }

  getNewGeoJSONS0(data,ttype){
    var select = document.getElementById("newgeojson_combo");
    select.innerHTML = "";
    if ( data != null ){
      console.log('getNewGeoJSONS0::: data='+JSON.stringify(data));
      this.newgeojson_names = [];
      for(var i = 0; i < data.length; i++) {
          console.log("getNewGeoJSONS0::: data["+i+"]="+JSON.stringify(data[i]));
          var opt = data[i];
          var el = document.createElement("option");
          el.textContent = i+": "+opt["name"];  
          this.newgeojson_names.push(opt["name"]);
          this.newgeojsons.push(opt);
          el.value = i+"";
          console.log("getNewGeoJSONS0::: value="+el.value);
          select.appendChild(el);
      };
    }
    if ( ttype === 1 ){
      this.getMapsFromDB(ttype);
    }
    console.log("getNewGeoJSONS0::: success data="+JSON.stringify(data));
  }

  getModelsFromDB0(data, ttype){
    console.log('getModelsFromDB0='+JSON.stringify(data));
    let selectElement = document.getElementById('comps_combo') as HTMLSelectElement;
    selectElement.innerText = null;
    this.pnts = [];
    this.vals63 = [];
    this.vals125 = [];
    this.vals250  = [];
    this.vals500  = [];
    this.vals1000 = [];
    this.vals2000 = [];
    this.vals4000 = [];
    this.vals8000 = [];
    this.valsleq = [];
    this.valslaeq = [];
    this.model_names = [];
    for(var i = 0; i < data.length; i++) {
        var opt = data[i];
        this.model_names.push(opt['name']);
        this.insertRow(opt['name'], opt['status'], opt['mapa'], opt['traffic']);
        if ( opt["gpss"] != null ){
          console.log("getModelsFromDB::: i="+i+"\t size="+opt["gpss"].length);     
        } 
        this.pnts.push(JSON.parse(opt["gpss"]));
        this.vals63.push(JSON.parse(opt["hz63"]));
        this.vals125.push(JSON.parse(opt["hz125"]));
        this.vals250.push(JSON.parse(opt["hz250"]));
        this.vals500.push(JSON.parse(opt["hz500"]));
        this.vals1000.push(JSON.parse(opt["hz1000"]));
        this.vals2000.push(JSON.parse(opt["hz2000"]));
        this.vals4000.push(JSON.parse(opt["hz4000"]));
        this.vals8000.push(JSON.parse(opt["hz8000"]));
        this.valsleq.push(JSON.parse(opt["leq"]));
        this.valslaeq.push(JSON.parse(opt["laeq"]));
    };
    if ( ttype == 1 ){
      this.getTrafficFromDB();
    }
    //console.log('getModelsFromDB0::: vals63[0].length='+this.vals63[0].length);
    //console.log('getModelsFromDB0::: vals63='+this.vals63[0]);
  }

  insertRow(name, status, mapa, traffic){
    let selectElement = document.getElementById('comps_combo') as HTMLSelectElement;
    var el = document.createElement("option");
    el.textContent = name+": <<"+mapa.replace(".osm","")+","+traffic+">> ["+status+"]";
    el.value = name;
    console.log("getMapsFromDB0::: value="+el.value);
    selectElement.appendChild(el);
  }

  asc = arr => arr.sort((a, b) => a - b);
  sum = arr => arr.reduce((a, b) => a + b, 0);
  mean = arr => this.sum(arr) / arr.length;

  quantile(arr, q){
      const sorted = this.asc(arr);
      const pos = (sorted.length - 1) * q;
      const base = Math.floor(pos);
      const rest = pos - base;
      if (sorted[base + 1] !== undefined) {
          return sorted[base] + rest * (sorted[base + 1] - sorted[base]);
      } else {
          return sorted[base];
      }
  };
  
  getQuantiles(vlsI){
      let vls = [];
      let q1 = 0.0;
      let q2 = 0.0;
      let q3 = 0.0;
      let q4 = 0.0;
      let q5 = 0.0;
      let q6 = 0.0;
      let q7 = 0.0;
      let q8 = 0.0;
      let q9 = 0.0;
      if ( vlsI != null ){
        for(let i=0;i<vlsI.length;i++){ 
            // if ( vlsI[i] > 0 ){ 
              vls.push(vlsI[i]); 
            //} 
        }    
        q1 = this.quantile(vls, .1);
        q2 = this.quantile(vls, .2);
        q3 = this.quantile(vls, .3);
        q4 = this.quantile(vls, .4);
        q5 = this.quantile(vls, .5);
        q6 = this.quantile(vls, .6);
        q7 = this.quantile(vls, .7);
        q8 = this.quantile(vls, .8);
        q9 = this.quantile(vls, .9);
      }else{
        console.log('getQuantiles::: vlsI is NULL');
      }
      return [q1,q2,q3,q4,q5,q6,q7,q8,q9];
  }

  getMax(vl){
    let mmax = -1000000.0;
    if ( vl != null ){
      console.log('getMax::: vl.length='+vl.length);
      for(let j=0;j<vl.length;j++){
          if( vl[j] > mmax){ mmax = vl[j];}
      }
      console.log('getMax::: mmax='+mmax);
    }else{
      console.log('getMax::: vl is NULL');
    }
    return mmax;
  }

  getClr2(vl,j,qs){
    let v = vl[j];   
    //let out =  NoiseCompComponent.colors[qs.length-1]
    //if ( v <= 0.0 ) { return '#303030'; }
    //else{
    for(let k=0;k<qs.length;k++){
      if ( v < qs[k] ){ return NoiseCompComponent.colors[k]; }
    }
    //}
    return NoiseCompComponent.colors[10];
  }
  
  printPnts(){
    let sel = document.getElementById('comps_combo') as HTMLSelectElement;
    this.gidx = sel.selectedIndex;
    console.log('printPnts::: gidx='+this.gidx+" name="+sel.value);
    if ( this.gidx !== -1 ){
        let ps = this.pnts[this.gidx];
        let vl = this.vals63[this.gidx];
        /*
        if      ( document.getElementById('hz63').checked )  { vl = this.vals63[this.gidx-1]; }
        else if ( document.getElementById('hz125').checked ) { vl = this.vals125[this.gidx-1]; }
        else if ( document.getElementById('hz250').checked ) { vl = this.vals250[this.gidx-1]; }
        else if ( document.getElementById('hz500').checked ) { vl = this.vals500[this.gidx-1]; }
        else if ( document.getElementById('hz1000').checked ){ vl = this.vals1000[this.gidx-1]; }
        else if ( document.getElementById('hz2000').checked ){ vl = this.vals2000[this.gidx-1]; }
        else if ( document.getElementById('hz4000').checked ){ vl = this.vals4000[this.gidx-1]; }
        else if ( document.getElementById('hz8000').checked ){ vl = this.vals8000[this.gidx-1]; }
        else if ( document.getElementById('leq').checked )   { vl = this.valsleq[this.gidx-1]; }
        else if ( document.getElementById('laeq').checked )  { vl = this.valslaeq[this.gidx-1]; }*/
        if ( ps != null ){
          console.log('printPnts::: ps.length='+ps.length);
        }
        if ( vl != null ){
          console.log('printPnts::: vl.length='+vl.length);
        }
        let max = this.getMax(vl);
        console.log('printPnts::: max(vl)='+max);
        let qs = this.getQuantiles(vl);
        console.log('db.js.printPnts::: qs='+qs);
        for(let j=0;j<this.marks.length;j++){
            this.map.removeLayer(this.marks[j]);
        }
        if ( this.legend instanceof L.Control ){
            let ddiv = document.getElementById('legend_div');
            ddiv.parentNode.removeChild(ddiv);
            ddiv.innerHTML = '';
            this.legend.remove;
        }
        this.marks = [];
        if ( ps != null ){
          for(let j=0;j<ps.length;j++){
              let mark = new L.CircleMarker(ps[j], {radius: 2, color: this.getClr2(vl,j,qs), opacity:.5});  
              mark['value'] = vl[j];
              //mark['options']['value'] = vl[j];
              //mark['properties'] = {};
              //mark['properties']['value'] = vl[j];
              //mark['feature']['properties']['value'] = vl[j]
              //console.log(mark);
              mark.addTo(this.map);
              this.marks.push(mark);
          }
        }
        this.legend = new L.Control({position: 'bottomleft'});
        this.legend.onAdd = function (map) {
            var div = L.DomUtil.create('div', 'info legend');
            div.id = 'legend_div';
            div.style.background = '#C0C0C0DD';
            div.style.width = '150px';     
            div.style.paddingTop = '10px';
            div.style.paddingBottom = '20px';
            let labels = [];        
            div.innerHTML +=
                    labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:#303030"></div><label style="position:absolute;left:45px;top:12px"> &leq; '+qs[0].toFixed(2)+'</label></div>');
            for (let i = 0; i < qs.length-1; i++) {
                let vv = qs[i].toFixed(2) + ' &harr; ' + qs[i+1].toFixed(2);
                div.innerHTML +=
                    labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:' + NoiseCompComponent.colors[i] + '"></div><label style="position:absolute;left:45px;top:' + (12 + 30 * (i+1)) + 'px">' + vv + '</label></div>');
            }
            let i = qs.length-1;
            div.innerHTML +=
                    labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:' + NoiseCompComponent.colors[i] + '"></div><label style="position:absolute;left:45px;top:' + (12 + 30 * (i+1)) + 'px"> &geq; '+qs[i].toFixed(2)+'</label></div>');
            div.innerHTML = labels.join('<br>');
            return div;         
        };
        this.legend.addTo(this.map);
    }
  }

  printNewGeoJSON(){
    let sel = document.getElementById('newgeojson_combo') as HTMLSelectElement;
    console.log('printNewGeoJSON::: value =\t'+sel.value);
    console.log('printNewGeoJSON::: valueI=\t'+parseInt(sel.value));
    if ( parseInt(sel.value) !== -1 ){
        let ps = this.pnts[this.gidx];
        let vl = this.vals63[this.gidx];

        if ( ps != null ){
          console.log('printPnts::: ps.length='+ps.length);
        }
        if ( vl != null ){
          console.log('printPnts::: vl.length='+vl.length);
        }
        let max = this.getMax(vl);
        console.log('printPnts::: max(vl)='+max);
        let qs = this.getQuantiles(vl);
        console.log('db.js.printPnts::: qs='+qs);
        for(let j=0;j<this.marks.length;j++){
            this.map.removeLayer(this.marks[j]);
        }
        if ( this.legend instanceof L.Control ){
            let ddiv = document.getElementById('legend_div');
            ddiv.parentNode.removeChild(ddiv);
            ddiv.innerHTML = '';
            this.legend.remove;
        }
        this.marks =[];
        if ( ps != null ){
          for(let j=0;j<ps.length;j++){
            let mark = L.circle(ps[j], {radius: 2, color: this.getClr2(vl,j,qs), opacity:.5}).addTo(this.map);
            this.marks.push(mark);
          }
        }
        this.legend = new L.Control({position: 'bottomleft'});
        this.legend.onAdd = function (map) {
            var div = L.DomUtil.create('div', 'info legend');
            div.id = 'legend_div';
            div.style.background = '#C0C0C0DD';
            div.style.width = '150px';     
            div.style.paddingTop = '10px';
            div.style.paddingBottom = '20px';
            let labels = [];        
            div.innerHTML +=
                    labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:#303030"></div><label style="position:absolute;left:45px;top:12px"> &leq; '+qs[0].toFixed(2)+'</label></div>');
            for (let i = 0; i < qs.length-1; i++) {
                let vv = qs[i].toFixed(2) + ' &harr; ' + qs[i+1].toFixed(2);
                div.innerHTML +=
                    labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:' + NoiseCompComponent.colors[i] + '"></div><label style="position:absolute;left:45px;top:' + (12 + 30 * (i+1)) + 'px">' + vv + '</label></div>');
            }
            let i = qs.length-1;
            div.innerHTML +=
                    labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:' + NoiseCompComponent.colors[i] + '"></div><label style="position:absolute;left:45px;top:' + (12 + 30 * (i+1)) + 'px"> &geq; '+qs[i].toFixed(2)+'</label></div>');
            div.innerHTML = labels.join('<br>');
            return div;         
        };
        this.legend.addTo(this.map);
    }
  }

  getModelsFromDB(ttype){
    this.getNoiseComputationService.getModels().subscribe(res => this.getModelsFromDB0(res,ttype), err => console.log("getModelsFromDB::: ERROR: err="+err));
  }

  getTrafficFromDB(){
    this.getNoiseComputationService.getTraffics().subscribe(res => this.getTrafficFromDB0(res), err => console.log("getTrafficFromDB::: ERROR: err="+err));
  }

  getMapsFromDB(ttype){
    this.getNoiseComputationService.getMaps().subscribe(res => this.getMapsFromDB0(res,ttype), err => console.log("getMapsFromDB::: ERROR: err="+err));
  }

  getNewGeoJSONsFromDB(ttype){
    this.getNoiseComputationService.getNewGeoJSONs().subscribe(res => this.getNewGeoJSONS0(res,ttype), err => console.log("getMapsFromDB::: ERROR: err="+err));
  }

  new_map(){
    document.getElementById('newMapModal').style.display = 'block';
    let ele1 = document.getElementById('map_combo') as HTMLSelectElement;
    let name = ele1.options[ele1.selectedIndex].value.split('#')[0];
    if ( NoiseCompComponent.newSquareLayer != null ){
      NoiseCompComponent.newmap.removeLayer(NoiseCompComponent.newSquareLayer);
    }
    if ( name !== "0" ){
      console.log('new_map::: name='+name);
      let ele:HTMLInputElement = document.getElementById("newMapName") as HTMLInputElement;
      ele.value  = name.replace(".osm","");
      let bbox = JSON.parse(ele1.options[ele1.selectedIndex].value.split('#')[1]);
      let sw = [];
      sw.push(bbox[1]);
      sw.push(bbox
        [0]);
      let ne = [];
      ne.push(bbox[3]);
      ne.push(bbox[2]);
      let nbbox = [];
      nbbox.push(sw);
      nbbox.push(ne);
      console.log('new_map::: bbox='+bbox);
      console.log('new_map::: nbbox='+nbbox);
      NoiseCompComponent.newSquareLayer = L.rectangle(nbbox, {color: "#ff0000", weight: 1});
      NoiseCompComponent.newmap.addLayer(NoiseCompComponent.newSquareLayer);
      
    }else{
      console.log("new_map::: name is ZERO");
    }
    NoiseCompComponent.newmap.invalidateSize();
  }

  new_traffic(){
    console.log("noise-comp.new_traffic::: NOT YET");
    document.getElementById('newTrafficModal').style.display = 'block';
    
  }

  erase_run(){
    console.log("NOT YET");
    let sel = document.getElementById('comps_combo') as HTMLSelectElement;
    let selOpts = sel.selectedOptions;
    let val = selOpts[0].value;
    console.log('erase_run::: val='+val);
    let input = {'name':val}; 
    this.getNoiseComputationService.eraseModel(input).subscribe(res => this.getModelsFromDB(0), err => console.log("erase Model::: ERROR: err="+err));
  }

  update_runs(){
    this.getNewGeoJSONsFromDB(1);
  }

  closeNewMapModal(){
    document.getElementById('newMapModal').style.display = 'none';
  }
  closeNewTrafficModal(){
    document.getElementById('newTrafficModal').style.display = 'none';
  }
  create_model(){
    let inputName  = document.getElementById('nameID') as HTMLInputElement;
    let name = inputName.value;
    if ( this.model_names.includes(name) ){
        alert("Name:"+name+" is already used in the platform.\nPlease choose different name.")
    }
    let ele1 = document.getElementById('map_combo') as HTMLSelectElement;
    let map = ele1.options[ele1.selectedIndex].value.split('#')[0];
    let ele2 = document.getElementById('traffic_combo') as HTMLSelectElement;
    let traffic = "0";
    if  ( ele2.options[ele2.selectedIndex].value.includes('#') ){
      if (ele2.options[ele2.selectedIndex].value.split('#')[0] != "0" ){
        traffic = ""+ele2.options[ele2.selectedIndex].value.split('#')[1];
      }
    }
    let input = {'name':name, 'map':map, 'traffic':traffic}; 
    console.log("create_model::: input="+JSON.stringify(input));
    this.getNoiseComputationService.createModel(input).subscribe(res => console.log("createModel::: success data="+JSON.stringify(res)), err => console.log("createNewMap::: ERROR: err="+err));
  }

  downloadPnts(){
    let sel = document.getElementById('comps_combo') as HTMLSelectElement;
    this.gidx = sel.selectedIndex;
    console.log('printPnts::: gidx='+this.gidx);
    if ( this.gidx !== -1 ){  
      let nme = sel.selectedOptions[0].text.replace(": ","_").replace("=","_").split("[")[0].trim().replace("map_","").replace(",","_");
      console.log('printPnts::: nme='+nme); 
      let ps = this.pnts[this.gidx];
      let vl = this.vals63[this.gidx];
      let out = {};
      out["points"] = ps;
      out["values"] = vl;
      console.log("printPnts::: out="+JSON.stringify(out));
      const blob = new Blob([JSON.stringify(out)], { type: "text/json" });
      const link = document.createElement("a");
      link.download = nme+".json";
      link.href = window.URL.createObjectURL(blob);
      link.dataset.downloadurl = ["text/json", link.download, link.href].join(":");
      const evt = new MouseEvent("click", {
        view: window,
        bubbles: true,
        cancelable: true,
      });
      console.log();
      link.dispatchEvent(evt);
      link.remove();
    }
  }
  
  openInstructions() {
    document.getElementById('instructionModal').style.display = 'block';
  }

  closeInstructions() {
    document.getElementById('instructionModal').style.display = 'none';
  }

  selectFile(event:any) {
    console.log('selectFile::: event='+JSON.stringify(event.target.files));
    const file:File = event.target.files[0];
    this.trafficFilename = file.name;
    console.log('selectFile::: file='+file.name);
    console.log('selectFile::: traffic_names='+JSON.stringify(this.traffic_names));
    console.log('selectFile:::----------------------------------------------');
    if ( this.traffic_names.includes(file.name.replace(".zip","")) ){
      alert(file.name+" is already uploaded to the platform.\nPlease choose different file.");
    }
    if ( file ) {
      this.formData = new FormData();
      this.formData.append("thumbnail", file);
    }
  }
  generate_geojson(){
    this.geojson = {};
    this.geojson['type'] = 'FeatureCollection';
    this.geojson['features'] = []; 
    for (let j=0;j<this.marks.length;j++){
      let mark:L.CircleMarker = <L.CircleMarker>this.marks[j];
      let feature = {};
      feature['type'] = 'Feature';
      feature['properties'] = {'value': mark['value']};
      feature['geometry'] = {};
      feature['geometry']['type'] = 'Point';
      console.log(mark.getLatLng());
      feature['geometry']['coordinates'] = [mark.getLatLng()['lng'],mark.getLatLng()['lat']];
      this.geojson['features'].push(feature);
    }
  }
  upload_geojson(){
    console.log("upload_geojson::: clicked");
    /*
    var fg = L.featureGroup();
    this.map.eachLayer((layer)=>{
      if( layer instanceof L.Path || layer instanceof L.Marker){
        fg.addLayer(layer);
      }
    });
    console.log(fg.toGeoJSON());*/
    this.generate_geojson();
    //console.log(JSON.stringify(this.marks));
    let dt = new Date();
    let sel = document.getElementById('comps_combo') as HTMLSelectElement;
    let name = "noise_computation_"+sel.value+"_"+dt.getTime();
    console.log("upload_geojson::: name="+name);
    console.log("upload_geojson::: this.geojson="+JSON.stringify(this.geojson));
    this.putGeoJSONService.putGeoJSON(this.geojson,name,this.city, name, name).subscribe(
          res => {
            document.getElementById('nameNGJ').innerHTML = name;
            document.getElementById('newNewGeoJSONModal').style.display = "block";
            /*
            alert("\<html\>\<body\>\<h1\>this is alert heading\</h1\>\</html\>\</body\>");
                  alert("geojson "+name+" added to storage.\n"+
                        "<button  id='process_geojson_btn' (click)='process_geojson();'>Process GEOJSON</button>"
                       ); 
                  console.log("putGeoJSONService: SUCESS::: res="+JSON.stringify(res))
                  */
                 }, 
          err => {alert('ERROR!!!\ngeojson '+name+' NOT added to storage.');console.log("putGeoJSONService: ERROR::: err="+JSON.stringify(err));}
    );
  }
  
  clear_newmap(){
    if ( NoiseCompComponent.newSquareLayer != null ){
      NoiseCompComponent.newmap.removeLayer(NoiseCompComponent.newSquareLayer);
      NoiseCompComponent.newSquareLayer = null;
    }
    let ele:HTMLInputElement = document.getElementById("newMapName") as HTMLInputElement;
    ele.value ="";
  }

  process_newgeojson(){
    let filename = document.getElementById('nameNGJ').innerHTML;
    console.log("process_newgeojson::: filename="+filename);
    console.log("process_newgeojson::: should call to end point");
    let conf = {};
    conf["layer_id"] = 0;
    conf["title_layer_predicted"] = filename;
    conf["description_layer_predicted"] = filename;
    conf["geojson_layer_predicted"] = this.geojson;
    document.getElementById("newNewGeoJSONModal").style.display = "none";
  }

  closenewNewGeoJSONModal(){
    console.log('closenewNewGeoJSONModal::: in routine...');
    document.getElementById("newNewGeoJSONModal").style.display = "none";
  }
}
