import { Component, OnInit } from '@angular/core';
import * as L from 'leaflet';
import { ConfigService } from '@ngx-config/core';
import { GetOdfromcountsService } from '../services/get-odfromcounts.service';
import { LeafletLayerDirective } from '@asymmetrik/ngx-leaflet';
@Component({
  selector: 'ngx-od-from-counts',
  templateUrl: './od-from-counts.component.html',
  styleUrls: ['./od-from-counts.component.scss']
})
export class OdFromCountsComponent implements OnInit {

  constructor(
    private configService: ConfigService,
    private getOdfromcountsService: GetOdfromcountsService) { }


  private map: L.Map;

  private city: string = "0";
  public lat: number = 0;
  public lon: number = 0;
  public zoom: number = 12;
  private static BIO_LON: number = -2.9356732032001926;
  private static BIO_LAT: number = 43.26201529732467;
  private static AMS_LON: number = 4.895325492686641;
  private static AMS_LAT: number = 52.375147927015995;
  private static HEL_LON: number = 24.925176054721625;
  private static HEL_LAT: number = 60.22470512386342;
  private static MES_LON: number = 15.553426081599502;
  private static MES_LAT: number = 38.18445422457372;
  public options = {};
  private areas: any = {};
  private area;
  private layerGroup;
  private points: any = {};
  private models: any = {};
  private legend;
  private legend2;
  public static clrs: any = ['#fef5f0', '#fbd7c4', '#f6a585', '#f37048', '#e23218', '#b40b02', '#630000'];
  toggleLbls: boolean = false;

  ngOnInit(): void {
    let pilotName = this.configService.getSettings("default_pilot");
    if (pilotName === "BILBAO" || pilotName === "URBANITE") { this.city = "0"; }
    else if (pilotName === "AMSTERDAM") { this.city = "1"; }
    else if (pilotName === "HELSINKI") { this.city = "2"; }
    else if (pilotName === "MESSINA") { this.city = "3"; }
    this.initMapOptions();
    this.getAreas0(1);
    //this.getPoints0();
    //this.getModels0(); 
    let sel = (document.getElementById("ini_combo") as HTMLSelectElement);
    for (let i = 1; i < 9; i++) {
      let option = document.createElement("option");
      option.text = i + "";
      option.value = i + "";
      sel.add(option);
    }
    let option = document.createElement("option");
    option.text = "Ext";
    option.value = 8 + "";
    sel.add(option);
  }

  initMapOptions() {
    this.zoom = 15;
    let pointsFile = '/assets/geojsons/aforadores_bilbao.json';
    let areasFile = '/assets/geojsons/distritos_bilbao.geojson';
    if (this.city === "0") {
      this.lat = OdFromCountsComponent.BIO_LAT;
      this.lon = OdFromCountsComponent.BIO_LON;
      areasFile = '/assets/geojsons/distritos_bilbao.geojson';
      pointsFile = '/assets/geojsons/aforadores_bilbao.geojson';
      this.zoom = 13;
    }
    else if (this.city === "1") {
      this.lat = OdFromCountsComponent.AMS_LAT;
      this.lon = OdFromCountsComponent.AMS_LON;
    }
    else if (this.city === "2") {
      this.lat = OdFromCountsComponent.HEL_LAT;
      this.lon = OdFromCountsComponent.HEL_LON;
      areasFile = '/assets/geojsons/helsinki_regions.geojson';
      pointsFile = '/assets/geojsons/aforadores_bilbao.json';
      this.zoom = 11;
    }
    else if (this.city === "3") {
      this.lat = OdFromCountsComponent.MES_LAT;
      this.lon = OdFromCountsComponent.MES_LON;
      this.zoom = 13;
    }

    this.options = {
      layers: [
        L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' }),
      ],
      zoom: this.zoom,
      center: L.latLng({ lat: this.lat, lng: this.lon }),
    };

    fetch(pointsFile).then(res => res.text()).then(data => {
      document.getElementById('inputSensorsID').innerHTML = data;
    });

    fetch(areasFile).then(res => res.text()).then(data => {
      document.getElementById('inputAreasID').innerHTML = data;
    });

  }

  onMapReady(map: L.Map) {
    this.map = map;
    setTimeout(() => { }, 50);
    this.map.createPane("paneLow").style.zIndex = '250'; // between tiles and overlays
    this.map.createPane("paneHigh").style.zIndex = '450'; // between overlays and shadows
    this.map.on('click', function (evt: any) {
      console.log('latLon=' + evt.latlng);
    });
  }

  openLoadPointsModal() {
    document.getElementById('loadPointsModal').style.display = 'block';
  }

  openLoadAreasModal() {
    document.getElementById('loadAreasModal').style.display = 'block';
  }

  style(feature) {
    return {
      fillColor: '#E31A1C',
      weight: 2,
      opacity: 1,
      color: 'white',
      dashArray: '3',
      fillOpacity: 0.7
    };
  }
  clearAllData() {
    if (this.areas !== undefined && this.areas !== null) {
      this.map.removeLayer(this.areas);
    }
    if (this.points !== undefined && this.points !== null) {
      this.map.removeLayer(this.points);
    }
  }

  loadAreas() {
    document.getElementById('loadAreasModal').style.display = 'none';
    let areasText = document.getElementById('inputAreasID').textContent;
    let areasJSON = JSON.parse(areasText);
    let areasName = (<HTMLInputElement>document.getElementById("newAreasName")).value;
    let input = {};
    input["areas_id"] = areasName;
    input["areas_json"] = areasJSON;
    this.printAreas(areasJSON);
    this.createNewAreas(input);
  }

  createNewAreas(input) {
    this.getOdfromcountsService.createNewAreas(input).subscribe(res => this.getAreas0(0), err => console.log("createNewAreas::: ERROR: err=" + JSON.stringify(err)));
  }

  loadPoints() {
    document.getElementById('loadPointsModal').style.display = 'none';
    let pointsText = document.getElementById('inputSensorsID').textContent;
    let pointsJSON = JSON.parse(pointsText);
    let pointsName = (<HTMLInputElement>document.getElementById("newPointsName")).value;
    let input = {};
    input["points_id"] = pointsName;
    input["points_json"] = pointsJSON;
    this.printPoints(pointsJSON);
    this.createNewPoints(input);
  }

  createNewPoints(input) {
    this.getOdfromcountsService.createNewSensors(input).subscribe(res => this.getPoints0(0), err => console.log("createNewPoints::: ERROR: err=" + JSON.stringify(err)));
  }

  getPoints0(val) {
    this.getOdfromcountsService.getSensors().subscribe(res => this.getPoints(res, val)), err => console.log("getPoints0::: ERROR: err=" + JSON.stringify(err));
  }

  getPoints(res, val) {
    //console.log(JSON.stringify(res));
    var select = document.getElementById("sensors_combo");
    select.innerHTML = "";
    for (let i = 0; i < res.length; i++) {
      let opt = res[i];
      this.points[opt["sensors_id"]] = opt["sensors"]
      let el = document.createElement("option");
      el.textContent = opt["sensors_id"];
      el.value = opt["sensors_id"];
      //console.log("getPoints::: value="+el.value);
      //console.log("getPoints::: opt="+JSON.stringify(opt));
      select.appendChild(el);
    }
    if (val == 1) {
      this.getModels0();
    }
  }

  getAreas0(val) {
    this.getOdfromcountsService.getAreas().subscribe(res => this.getAreas(res, val)), err => console.log("getAreas0::: ERROR: err=" + JSON.stringify(err));
  }

  getAreas(res, val) {
    var select = document.getElementById("areas_combo");
    select.innerHTML = "";
    for (let i = 0; i < res.length; i++) {
      let opt = res[i];
      this.areas[opt["areas_id"]] = opt["areas"]
      let el = document.createElement("option");
      el.textContent = opt["areas_id"];
      el.value = opt["areas_id"];
      //console.log("getAreas::: opt="+JSON.stringify(opt));
      select.appendChild(el);
    }
    if (val == 1) {
      this.getPoints0(val);
    }
  }

  getModels0() {
    this.getOdfromcountsService.getModels().subscribe(res => this.getModels(res)), err => console.log("getModels0::: ERROR: err=" + JSON.stringify(err));
  }

  getModels(res) {
    var select = document.getElementById("models_combo");
    select.innerHTML = "";
    for (let i = 0; i < res.length; i++) {
      let opt = res[i];
      this.models[opt["sensors_id"] + "#" + opt["areas_id"]] = opt["result"]
      let el = document.createElement("option");
      el.textContent = opt["sensors_id"] + " " + opt["areas_id"] + "[" + opt["status"] + "]";
      el.value = opt["sensors_id"] + "#" + opt["areas_id"];
      //console.log("getModels::: value="+el.value);
      //console.log("result["+opt["sensors_id"]+"#"+opt["areas_id"]+"]="+JSON.stringify(this.models[opt["sensors_id"]+"#"+opt["areas_id"]]));
      select.appendChild(el);
    }
  }

  printPoints0() {
    let val = (document.getElementById("sensors_combo") as HTMLSelectElement).value;
    //console.log("printPoints0::: val="+val);
    //console.log("printPoints0::: points="+JSON.stringify(this.points[val]));  
    this.printPoints(this.points[val]);
  }

  printAreas0() {
    let val = (document.getElementById("areas_combo") as HTMLSelectElement).value;
    //console.log("printAreas0::: val="+val);
    //console.log("printAreas0::: areas="+JSON.stringify(this.areas[val]));
    this.printPoints(this.areas[val]);
  }

  printPoints(pointsJSON) {
    if (this.points !== undefined && this.points !== null) {
      this.map.removeLayer(this.points);
    }
   // this.markers = [];
    var geojsonMarkerOptions = {
      radius: 3,
      fillColor: "#00f",
      color: "#00f",
      weight: 1,
      opacity: 1,
      fillOpacity: 1,
      zIndex: '10000'
    };
    this.points = L.geoJSON(pointsJSON, {
      pointToLayer: function (feature, latlng) {
        return L.circleMarker(latlng, geojsonMarkerOptions);
      },
      pane: 'paneHigh'
    }).addTo(this.map);
  }

  printAreas(areasJSON) {
    if (this.layerGroup !== undefined && this.layerGroup !== null) {
      this.map.removeLayer(this.layerGroup);
    }
    //his.markers = [];
    this.layerGroup = L.layerGroup().addTo(this.map);
    let regions = [...areasJSON.features];
    regions.forEach((region) => {
      L.geoJSON(region).addTo(this.layerGroup);
    });

  }

  closeLoadPointsModal() {
    document.getElementById('loadPointsModal').style.display = 'none';
  }

  erase_run() {
    console.log("NOT YET");
  }

  printSolution() {
    let sol = (document.getElementById("models_combo") as HTMLSelectElement).value;
    let iniend_zone_id = (document.getElementById("ini_combo") as HTMLSelectElement).value;
    let iniend_vec_index = parseInt(iniend_zone_id)-1;
    let areas_id = sol.split("#")[1];
    this.area = this.areas[areas_id];
    this.printAreas(this.area);
    let res = this.models[sol];
    

    let vec = [];
    if (this.toggleLbls) {
      for (let i = 0; i < res[iniend_vec_index].length; i++) { vec.push(res[i][iniend_vec_index]); }
    } else {
      for (let i = 0; i < res[iniend_vec_index].length; i++) { vec.push(res[iniend_vec_index][i]); }
    }
    console.log("printSolution::: iniend_vec_index="+iniend_vec_index+" vec="+JSON.stringify(vec));
    let maxval = 0;
    for (let i = 0; i < vec.length; i++) { if (vec[i] > maxval) { maxval = vec[i]; } }
    console.log("printSolution::: maxval="+maxval);
    this.layerGroup.eachLayer((layer) => {
      let zone_id = parseInt((Object.values(layer._layers)[0] as L.Polygon).feature.properties.zone_id);
      let vec_index = zone_id - 1 ;
      let clr = this.fromValueToClr(vec[vec_index], maxval);
      layer.setStyle(() => ({ color: clr, fillColor: clr, fillOpacity: 0.6 }));
      layer.bindTooltip("<p><strong>"+zone_id+"</strong><br>val:" + vec[vec_index]+"</p>", { permanent: true, direction: "center", opacity:0.7 }).openTooltip();
    });
    let extValue = vec[vec.length-1];
    let extClr = this.fromValueToClr(extValue, maxval);
    if ( this.legend instanceof L.Control ){
      let ddiv = document.getElementById('legend_div');
      ddiv.parentNode.removeChild(ddiv);
      ddiv.innerHTML = '';
      this.legend.remove;
    }    
    this.legend = new L.Control({position: 'bottomleft'});
    this.legend.onAdd = function () {
        var div = L.DomUtil.create('div', 'info legend');
        div.id = 'legend_div';
        div.style.background = '#C0C0C0DD';
        div.style.width = '150px';     
        div.style.paddingTop = '10px';
        div.style.paddingBottom = '20px';
        let labels = [];        
        //div.innerHTML +=
        //        labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:#303030"></div><label style="position:absolute;left:45px;top:12px"> <= 0</label></div>');
        let v0 = 0;
        let delta = maxval/(OdFromCountsComponent.clrs.length);
        console.log('legend::: maxval='+maxval+' delta='+delta);
        for (let i = 0; i < OdFromCountsComponent.clrs.length; i++) {
            let vnext = v0 + delta;
            div.innerHTML +=
                labels.push('<div style="width:25px;height:12px;padding-left:10px;white-space:nowrap;"><div style="width:25px;height:20px;background:' + OdFromCountsComponent.clrs[i] + '"></div><label style="position:absolute;left:45px;top:' + (12 + 30 * (i)) + 'px">' + v0.toFixed(0) +'-'+vnext.toFixed(0)+ '</label></div>');
            v0 = vnext;
        }
        div.innerHTML = labels.join('<br>');
        return div;         
    };
    this.legend.addTo(this.map);  

    if ( this.legend2 instanceof L.Control ){
      let ddiv = document.getElementById('legend2_div');
      ddiv.parentNode.removeChild(ddiv);
      ddiv.innerHTML = '';
      this.legend2.remove;
    }    
    //<div class="" style="opacity: 0.7; transform: translate3d(503px, 226px, 0px);"><p><strong>2</strong><br>val:308</p></div>


    this.legend2 = new L.Control({position: 'topright'});
    this.legend2.onAdd = function () {
        var div = L.DomUtil.create('div', 'info legend');
        div.id = 'legend2_div';
        div.style.background = extClr;
        div.style.width = '150px';     
        div.style.paddingTop = '10px';
        div.style.paddingLeft = '10px';
        div.style.paddingBottom = '0px';
        div.style.height = '90px';
        div.style.opacity = '0.7';
        var div2 = L.DomUtil.create('div','leaflet-tooltip leaflet-zoom-animated leaflet-tooltip-center');
        div2.style.opacity = '0.7';
        div2.innerHTML = "<p><strong>Ext</strong><br>val:"+extValue+"</p>";
        div.append(div2);
        return div;         
    };
    this.legend2.addTo(this.map);  
  }

  fromValueToClr(val, maxval) {
    let idx = Math.floor(val / maxval * 6);
    return OdFromCountsComponent.clrs[idx];
  }

  oriDestChange() {
    console.log("oriDestChange::: this.toggleLbls=" + this.toggleLbls);
    this.toggleLbls = !this.toggleLbls;
    if (this.toggleLbls) {
      document.getElementById("odLabel").innerHTML = "Dest.";
    } else {
      document.getElementById("odLabel").innerHTML = "Ori.";
    }
  }
  openInstructions() {
    document.getElementById('instructionModal').style.display = 'block';
  }
  closeInstructions() {
    document.getElementById('instructionModal').style.display = 'none';
  }
}
