import { HttpClient,HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ConfigService } from '@ngx-config/core';
import { Observable, throwError } from 'rxjs';
import { retry, catchError } from 'rxjs/operators';

@Injectable({
  providedIn: 'root'
})
export class BikeAnalysisAMSService{
  private apiURL: any;
  constructor(configService: ConfigService,private http:HttpClient) {
    this.apiURL = configService.getSettings("bike_analysis_api_base_url");
  }
  httpOptions = {
    headers: new HttpHeaders({ 
      'Content-Type': 'application/json;charset=utf-8'

    })
  };
  getModels(): Observable<any> {
    return this.http.get<any>(`${this.apiURL}/webresources/getBikeAnalysis/getModels`);
  }

  getAreas(): Observable<any> {
    return this.http.get<any>(`${this.apiURL}/webresources/getBikeAnalysis/getAreas`);
  }

  putAreas(payload:any): Observable<any>{
    return this.http.put<any>(`${this.apiURL}/webresources/getBikeAnalysis/putAreas`, JSON.stringify(payload), this.httpOptions).pipe(retry(1),catchError(this.httpError));
  }

  createModel(input:any): Observable<any> {
    let content = JSON.stringify(input);
    return this.http.put<any>(`${this.apiURL}/webresources/getBikeAnalysis/createModel`,content, this.httpOptions).pipe(retry(1),catchError(this.httpError));
  }  

  httpError(error:any) {
    let msg = '';
    if(error.error instanceof ErrorEvent) {
      msg = error.error.message;
    } else {
      msg = `Error Code: ${error.status}\nMessage: ${error.message}`;
    }
    console.log(msg);
    return throwError(msg);
  }  
}