import { Component, OnInit } from '@angular/core';
import * as L from 'leaflet';
import { GetEspirasService } from '../services/get-espiras.service';
import { GetModelsService } from '../services/get-models.service';
import { GetEspiradataService } from '../services/get-espiradata.service'; 
import { DeleteTrafficModelService } from '../services/delete-trafficmodel.service';
import { GetPredictionService } from '../services/get-prediction.service';
import { CreateNewmodelService } from '../services/create-newmodel.service';
import { ConfigService } from '@ngx-config/core';


@Component({
  selector: 'ngx-traffic-prediction',
  templateUrl: './traffic-prediction.component.html',
  styleUrls: ['./traffic-prediction.component.scss']
})

export class TrafficPredictionComponent implements OnInit {
  constructor(private getEspirasService: GetEspirasService,
    private getModelsService: GetModelsService,
    private getEspiradataService: GetEspiradataService,
    private getPredictionService: GetPredictionService,
    private createNewmodelService: CreateNewmodelService,
    private configService: ConfigService,
    private deleteTrafficModelService: DeleteTrafficModelService
  ) { }

  private markers: any[] = [];
  private intensidadJA: any = {};
  private temperaturaJA: any = {};
  toggleLbls: boolean = false;
  toggleFilter:boolean = false;
  private calendarInfo: any = {};
  private espirasLocations: any = {};
  private map: L.Map;
  private city: string = "0";

  public lat: number = 0;
  public lon: number = 0;
  public zoom: number = 14;
  private models;
  private plys: object = {};
  private static BIO_LON: number = -2.929832;
  private static BIO_LAT: number = 43.259695;
  private static AMS_LON: number = 4.895325492686641;
  private static AMS_LAT: number = 52.375147927015995;
  private static HEL_LON: number = 24.925176054721625;
  private static HEL_LAT: number = 60.22470512386342;
  private static MES_LON: number = 15.553426081599502;
  private static MES_LAT: number = 38.18445422457372;
  private emptyData: any = [[0, 0], [0, 0]];

  chart_config: any = {};
  public nfecha: Date = new Date();
  public ofecha: Date = new Date();
  public sfecha: Date = new Date();
  public fecha:  Date = new Date();

  public rfecha: Date = new Date();
  public rDate:  Date = new Date();
  public lDate:  Date = new Date();
  public toDownload:any = [];
  public toDownloadName: String = "noData";
  public sfeature_combo: number = -1;
  public calendarRestriction = [];
  public allowedDay = true;
  public static pnts:any[]= [];
  public static pntsO:any[]= [];
  private static sespira:any[] = [];
  public dataList: Array<ILine> = [];

  public options = {
    layers: [
      L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors' }),
    ],
    zoom: 12,
    center: L.latLng({ lat: 0, lng: 0 }),
  };


  public static polyLineOptions:L.PolylineOptions = {
    color: 'blue',
    opacity: 0.9
  };

  onMapReady(map: L.Map) {
    this.map = map;
    setTimeout(() => {
    }, 50);
    TrafficPredictionComponent.pnts = [];
    map.on('click', function (e:L.LeafletMouseEvent){
      var lat = e.latlng.lat;
      var lon = e.latlng.lng; 
      let pt:any[] = [lat,lon];
      let ptO = new L.LatLng(lat, lon);
      TrafficPredictionComponent.pnts.push(pt);
      TrafficPredictionComponent.pntsO.push(ptO);
      const polyline = L.polyline(TrafficPredictionComponent.pntsO, TrafficPredictionComponent.polyLineOptions).addTo(map);
      map.fitBounds(polyline.getBounds());
      let ele = {};
      ele['id'] = TrafficPredictionComponent.sespira[0];
      ele['offset'] = [0,0];
      ele['coor'] = TrafficPredictionComponent.pnts;
      console.log(TrafficPredictionComponent.sespira[0]+'\t'+pt[0]+'\t'+pt[1]);
      console.log(JSON.stringify(ele));
    });
    map.on('contextmenu',function(){
      console.log('erassing array');
      TrafficPredictionComponent.pnts = [];
      TrafficPredictionComponent.pntsO = [];
    });
  }

  addDays(dateTime, count_days = 0){
    return new Date(new Date(dateTime).setDate(dateTime.getDate() + count_days));
  }

  ngOnInit(): void {
    this.calendarRestriction = [];
    let dd = new Date();
    for(let i=0;i<4;i++){
      dd = this.addDays(dd,1);
      let mS = (dd.getMonth()+1)+'';
      if ((dd.getMonth()+1) < 10){
        mS = '0'+mS;
      }
      let dS = dd.getDate()+'';
      if ( dd.getDate() < 10 ){
        dS = '0'+dS;
      }
      let ll = dd.getFullYear()+"-"+mS+"-"+dS;
      this.calendarRestriction.push(ll);
    }
    console.log('ngOnInit::: calendarRestriction='+this.calendarRestriction);
    let pilotName = this.configService.getSettings("default_pilot");
    if (pilotName === "BILBAO" || pilotName === "URBANITE") { this.city = "0"; }
    else if ( pilotName === "AMSTERDAM") { this.city = "1"; }
    else if ( pilotName === "HELSINKI" ) { this.city = "2"; document.getElementById("last_option").innerHTML = "5: HoD, DoW, SaH, MET and Ferry Arrival/Departure"; }
    else if ( pilotName === "MESSINA"  ) { this.city = "3"; }
    this.getEspirasLocations0();
    this.initMapOptions();
  }

  openInstructions() {
    document.getElementById('instructionModal').style.display = 'block';
  }
  
  closeInstructions() {
    document.getElementById('instructionModal').style.display = 'none';
  }

  initMapOptions() {
    console.log("initMapOptions::: this.city="+this.city);
    this.zoom = 14;
    if (this.city === "0") {
      this.lat = TrafficPredictionComponent.BIO_LAT;
      this.lon = TrafficPredictionComponent.BIO_LON;
    }
    else if (this.city === "1") {
      this.lat = TrafficPredictionComponent.AMS_LAT;
      this.lon = TrafficPredictionComponent.AMS_LON;
    }
    else if (this.city === "2") {
      this.lat = TrafficPredictionComponent.HEL_LAT;
      this.lon = TrafficPredictionComponent.HEL_LON;
      this.zoom = 11;
    }
    else if (this.city === "3") {
      this.lat = TrafficPredictionComponent.MES_LAT;
      this.lon = TrafficPredictionComponent.MES_LON;
      this.zoom = 13;
    }
    this.options = {
      layers: [
        L.tileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', { maxZoom: 18, attribution: '...' }),
      ],
      zoom: this.zoom,
      center: L.latLng({ lat: this.lat, lng: this.lon }),
    };
  }

  clickModelTable(e: any) {
    console.log('mtable clicked...');
    this.sfeature_combo = -1;
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById("models_table");
    for (let i = 0; i < ttable.rows.length; i++) {
      let row = ttable.rows[i];
      for (let j = 0; j < row.cells.length; j++) {
        let ccell = row.cells[j];
        ccell.style.backgroundColor = "white";
        ccell.style.border = 'solid';
        ccell.style.borderWidth = '1px';
      }
    }
    let et: HTMLElement = <HTMLElement>e.target;
    let row: HTMLTableRowElement = <HTMLTableRowElement>et.parentNode;
    let espira_id = row.cells[1].innerHTML;
    this.closeAllPopupsBut(espira_id);
    this.selectEspira(this.plys[espira_id]);
    for (let j = 0; j < row.cells.length; j++) {
      let ccell = row.cells[j];
      ccell.style.backgroundColor = "#FED8B1";
      ccell.style.border = 'solid';
      ccell.style.borderWidth = '1px';
      if ( j == 2 ){
        this.sfeature_combo = parseInt(row.cells[j].innerHTML);
      }
    }
    let firstRow = ttable.rows[0];
    for (let j = 0; j < firstRow.cells.length; j++) {
      let ccell = firstRow.cells[j];
      ccell.style.backgroundColor = "white";
      ccell.style.border = 'solid';
      ccell.style.borderWidth = '1px';
    }
    console.log("clickModelTable::: feature_combo selected="+this.sfeature_combo);
    this.createCalendar(this.fecha);
  }

  closeAllPopups() {
    for (let p in this.plys) {
      let ply = this.plys[p];
      ply.closePopup();
    }
  }

  tooltipChange() {
    this.toggleLbls = !this.toggleLbls;
    console.log('toogleLabel=' + this.toggleLbls);
    if (this.toggleLbls) {
      for (let p in this.plys) {
        let ply = this.plys[p];
        ply.openPopup();
      }
    } else {
      this.closeAllPopups();
    }
  }

  closeAllPopupsBut(espira_id: string) {
    for (let p in this.plys) {
      let ply = this.plys[p];
      ply.closePopup();
    }
    this.plys[espira_id].openPopup();
  }

  getEspirasLocations0(): void {
    this.getEspirasService.getValues().subscribe(res => this.getEspirasLocations(res), err => console.log(err));
  }

  public selectEspira(lmarker: any) {
    document.getElementById("chartDiv").style.display = "none";
    for (let k = 0; k < this.markers.length; k++) {
      this.markers[k].setStyle({ color: 'blue', fillColor: '#30f', });
    }
    lmarker.setStyle({ color: 'red', fillColor: '#f03' });
    console.log('title=' + lmarker['title']);  
    let lid = lmarker['title'];
    TrafficPredictionComponent.sespira.pop();
    TrafficPredictionComponent.sespira.push(lid);
    console.log('city=' + this.city + " id=" + lid);
    let esp = this.espirasLocations[lmarker['title']];
    lmarker['component'].dataToTable(esp);
   // this.dataToModelsTable(this.models);
  }

  getEspirasLocations(data: any): void {
    for (let k = 0; k < this.markers.length; k++) {
      console.log('removing... k=' + k);
      this.map.removeLayer(this.markers[k]);
    }
    this.markers = [];
    this.espirasLocations = {};
    this.plys = {};
    for (let k = 0; k < data.length; k++) {
      let espira = data[k];
      let lkey = espira['id'].trim();
      let coords = [espira['lat'], espira['lon']];
      var lmarker = L.circle([espira['lat'], espira['lon']], {
        color: 'blue',
        fillColor: '#30f',
        fillOpacity: 0.5,
        radius: 5
      }).addTo(this.map);
      lmarker['title'] = lkey;
      lmarker['component'] = this;
      lmarker.on("click", function (event) { this['component'].closeAllPopups(); this['component'].selectEspira(this); });
      this.espirasLocations[lkey] = espira;
      var new_popup = L.popup({ "autoClose": false, "closeOnClick": null }).setContent(lkey);
      lmarker.bindPopup(new_popup);
      this.plys[lkey] = lmarker;
      this.markers.push(lmarker);
    }
    this.getModels0();
  }

  getModels0(): void {
    this.getModelsService.getValues().subscribe(res => this.getModels(res), err => console.log(err));
  }

  getModels(models: any): void {
    this.models = models;
    this.dataToModelsTable(models);
    this.createCalendar(this.fecha);
  }

  createCalendar0(data: any) {
    this.calendarInfo = data;
    this.createCalendar(this.fecha);
  }

  toogleFilter(){
    this.toggleFilter = ! this.toggleFilter;
    this.dataToModelsTable(this.models);
  }

  dataToModelsTable(data: any[]) {
    console.log('espiras.js.dataToModelsTable::: models=' + data+"  toggleFilter="+this.toggleFilter);
    
    var ttable: HTMLTableElement = <HTMLTableElement>document.getElementById("models_table");
    ttable.innerHTML = "<tr><th style=\"border:solid;border-width:1px\"><b>Model</b></th>\n\
                            <th style=\"border:solid;border-width:1px\"><b>id</b></th>\n\
                            <th style=\"border:solid;border-width:1px\"><b>#</b></th>\n\
                            <th style=\"border:solid;border-width:1px\"><b>Status</b></th>\n\
                            <th style=\"border:solid;border-width:1px\"><b>Score</b></th></tr>";
    
    
                            
    this.dataList = [];
    for (let k = 0; k < data.length; k++) {
      let ele = data[k];
      if ( !this.toggleFilter ){
        let row = ttable.insertRow(ttable.rows.length);
        let cell0 = row.insertCell(0);
        let cell1 = row.insertCell(1);
        let cell2 = row.insertCell(2);
        let cell3 = row.insertCell(3);
        let cell4 = row.insertCell(4);
        cell0.innerHTML = ele['model_id'];
        cell0.style.border = 'solid';
        cell0.style.borderWidth = '1px';
        cell1.innerHTML = ele['id'];
        cell1.style.border = 'solid';
        cell1.style.borderWidth = '1px';
        cell2.innerHTML = ele['feature_combo'];
        cell2.style.border = 'solid';
        cell2.style.borderWidth = '1px';
        cell3.innerHTML = ele['status'];
        cell3.style.border = 'solid';
        cell3.style.borderWidth = '1px';
        let perfm_out = "-"; 
        if ( ele['performance'] !== null ){ perfm_out = ele['performance'].toFixed(2); }
        cell4.innerHTML = perfm_out;
        cell4.style.border = 'solid';
        cell4.style.borderWidth = '1px';
        this.dataList.push({model_id:ele['model_id'],id:ele['id'], feature_combo:ele['feature_combo'], status:ele['status'], score:perfm_out});
      }else{
        if ( ele['id'] == TrafficPredictionComponent.sespira[0] || TrafficPredictionComponent.sespira.length == 0){
          let row = ttable.insertRow(ttable.rows.length);
          let cell0 = row.insertCell(0);
          let cell1 = row.insertCell(1);
          let cell2 = row.insertCell(2);
          let cell3 = row.insertCell(3);
          let cell4 = row.insertCell(4);
          cell0.innerHTML = ele['model_id'];
          cell0.style.border = 'solid';
          cell0.style.borderWidth = '1px';
          cell1.innerHTML = ele['id'];
          cell1.style.border = 'solid';
          cell1.style.borderWidth = '1px';
          cell2.innerHTML = ele['feature_combo'];
          cell2.style.border = 'solid';
          cell2.style.borderWidth = '1px';
          cell3.innerHTML = ele['status'];
          cell3.style.border = 'solid';
          cell3.style.borderWidth = '1px';
          let perfm_out = "-";
          if ( ele['performance'] !== null ){ perfm_out = ele['performance'].toFixed(2);}
          cell4.innerHTML = perfm_out;
          cell4.style.border = 'solid';
          cell4.style.borderWidth = '1px';
          this.dataList.push({model_id:ele['model_id'],id:ele['id'], feature_combo:ele['feature_combo'], status:ele['status'], score:perfm_out});     
        }
      }
    }
  }

  getEspiraData(lid: string) {
    this.getEspiradataService.getValues(lid).subscribe(res => this.printEspira(res,lid), err => console.log(err));
  }

  printEspira(data: any[],lid) {
    console.log('espiras.js.printEspira::: data.length=' + data.length);
    this.toDownload = data;
    this.toDownloadName = "loopData_"+lid;
    let xs = [];
    let ys = [];
    let traffic = [];
    let temps = [];
    let precs = [];
    let max_x = data[data.length - 1][0];
    let min_x = data[0][0];
    for (let k = 0; k < data.length; k++) {
      xs.push(data[k][0]);
      ys.push(data[k][1]);
      traffic.push({ x: data[k][0], y: data[k][1] });
    }
    for (let k = 0; k < this.temperaturaJA.length; k++) {
      if (this.temperaturaJA[k][0] > min_x && this.temperaturaJA[k][0] < max_x) {
        temps.push({ x: this.temperaturaJA[k][0], y: this.temperaturaJA[k][1] });
      }
    }
    let avg_int = 0;
    for (let k = 0; k < this.intensidadJA.length; k++) {
      if (this.intensidadJA[k][0] > min_x && this.intensidadJA[k][0] < max_x) {
        precs.push({ x: this.intensidadJA[k][0], y: this.intensidadJA[k][1] });
        avg_int = avg_int + this.intensidadJA[k][1];
      }
    }
    avg_int = avg_int / (precs.length * 1.0);
    console.log('espiras.js.printChart::: avg_int=' + avg_int + ' precs.length=' + precs.length);

    let chart_data: any = {
      labels: xs,
      type: 'line',
      datasets: [
        {
          label: 'Veh Flow',
          type: 'line',
          fill: false,
          borderWidth: 1,
          pointRadius: 0,
          borderColor: 'rgb(0, 0, 255)',
          backgroundColor: 'rgb(0, 0, 255)',
          pointBorderColor: 'rgb(0, 0, 255)',
          pointBackgroundColor: 'rgb(0, 0, 255)',
          yAxisID: 'y-axis-1',
          data: traffic
        }
      ]
    };

    this.chart_config = {
      type: 'line',
      data: chart_data,
      options: {
        responsive: true,
        maintainAspectRatio: false,
        tooltips: {
          mode: 'label'
        },
        elements: {
          line: {
            fill: false
          }
        },
        scales: {
          xAxes: [{
            type: 'time',
            barPercentage: 1.0,
            time: {
              displayFormats: {
                'millisecond': 'DD-MM-YY',
                'second': 'DD-MM-YY',
                'minute': 'DD-MM-YY',
                'hour': 'DD-MM-YY',
                'day': 'DD-MM-YY',
                'week': 'DD-MM-YY',
                'month': 'MM-YY',
                'quarter': 'MM-YY',
                'year': 'MM-YY'
              }
            },
            ticks:{ 
              maxTicksLimit: 12,
              autoSkip: true, 
            }
          }],
          yAxes: [
            {
              id: 'y-axis-1',
              scaleLabel: {
                display: true,
                labelString: '# Vehs'
              },
              type: 'linear',
              display: true,
              position: 'right',
              gridlines: { display: true },
              labels: { show: true },
              ticks: { min:0, beginAtZero: true },
              min: 0,
            },
          ],
          y:{ min: 0 }
        },      
        plugins:{
          zoom: {
            pan: {
              enabled: true,
              mode: 'x',
            },
          zoom: {
            enabled: true,
            drag: false,
            mode: 'x',
          }
    
          }
        },
      },        
    }
    document.getElementById("chartDiv").style.display = "block";
  };

  dataToTable(data: any[]) {
    console.log('espiras.js.dataToTable ############################################');
    console.log(data);
    var ttable: HTMLTableElement = <HTMLTableElement>document.getElementById("stats_table");
    ttable.rows[1].cells[1].innerHTML = data['id'];
    ttable.rows[2].cells[1].innerHTML = data['size'];
    ttable.rows[3].cells[1].innerHTML = data['aver'];
    ttable.rows[4].cells[1].innerHTML = data['max'];
    ttable.rows[5].cells[1].innerHTML = data['min'];
    ttable.rows[6].cells[1].innerHTML = data['delta_t'];
    ttable.rows[7].cells[1].innerHTML = data['ini_date'];
    ttable.rows[8].cells[1].innerHTML = data['end_date'];
  };
  //////////////////////////////////////////////////////////////////////////
  // Models
  //////////////////////////////////////////////////////////////////////////
  deleteModel() {
    let model_id: string = this.getSelectedModel();
    console.log('models.js.deleteModel::: model_id=' + model_id);
    this.deleteTrafficModelService.delete(model_id).subscribe(res => this.getModels0(), err => console.log(err));
  }

  getSelectedModel() {
    let sel = this.getSelectedRow();
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('models_table');
    let row = ttable.rows[sel];
    return row.cells[0].innerHTML;
  }

  getSelectedId() {
    let sel = this.getSelectedRow();
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('models_table');
    let row = ttable.rows[sel];
    return row.cells[1].innerHTML;
  }

  getSelectedIdFromTable() {
    let sel = this.getSelectedRow();
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('stats_table');
    let row = ttable.rows[1];
    return row.cells[1].innerHTML;
  }  

  getSelectedRow() {
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('models_table');
    for (let i = 0; i < ttable.rows.length; i++) {
      let row = ttable.rows[i];
      if (row.cells[0].style.backgroundColor !== 'white') {
        return i;
      }
    }
    return -1;
  }

  getSelectedNum() {
    let sel = this.getSelectedRow();
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById('models_table');
    let row = ttable.rows[sel];
    return row.cells[2].innerHTML;
  }

  openNewModelModal() {
    let ttable: HTMLTableElement = <HTMLTableElement>document.getElementById("stats_table");
    //document.getElementById('cityNM').innerHTML = ttable.rows[1].cells[1].innerHTML;
    document.getElementById('idNM').innerHTML = ttable.rows[1].cells[1].innerHTML;
    /*
    let ini: HTMLSelectElement = <HTMLSelectElement>document.getElementById('iniNM');
    let end: HTMLSelectElement = <HTMLSelectElement>document.getElementById('endNM');
    ini.value = ttable.rows[7].cells[1].innerHTML;
    end.value = ttable.rows[8].cells[1].innerHTML;*/
    document.getElementById('createModelModal').style.display = 'block';
  }

  createNewModel() {
    console.log('espiras.js.createNewModel()::: in function');
    let id = document.getElementById('idNM').innerHTML;
    /*let typeS: HTMLSelectElement = <HTMLSelectElement>document.getElementById('typeNM');
    let type = typeS.value;*/
    let numS: HTMLSelectElement = <HTMLSelectElement>document.getElementById('numNM');
    let num: number = +numS.value;
    /*
    let iniS: HTMLSelectElement = <HTMLSelectElement>document.getElementById('iniNM');
    let ini_date = iniS.value;
    let endS: HTMLSelectElement = <HTMLSelectElement>document.getElementById('endNM');
    let end_date = endS.value;
    let prctS: HTMLSelectElement = <HTMLSelectElement>document.getElementById('prctNM');
    let prct: number = +prctS.value;*/
    console.log('espiras.js.createNewModel():::\t      id=\t' + id);
    //console.log('espiras.js.createNewModel():::\t    type=\t' + type);
    console.log('espiras.js.createNewModel():::\t     num=\t' + num);
    /*
    console.log('espiras.js.createNewModel():::\tini_date=\t' + ini_date);
    console.log('espiras.js.createNewModel():::\tend_date=\t' + end_date);
    console.log('espiras.js.createNewModel():::\t    prct=\t' + prct);*/
    this.createNewmodelService.createModel(id, num).subscribe(res => this.closeNewModelModal(res), err => console.log(err));
  }

  closeNewModelModal(data: any) {
    console.log(data);
    document.getElementById('createModelModal').style.display = 'none';
  }
  ////////////////////////////////////////////////////////////////////////
  // calendar
  ////////////////////////////////////////////////////////////////////////
  monthToWord(val: number) {
    var lmonth = [];
    lmonth[0] = "January";
    lmonth[1] = "February";
    lmonth[2] = "March";
    lmonth[3] = "April";
    lmonth[4] = "May";
    lmonth[5] = "June";
    lmonth[6] = "July";
    lmonth[7] = "August";
    lmonth[8] = "September";
    lmonth[9] = "October";
    lmonth[10] = "November";
    lmonth[11] = "December";
    return lmonth[val];
  }

  populateYearCB(yyear: number) {
    var mItem: HTMLOptionElement = <HTMLOptionElement>document.createElement("option");
    let ySel: HTMLSelectElement = <HTMLSelectElement>document.getElementById("yCBI");
    mItem.text = "2016";
    mItem.value = 2016 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2017";
    mItem.value = 2017 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2018";
    mItem.value = 2018 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2019";
    mItem.value = 2019 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2020";
    mItem.value = 2020 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2021";
    mItem.value = 2021 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2022";
    mItem.value = 2022 + "";
    ySel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "2023";
    mItem.value = 2023 + "";
    ySel.options.add(mItem);
    var idx = 0;
    if (yyear === 2016) { idx = 0; }
    else if (yyear === 2017) { idx = 1; }
    else if (yyear === 2018) { idx = 2; }
    else if (yyear === 2019) { idx = 3; }
    else if (yyear === 2020) { idx = 4; }
    else if (yyear === 2021) { idx = 5; }
    else if (yyear === 2022) { idx = 6; }
    else if (yyear === 2023) { idx = 7; }

    ySel.selectedIndex = idx;
    /*
    ySel.addEventListener("change",function(e) {
        var yyIdx = ySel.selectedIndex;
        var yy = 2021;
        if      ( yyIdx === 0 ){ yy = 2016; }
        else if ( yyIdx === 1 ){ yy = 2017; }
        else if ( yyIdx === 2 ){ yy = 2018; }
        else if ( yyIdx === 3 ){ yy = 2019; }
        else if ( yyIdx === 4 ){ yy = 2020; }
        else if ( yyIdx === 5 ){ yy = 2021; }
        else if ( yyIdx === 6 ){ yy = 2022; }
        else if ( yyIdx === 7 ){ yy = 2023; }
        
        let mSel:HTMLSelectElement = <HTMLSelectElement> document.getElementById("mCBI");
        let mm = mSel.selectedIndex;
        nfecha:Date = new Date(yy,mm,1);
        //console.log("populateYearCB::: nfecha="+nfecha+" sfecha="+sfecha);
        this.createCalendar(nfecha);
    });*/
  }

  populateMonthCB(yyear: number, monthI: number) {
    var mItem: HTMLOptionElement = <HTMLOptionElement>document.createElement("option");
    let mSel: HTMLSelectElement = <HTMLSelectElement>document.getElementById("mCBI");
    mItem.text = "January";
    mItem.value = 0 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "February";
    mItem.value = 1 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "March";
    mItem.value = 2 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "April";
    mItem.value = 3 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "May";
    mItem.value = 4 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "June";
    mItem.value = 5 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "July";
    mItem.value = 6 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "August";
    mItem.value = 7 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "September";
    mItem.value = 8 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "October";
    mItem.value = 9 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "November";
    mItem.value = 10 + "";
    mSel.options.add(mItem);
    var mItem = document.createElement("option");
    mItem.text = "December";
    mItem.value = 11 + "";
    mSel.options.add(mItem);
    mSel.selectedIndex = monthI - 1;
    /*
    mSel.addEventListener("change",function(e) {
        var mm = mSel.selectedIndex;
        var nfecha = new Date(yyear,mm,1);
        console.log("populateYearCB::: nfecha="+nfecha+" sfecha="+sfecha);
        createCalendar(nfecha);
    }); */
  }

  findMaxDaysInMonth(ofecha: Date) {
    var temp: Date = new Date(ofecha);
    var nowMonth = ofecha.getMonth();
    var lastDay = temp.getDate();
    for (var j = 1; j < 32; j++) {
      temp.setDate(temp.getDate() + 1);
      var lMonth = temp.getMonth();
      if (lMonth !== nowMonth) { break; }
      lastDay = temp.getDate();
    }
    return lastDay + 1;
  }

  findNumOfEmptyDays(ofecha: Date) {
    var temp = new Date(ofecha);
    temp.setDate(1);
    var out = temp.getDay() - 1;
    if (temp.getDay() === 0) out = 6;
    return out;
  }

  createCalendar(local_fecha: Date) {
    this.allowedDay = true;
    console.log('calendar.js.createCalendar::: local_fecha=' + local_fecha);
    this.sfecha = local_fecha;
    console.log('calendar.js.createCalendar::: calendarRestriction='+this.calendarRestriction);
    document.getElementById("calendar").innerHTML = '';
    var btn: HTMLDivElement = <HTMLDivElement>document.createElement("div");
    btn.style.boxSizing = "border-box";
    btn.style.fontFamily = "Verdana, sans-serinf";
    //btn.className        = "month";
    //btn.style.position   = 'absolute';
    btn.style.width = "100%";
    btn.style.background = '#cfb391';
    btn.style.border = '0px';
    btn.style.boxSizing = 'border-box';
    btn.style.marginTop = '-16px';
    var monthI       = local_fecha.getMonth() + 1;
    var maxDaysMonth = this.findMaxDaysInMonth(local_fecha);
    var edays        = this.findNumOfEmptyDays(local_fecha);
    var temp: Date   = new Date(local_fecha);
    this.lDate       = new Date(temp.setMonth(temp.getMonth() - 1));
    temp             = new Date(local_fecha);
    this.rDate       = new Date(temp.setMonth(temp.getMonth() + 1));
    var year         = local_fecha.getFullYear();
    btn.innerHTML    = "<ul id=\"parentUL\" style=\"height:100%;box-sizing:border-box;display:block;text-align:center\"><li id=\"prev\" style=\"padding-top:3px;list-style:none;float:left;\" class=\"prev\">&#10094;</li><li id=\"next\" style=\"list-style:none;float:right;padding-top:3px\" class=\"next\">&#10095;</li>";
    btn.innerHTML    = btn.innerHTML + "<li style=\"text-align:center;display:block\"><select style=\"margin-left:-40px;\" id=\"mCBI\"></select><br><span style=\"font-size:18;px\"><select id=\"yCBI\"></select></span> </li></ul>";
    // week
    var weekBtn: HTMLUListElement = <HTMLUListElement>document.createElement("ul");
    weekBtn.className = "weekdays";
    weekBtn.style.color = 'rgb(102,102,102)';
    weekBtn.style.backgroundColor = '#ddd';
    weekBtn.style.margin = '0px';
    weekBtn.style.padding = '0px';
    weekBtn.style.fontSize = '13px';
    weekBtn.style.listStyle = 'disc';
    weekBtn.style.verticalAlign = 'baseline';
    weekBtn.style.boxSizing = 'border-box';
    weekBtn.style.paddingInlineStart = '40px';
    weekBtn.style.paddingInlineEnd = '40px';
    weekBtn.style.paddingLeft = '58px';
    weekBtn.innerHTML = "  <li style=\"display:inline-block;text-transform:uppercase;color:#666;width: 14.1%;\">Mo</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Tu</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.6%;\">We</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Th</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Fr</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Sa</li><li style=\"display:inline-block;text-transform:uppercase;color:#666;width:14.1%;\">Su</li>";
    // weekBtn.style.display = "inline-block";
    weekBtn.style.color = "#666";
    btn.appendChild(weekBtn);
    // days
    var daysBtn: HTMLUListElement = <HTMLUListElement>document.createElement("ul");
    daysBtn.className = "days";
    daysBtn.style.padding = "2px";
    daysBtn.style.display = 'block';
    daysBtn.style.verticalAlign = 'baseline';
    daysBtn.style.fontSize = '13px';
    daysBtn.style.listStyleType = "none";
    daysBtn.style.color = "#58595b";
    daysBtn.style.paddingInlineStart = '40px';
    daysBtn.innerHTML = "";
    daysBtn.id = "daysUL";
    daysBtn.style.width = '100%'
    var llimit = 32;
    for (var i = 0; i < edays; i++) {
      var dayBtn = document.createElement("li");
      dayBtn.style.width = '12.6%';
      dayBtn.style.listStyleType = "none";
      dayBtn.style.color = "#777";
      dayBtn.style.marginBottom = '5px';
      dayBtn.style.fontSize = '100%';
      dayBtn.style.margin = '0';
      dayBtn.style.outline = '0';
      dayBtn.style.boxSizing = 'border-box';
      dayBtn.style.lineHeight = '1.35';
      dayBtn.style.listStyleType = "none";
      dayBtn.style.display = "inline-block";
      dayBtn.style.textAlign = "center";
      dayBtn.style.listStyle = 'disc';
      dayBtn.style.verticalAlign = 'baseline';
      dayBtn.id = -i + "";
      daysBtn.append(dayBtn);
    }
    for (var i = 1; i < maxDaysMonth; i++) {
      var dayBtn = document.createElement("li");
      dayBtn.style.width = '12.6%'
      dayBtn.style.listStyleType = "none";
      dayBtn.style.color = "#777";
      dayBtn.style.marginBottom = '5px';
      dayBtn.style.fontSize = '100%';
      dayBtn.style.margin = '0';
      dayBtn.style.outline = '0';
      dayBtn.style.boxSizing = 'border-box';
      dayBtn.style.lineHeight = '1.35';
      dayBtn.style.listStyleType = "none";
      dayBtn.style.display = "inline-block";
      dayBtn.style.textAlign = "center";
      dayBtn.style.listStyle = 'disc';
      dayBtn.style.verticalAlign = 'baseline';
      dayBtn.id = i + "";
      var mI = monthI + "";
      if (monthI < 10) mI = '0' + monthI;
      var iI = i + '';
      if (i < 10) iI = '0' + i;
      var dd = year + "-" + mI + "-" + iI;

      if (this.calendarInfo !== null) {
        if (dd in this.calendarInfo) {

          if (this.calendarInfo[dd]["lect"] === 0) {
            dayBtn.style.backgroundColor = "#0f0";
          }
          if (this.calendarInfo[dd]["fest"] === 1) {
            dayBtn.style.backgroundColor = "#f00";
          }
          else if (this.calendarInfo[dd]["fest"] === 2) {
            dayBtn.style.backgroundColor = "#f0f";
          }
          else if (this.calendarInfo[dd]["fest"] === 3) {
            dayBtn.style.backgroundColor = "#ff0";
          }
        } else {

        }
      }
      if ( this.sfeature_combo > 2 ){
        if ( this.calendarRestriction !== null ){
          if ( this.calendarRestriction.includes(dd) ){
            dayBtn.style.background = "rgb(238,238,238)";
          }else{
            daysBtn.style.background = "rgb(70,70,70)";
          }
        }
      }else{
        dayBtn.style.background = "rgb(238,238,238)";
      }
      if (i === this.sfecha.getDate() && (monthI - 1) === this.sfecha.getMonth() && year === this.sfecha.getFullYear()) {
        let sel_color = 'red';
        console.log('dd='+dd);
        if ( this.sfeature_combo > 2 ){
          if ( this.calendarRestriction !== null ){
            if ( !this.calendarRestriction.includes(dd) ){
              sel_color = 'black';
              this.allowedDay = false;
            }
          }
        }
        dayBtn.style.padding = '0px';
        dayBtn.style.border = 'solid';
        dayBtn.style.borderWidth = '2px';
        dayBtn.style.borderColor = sel_color;
        dayBtn.style.color = 'blue !important';
        dayBtn.className = "active";
        var sp: HTMLSpanElement = <HTMLSpanElement>document.createElement("span");
        sp.className = "active";
        sp.innerHTML = i + "";
        dayBtn.appendChild(sp);

      } else {
        dayBtn.innerHTML = i + "";
      }
      daysBtn.append(dayBtn);
    }
    btn.appendChild(daysBtn);
    document.getElementById("calendar").appendChild(btn);
    document.getElementById("parentUL")['component'] = this;
    document.getElementById("parentUL").addEventListener("click", function (e) {
      let targ: HTMLLIElement = <HTMLLIElement>e.target;
      if (targ && targ.nodeName === "LI") {
        if (targ.id === "prev") {
          this['component'].createCalendar(this['component'].lDate);
        }
        else if (targ.id === "next") {
          this['component'].createCalendar(this['component'].rDate);
        }
      }
    });

    document.getElementById("daysUL")['component'] = this;
    document.getElementById("daysUL").addEventListener("click", function (e) {
      let targ: HTMLLIElement = <HTMLLIElement>e.target;
      if (targ && targ.nodeName === "LI") {
        if (+targ.id > 0) {
          this['component'].sfecha = new Date(local_fecha.setDate(+targ.id));
          //changeChartZoomAndDate();
          this['component'].createCalendar(local_fecha);
        }
      }
    });
    this.populateMonthCB(year, monthI);
    this.populateYearCB(year);
    console.log('traffic-prediction.component.ts.createCalendar::: this.sfecha='+this.sfecha+" feature_combo="+this.sfeature_combo+' this.allowedFecha='+this.allowedDay);
  }

  lformatDate(local_date: Date) {
    //var d = new Date(date),
    let month: string = '' + (local_date.getMonth() + 1);
    let day: string = '' + local_date.getDate();
    let year: string = '' + local_date.getFullYear();

    if (month.length < 2)
      month = '0' + month;
    if (day.length < 2)
      day = '0' + day;
    let out = [year, month, day].join('-');
    return out;
  }

  visualize(){
    let lid: string = this.getSelectedIdFromTable();
    console.log("visualize.id="+lid);
    this.getEspiraData(lid);
  }
  ////////////////////////////////////////////////////////////////////////
  // prediction
  ////////////////////////////////////////////////////////////////////////
  predict(period: number) {
    console.log('traffic-prediction.component.ts.predict::: this.allowedDay='+this.allowedDay+" period="+period+ " dateS="+this.sfecha);
    if ( !this.allowedDay ){
      document.getElementById('wrongDay').style.display = 'block';
    }else{
      if ( period == 2 &&  ( this.lformatDate(this.sfecha) != this.calendarRestriction[0] ) ){
        this.sfecha = new Date(this.calendarRestriction[0]);
        this.createCalendar(this.sfecha);
        document.getElementById('correctDay').style.display = 'block';
      }
      
      let ddate = this.lformatDate(this.sfecha);
      let model_id = this.getSelectedModel();
      let id: string = this.getSelectedId();
      let city = this.city;
      let num: number = +this.getSelectedNum();
      if (period < 3) {
        this.getPredictionService.getValues(model_id, ddate.toString(), period, id, num).subscribe(res => this.printPrediction(res['result'],id,num), err => console.log(err));
      }
      else if (period === 3) {
        let iniS: HTMLSelectElement = <HTMLSelectElement>document.getElementById("iniDateCP");
        let ini = iniS.value;
        let endS: HTMLSelectElement = <HTMLSelectElement>document.getElementById("endDateCP");
        let end = endS.value;
        this.getPredictionService.getValuesCustom(model_id, ini, end, id, num).subscribe(res => this.printPrediction(res['result'],id,num), err => console.log(err));
      }
    }
  }

  closeWrongDay(){
    document.getElementById('wrongDay').style.display = 'none';
  }

  closeCorrectDay(){
    document.getElementById('correctDay').style.display = 'none';
  }

  printPrediction(data: any,id:string,num:number) {
    this.toDownload = data;
    this.toDownloadName = "predictionLoop_"+id+"_"+num;
    document.getElementById('chartDiv').style.display = 'block';
    let max_x = data['preds'][data['preds'].length - 1][0];
    let min_x = data['preds'][0][0];
    let xs = [];
    let ys = [];
    let yups = [];
    let ydowns = [];
    let preds = [];
    let temps = [];
    let ts = [];
    let precs = [];
    let ps = [];
    let max_y = 0;
    for (let k = 0; k < data['preds'].length; k++) {
      xs.push(data['preds'][k][0]);
      preds.push({
        "name":  data['preds'][k][0],
        "value": data['preds'][k][1],
        "min":   data['limits'][k][1],
        "max":   data['limits'][k][2]
      });

      ys.push({ "x": data['preds'][k][0], "y": data['preds'][k][1] });
      yups.push({ "x": data['limits'][k][0], "y": data['limits'][k][1] });
      ydowns.push({ "x": data['limits'][k][0], "y": data['limits'][k][2] });
      max_y = Math.max(max_y, data['preds'][k][1]);
    }
    max_y = 1.2 * max_y;
    for (let k = 0; k < this.temperaturaJA.length; k++) {
      if (this.temperaturaJA[k][0] > min_x && this.temperaturaJA[k][0] < max_x) {
        temps.push({ "name": this.temperaturaJA[k][0], "value": this.temperaturaJA[k][1] });
        ts.push({ "x": this.temperaturaJA[k][0], "y": this.temperaturaJA[k][1] });
      }
    }
    let avg_int = 0;
    for (let k = 0; k < this.intensidadJA.length; k++) {
      if (this.intensidadJA[k][0] > min_x && this.intensidadJA[k][0] < max_x) {
        precs.push({ "name": this.intensidadJA[k][0], "value": this.intensidadJA[k][1] });
        ps.push({ "x": this.intensidadJA[k][0], "y": this.intensidadJA[k][1] });
        avg_int = avg_int + this.intensidadJA[k][1];
      }
    }
    avg_int = avg_int / (precs.length * 1.0);
    
    let chart_data:any = {
      labels: xs,
      datasets: [
        {
          label: 'Pred',
          backgroundColor: 'rgb(255, 165, 0,0.1)',
          type: 'line',
          fill: false,
          borderColor: 'rgb(255, 0, 0,1)',
          borderWidth: 1,
          pointRadius: 0,
          yAxisID: 'y-axis-1',
          data: ys
        },
        {
          label: "BandTop",
          type: "line",
          backgroundColor: "rgb(255, 165, 0,0.3)",
          borderColor: "transparent",
          pointRadius: 0,
          legend: { display: false },
          fill: 0,
          tension: 0,
          yAxisID: 'y-axis-1',
          data: yups
        },
        {
          label: "BandBottom",
          type: "line",
          backgroundColor: "rgb(255, 165, 0,0.3)",
          borderColor: "transparent",
          pointRadius: 0,
          legend: { display: false },
          fill: 0,
          tension: 0,
          yAxisID: 'y-axis-1',
          data: ydowns
        }]
    };
    
    this.chart_config = {
      data: chart_data, 
      options: {
        responsive: true,
        maintainAspectRatio: false,
        legend: {
          labels: {
            filter: function (legendItem, chartData) {
              if (legendItem.datasetIndex == 1 || legendItem.datasetIndex == 2) {
                return false;
              } else {
                return true;
              }
            }
          }
        },
        scales: {
          xAxes: [{
            type: 'time',
            barPercentage: 1.0,
            time: {
              displayFormats: {
                'hour': 'HH:mm DD-MM',
                'day': 'HH:mm DD-MM'
              }
            },
            ticks:{ 
              maxTicksLimit: 12,
              autoSkip: true, 
            }
          }],          
          yAxes: [
            {
              id: 'y-axis-1',
              scaleLabel: {
                display: true,
                labelString: '# Vehs'
              },
              type: 'linear',
              display: true,
              position: 'right',
              gridlines: { display: true },
              labels: { show: true },
              ticks: { min:0, beginAtZero: true },
              min: 0,
            }
          ],
          y:{ min:0 }
        },
        plugins:{
          zoom: {
            pan: {
              enabled: true,
              mode: 'x',
            },
            zoom: {
              enabled: true,
              drag: false,
              mode: 'x',
            }
          }
        },
      }
    };
  }

  closeCreateModelModal() {
    document.getElementById('createModelModal').style.display = 'none';
  }

  closeCustomPredictModal() {
    document.getElementById('customPredictModal').style.display = 'none';
  }

  openPredictCustomModal() {
    document.getElementById('customPredictModal').style.display = 'block';
  }

  download(){
    const blob = new Blob([JSON.stringify(this.toDownload)], { type: "text/json" });
    const link = document.createElement("a");
    link.download = this.toDownloadName+".json";
    link.href = window.URL.createObjectURL(blob);
    link.dataset.downloadurl = ["text/json", link.download, link.href].join(":");
    const evt = new MouseEvent("click", {
        view: window,
        bubbles: true,
        cancelable: true,
    });
    link.dispatchEvent(evt);
    link.remove();
  }
  
}

export interface ILine{
  model_id: string;
  id: string;
  feature_combo: number;
  status: number;
  score: string; 
}

export class Sort{
  private sortOrder = 1;
  private collator = new Intl.Collator(undefined, { numeric: true, sensitivity: "base"});
  constructor(){}
  public startSort(property, order, type = ""){
    if (order == "desc"){ this.sortOrder = -1; }
      return (a,b) => { 
      if (type==="date"){ return this.collator.compare(a[property], b[property])* this.sortOrder;
      }
    }
  }
  private sortData(a, b){
    if (a<b){ return -1*this.sortOrder;}
    else if (a>b){ return 1*this.sortOrder;}
    else{ return 0*this.sortOrder;}
  }
}